// -------------------------------------------------------
// AODtoLiverpoolNtuple
//
// Purpose: Making an Ntuple of quantities from the AOD.
//
// Author: Barry King
// Modifications by: Joost Vossebeld, Carl Gwilliam, Mike Flowerdew
//
// This is for version 12 of athena and has been tested in 12.0.31 
// and 12.0.4.
// -------------------------------------------------------

#include "AODtoLiverpoolNtuple/AODtoLiverpoolNtuple.h"

#include "GaudiKernel/MsgStream.h"
#include "GaudiKernel/AlgFactory.h"
#include "GaudiKernel/IToolSvc.h"

#include "StoreGate/StoreGateSvc.h"
#include "StoreGate/DataHandle.h"

#include "GaudiKernel/INTupleSvc.h"
#include "GaudiKernel/NTuple.h"
#include "GaudiKernel/SmartDataPtr.h" /* NTupleFilePtr */
#include "GaudiKernel/ListItem.h"

#include "AIDA/IHistogram1D.h"
#include "AIDA/IHistogram2D.h"

// ElectronCollection
#include "ElectronPhotonIDEvent/ElectronContainer.h"
#include "egammaEvent/egammaPIDdefs.h"

// PhotonCollection
#include "ElectronPhotonIDEvent/PhotonContainer.h"

// MuonCollection
#include "MuonIDEvent/MuonContainer.h"
//#include "MuonIDEvent/Muon.h"
#include "muonEvent/CaloEnergy.h"

// TauJets
#include "TauIDEvent/TauJetContainer.h"

// TruthParticle
#include "McParticleEvent/TruthParticleContainer.h"

// Jets
#include "JetTagEvent/ParticleJetContainer.h"
#include "JetTagEvent/TrackConstituents.h"

// Tag
#include "JetTagInfo/ITagInfo.h"
#include "JetTagInfo/IPInfoBase.h"
#include "JetTagInfo/IPInfoPlus.h"
#include "JetTagInfo/IPTrackInfo.h"
#include "JetTagInfo/SVInfoPlus.h"
#include "JetTagInfo/SVInfoBase.h"
#include "JetTagInfo/SecVtxInfo.h"
#include "JetTagInfo/LifetimeInfo.h"
#include "JetTagInfo/TruthInfo.h"
#include "JetTagInfo/SoftMuonInfo.h"
#include "JetTagInfo/SMTrackInfo.h"
#include "JetTagInfo/SoftElectronInfo.h"
#include "JetTagInfo/SETrackInfo.h"
#include "JetTagInfo/JetProbInfoBase.h"

// fixed header paths above this line
#include "TruthHelper/IsGenerator.h"

// CaloClusterCollection
#include "CaloEvent/CaloClusterContainer.h"
#include "CaloEvent/CaloCluster.h"
#include "CaloEvent/CaloClusterMoment.h"

// TriggerDecision
#include "TrigSteeringEvent/TriggerDecision.h"
 
// L1EMTauObjectCollection
#include "AnalysisTriggerEvent/CTP_Decision.h"
#include "AnalysisTriggerEvent/L1JetObjectContainer.h"
#include "AnalysisTriggerEvent/L1EtmissObject.h"
#include "AnalysisTriggerEvent/L1EMTauObjectContainer.h"
//#include "AnalysisTriggerEvent/L1EMTauObject.h"

// HLT electron/egamma
#include "TrigParticle/TrigElectron.h"
#include "TrigParticle/TrigElectronContainer.h"
#include "egammaUtils/egammaqpoint.h"
#include "egammaUtils/egammaqgcld.h"
#include "egammaEvent/egamma.h"
#include "egammaEvent/EMShower.h"
#include "egammaEvent/EMTrackMatch.h"
#include "egammaEvent/EMConvert.h"
#include "egammaEvent/EMBremFit.h"
#include "egammaEvent/egammaContainer.h"

// TrackParticle
#include "Particle/TrackParticleContainer.h"
#include "Particle/TrackParticle.h"

// Primary Vtx
#include "VxVertex/VxContainer.h"
#include "VxVertex/VxCandidate.h"
#include "VxVertex/RecVertex.h"

// EtMiss
#include "MissingETEvent/MissingET.h"
#include "MissingETEvent/MissingEtRegions.h"

// EtMiss Truth
#include "MissingETEvent/MissingEtTruth.h"  

// Event
#include "EventInfo/EventInfo.h"
#include "EventInfo/EventID.h"

// Others
#include "ParticleEvent/ParticleBaseContainer.h"
//#include "EventKernel/IParticleContainer.h"
#include "CompositeParticleEvent/CompositeParticle.h"
#include "AnalysisTools/IAnalysisTools.h"
#include "AnalysisUtils/AnalysisCombination.h"
#include "CLHEP/Vector/LorentzVector.h"

// C/C++
#include <algorithm>
#include <math.h>
#include <functional>

using namespace std;

AODtoLiverpoolNtuple::AODtoLiverpoolNtuple(const std::string& name, ISvcLocator* pSvcLocator) : 
  Algorithm(name, pSvcLocator), m_analysisTools(0), nt(ntupleSvc(), "")
{
  // DataType
  declareProperty("FastSimulation", m_fastsim = 0);
  declareProperty("Release", m_release = 4);
  declareProperty("DoElectronTrigger", m_doElectronTrigger = 0);
}

AODtoLiverpoolNtuple::~AODtoLiverpoolNtuple() {}

StatusCode AODtoLiverpoolNtuple::initialize() 
{

  MsgStream mLog( messageService(), name() );
  mLog << MSG::INFO << "Initializing AODtoLiverpoolNtuple" << endreq;

  // Store gate
  StatusCode sc = service("StoreGateSvc", m_storeGate);
  if (sc.isFailure()) {
    mLog << MSG::ERROR << "Unable to retrieve pointer to StoreGateSvc" << endreq;
    return sc;
  }
  
  // Analysis Tools
  IToolSvc* toolSvc;
  sc = service("ToolSvc",toolSvc);
  if (StatusCode::SUCCESS != sc) {
    mLog << MSG::ERROR << " Cannot get ToolSvc" << endreq;
    return StatusCode::FAILURE;
  }

  if ( m_doElectronTrigger )
    {
      //// egamma pointer tool
      IAlgTool* algTool;
      std::string egammaqpointTool_name="egammaqpoint/egammaqpoint";
      ListItem egammaqpointTool(egammaqpointTool_name);
      sc = toolSvc->retrieveTool(egammaqpointTool.type(), egammaqpointTool.name(), algTool);
      if (sc.isFailure())
	{
	  mLog << MSG::FATAL
	       << "Unable to find tool for " << egammaqpointTool_name
	       << endreq;
	  return sc;
	} else {
	  m_egammaqpoint=dynamic_cast<egammaqpoint*>(algTool);
	}
      //// egamma shower depth tool
      std::string egammaqgcldTool_name="egammaqgcld/egammaqgcld";
      ListItem egammaqgcldTool(egammaqgcldTool_name);
      sc = toolSvc->retrieveTool(egammaqgcldTool.type(), egammaqgcldTool.name(), algTool);
      if (sc.isFailure()) 
	{
	  mLog << MSG::FATAL
	       << "Unable to find tool for " << egammaqgcldTool_name
	       << endreq; 
	  return sc;
	} else {
	  m_egammaqgcld=dynamic_cast<egammaqgcld*>(algTool);  
	}
    }

  IAlgTool *tmp_analysisTools;
  sc = toolSvc->retrieveTool("AnalysisTools",tmp_analysisTools);
  if (StatusCode::SUCCESS != sc) {
    mLog << MSG::ERROR << "Cannot get handle on analysis tools" << endreq;
    return StatusCode::FAILURE;
  }
  m_analysisTools = dynamic_cast<IAnalysisTools *>(tmp_analysisTools);

  StatusCode status = StatusCode::SUCCESS;

  // Initalise ntuple
  NTupleFilePtr file1(ntupleSvc(), "/NTUPLES/FILE1");
  NTuplePtr nt(ntupleSvc(), "/NTUPLES/FILE1/Ntuple1");

  if (!nt) { // Book ntuple if not already done
    nt = ntupleSvc()->book ("/NTUPLES/FILE1/Ntuple1", CLID_ColumnWiseTuple, "Some Description");
    if (nt) { 
      mLog << MSG::DEBUG << "booked ntuple " << endreq;
       
      status = nt->addItem ("version_h", m_h_version_h, 0.0,  100.0);
      status = nt->addItem ("version_c", m_h_version_c, 0.0,  100.0);

      status = nt->addItem ("event_eventnum",  m_h_event_eventnum, 0, 1000000);
      status = nt->addItem ("event_runnum",  m_h_event_runnum, 0, 1000000);

      status = BookElectron(nt);       
      status = BookPhoton(nt);
      status = BookMuidMuons(nt);
      status = BookStacoMuons(nt);
      status = BookTriggerDecision(nt);
      if ( m_doElectronTrigger )
	{
	  status = BookEFegamma(nt);
	  status = BookTrigElectron(nt);
	}
      status = BookL1EMTauObject(nt);
      status = BookL1JetObject(nt);
      status = BookL1EtmissObject(nt);
      // is this used by anyone?
      //status = BookCTP_Decision(nt);
      status = BookEtMiss(nt);
      status = BookEtMissRefFinal(nt);//ED
      status = BookEtMissLocHadTopoObj(nt);//ED
      status = BookEtMissObjFinal(nt);//ED
      status = BookTauJet(nt);
      // container not available
      status = BookTauJet1p3p(nt);
      status = BookTrack(nt);
      status = BookJet(nt);
      status = BookTruthParticle(nt);
      status = BookTruthJet(nt);
      status = BookTruthEtMiss(nt);
      status = BookPrimaryVertex(nt);
      mLog << MSG::DEBUG << " Ntuple booked. " << endreq;
       
    } else {
      mLog << MSG::DEBUG << " Could not book Ntuple. " << endreq;
    }
  }
  return StatusCode::SUCCESS;
}	

StatusCode AODtoLiverpoolNtuple::BookElectron(NTuplePtr nt)
{
  MsgStream mLog( messageService(), name() );
  StatusCode status = StatusCode::SUCCESS;

  mLog << MSG::DEBUG << " Booking ElectronContainer variables. " << endreq;
  // Electron
  
  status = nt->addItem ("electron_num", m_h_electron_num, 0,  max_electron_num);
  status = nt->addItem ("electron_pt",  m_h_electron_num, m_h_electron_pt);
  status = nt->addItem ("electron_eta", m_h_electron_num, m_h_electron_eta);
  status = nt->addItem ("electron_phi", m_h_electron_num, m_h_electron_phi);
  status = nt->addItem ("electron_e",   m_h_electron_num, m_h_electron_e);
  status = nt->addItem ("electron_hasTrack",  m_h_electron_num, m_h_electron_hasTrack);
  status = nt->addItem ("electron_track_pt",  m_h_electron_num, m_h_electron_track_pt);
  status = nt->addItem ("electron_track_eta",  m_h_electron_num, m_h_electron_track_eta);
  status = nt->addItem ("electron_track_phi",  m_h_electron_num, m_h_electron_track_phi);
  status = nt->addItem ("electron_track_e",  m_h_electron_num, m_h_electron_track_e);
  status = nt->addItem ("electron_track_d0",  m_h_electron_num, m_h_electron_track_d0);
  status = nt->addItem ("electron_track_z0",  m_h_electron_num, m_h_electron_track_z0);
  status = nt->addItem ("electron_track_charge",  m_h_electron_num, m_h_electron_track_charge);
  status = nt->addItem ("electron_numberOfBLayerHits",  m_h_electron_num, m_h_electron_numberOfBLayerHits);
  status = nt->addItem ("electron_numberOfPixelHits",  m_h_electron_num, m_h_electron_numberOfPixelHits);
  status = nt->addItem ("electron_numberOfSCTHits",  m_h_electron_num, m_h_electron_numberOfSCTHits);
  status = nt->addItem ("electron_numberOfTRTHits",  m_h_electron_num, m_h_electron_numberOfTRTHits);
  status = nt->addItem ("electron_numberOfTRTHighThresholdHits",  m_h_electron_num, m_h_electron_numberOfTRTHighThresholdHits);
  status = nt->addItem ("electron_egamma",  m_h_electron_num, m_h_electron_egamma);
  status = nt->addItem ("electron_softe",  m_h_electron_num, m_h_electron_softe);
  status = nt->addItem ("electron_IsEM",  m_h_electron_num, m_h_electron_IsEM);
  status = nt->addItem ("electron_ElectronWeight",  m_h_electron_num, m_h_electron_ElectronWeight);
  status = nt->addItem ("electron_BgWeight",  m_h_electron_num, m_h_electron_BgWeight);
  status = nt->addItem ("electron_NeuralNet",  m_h_electron_num, m_h_electron_NeuralNet);
  status = nt->addItem ("electron_Hmatrix",  m_h_electron_num, m_h_electron_Hmatrix);
  status = nt->addItem ("electron_SofteIsEM",  m_h_electron_num, m_h_electron_SofteIsEM);
  status = nt->addItem ("electron_SofteElectronWeight",  m_h_electron_num, m_h_electron_SofteElectronWeight);
  status = nt->addItem ("electron_SofteBgWeight",  m_h_electron_num, m_h_electron_SofteBgWeight);
  status = nt->addItem ("electron_SofteNeuralNet",  m_h_electron_num, m_h_electron_SofteNeuralNet);
  status = nt->addItem ("electron_EoverP",  m_h_electron_num, m_h_electron_EoverP);
  status = nt->addItem ("electron_etcone",  m_h_electron_num, m_h_electron_etcone);
  status = nt->addItem ("electron_etcone20",  m_h_electron_num, m_h_electron_etcone20);
  status = nt->addItem ("electron_etcone30",  m_h_electron_num, m_h_electron_etcone30);
  status = nt->addItem ("electron_etcone40",  m_h_electron_num, m_h_electron_etcone40);
  // for egamma electrons 
  status = nt->addItem ("electron_etaBE2",m_h_electron_num, m_h_electron_etaBE2);
  status = nt->addItem ("electron_et37",m_h_electron_num, m_h_electron_et37);
  status = nt->addItem ("electron_e233",m_h_electron_num, m_h_electron_e233);
  status = nt->addItem ("electron_e237",m_h_electron_num, m_h_electron_e237);
  status = nt->addItem ("electron_e277",m_h_electron_num, m_h_electron_e277);
  status = nt->addItem ("electron_ethad1",m_h_electron_num, m_h_electron_ethad1);
  status = nt->addItem ("electron_weta1",m_h_electron_num, m_h_electron_weta1);
  status = nt->addItem ("electron_weta2",m_h_electron_num, m_h_electron_weta2);
  status = nt->addItem ("electron_f1",m_h_electron_num, m_h_electron_f1);
  status = nt->addItem ("electron_e2tsts1",m_h_electron_num, m_h_electron_e2tsts1);
  status = nt->addItem ("electron_emins1",m_h_electron_num, m_h_electron_emins1);
  status = nt->addItem ("electron_wtots1",m_h_electron_num, m_h_electron_wtots1);
  status = nt->addItem ("electron_fracs1",m_h_electron_num, m_h_electron_fracs1);
  // for softe electrons
  status = nt->addItem ("electron_EtaCorrMag",m_h_electron_num, m_h_electron_EtaCorrMag);
  status = nt->addItem ("electron_f1core",m_h_electron_num, m_h_electron_f1core);
  status = nt->addItem ("electron_f3core",m_h_electron_num, m_h_electron_f3core);
  status = nt->addItem ("electron_pos7",m_h_electron_num, m_h_electron_pos7);
  status = nt->addItem ("electron_iso",m_h_electron_num, m_h_electron_iso);
  status = nt->addItem ("electron_widths2",m_h_electron_num, m_h_electron_widths2);

  // LArEM cluster 
  mLog << MSG::DEBUG << " Booking LarEM variables. " << endreq;
  status = nt->addItem ("LArClusNum", m_h_larn, 0,  max_lar_num);
  status = nt->addItem ("LArClusNsamples",  m_h_larn, m_h_larnsamples);

  status = nt->addItem ("LArClusEM_Time",  m_h_larn, m_h_lar_time);
  status = nt->addItem ("LArClusEMB0E",  m_h_larn, m_h_larEMB0E);
  status = nt->addItem ("LArClusEMB1E",  m_h_larn, m_h_larEMB1E);
  status = nt->addItem ("LArClusEMB2E",  m_h_larn, m_h_larEMB2E);
  status = nt->addItem ("LArClusEMB3E",  m_h_larn, m_h_larEMB3E);
  status = nt->addItem ("LArClusEME0E",  m_h_larn, m_h_larEME0E);
  status = nt->addItem ("LArClusEME1E",  m_h_larn, m_h_larEME1E);
  status = nt->addItem ("LArClusEME2E",  m_h_larn, m_h_larEME2E);
  status = nt->addItem ("LArClusEME3E",  m_h_larn, m_h_larEME3E);
  status = nt->addItem ("LArClusEMB0Eta",  m_h_larn, m_h_larEMB0Eta);
  status = nt->addItem ("LArClusEMB1Eta",  m_h_larn, m_h_larEMB1Eta);
  status = nt->addItem ("LArClusEMB2Eta",  m_h_larn, m_h_larEMB2Eta);
  status = nt->addItem ("LArClusEMB3Eta",  m_h_larn, m_h_larEMB3Eta);
  status = nt->addItem ("LArClusEME0Eta",  m_h_larn, m_h_larEME0Eta);
  status = nt->addItem ("LArClusEME1Eta",  m_h_larn, m_h_larEME1Eta);
  status = nt->addItem ("LArClusEME2Eta",  m_h_larn, m_h_larEME2Eta);
  status = nt->addItem ("LArClusEME3Eta",  m_h_larn, m_h_larEME3Eta);
  status = nt->addItem ("LArClusEMB0Phi",  m_h_larn, m_h_larEMB0Phi);
  status = nt->addItem ("LArClusEMB1Phi",  m_h_larn, m_h_larEMB1Phi);
  status = nt->addItem ("LArClusEMB2Phi",  m_h_larn, m_h_larEMB2Phi);
  status = nt->addItem ("LArClusEMB3Phi",  m_h_larn, m_h_larEMB3Phi);
  status = nt->addItem ("LArClusEME0Phi",  m_h_larn, m_h_larEME0Phi);
  status = nt->addItem ("LArClusEME1Phi",  m_h_larn, m_h_larEME1Phi);
  status = nt->addItem ("LArClusEME2Phi",  m_h_larn, m_h_larEME2Phi);
  status = nt->addItem ("LArClusEME3Phi",  m_h_larn, m_h_larEME3Phi);

  status = nt->addItem ("LArCluseta",  m_h_larn, m_h_lareta);
  status = nt->addItem ("LArClusphi",  m_h_larn, m_h_larphi);
  status = nt->addItem ("LArClusenergy",  m_h_larn, m_h_larenergy);
  status = nt->addItem ("LArClusinBarrel",  m_h_larn, m_h_larinBarrel);
  status = nt->addItem ("LArClusinEndcap",  m_h_larn, m_h_larinEndcap);

  // EMTopo cluster
  mLog << MSG::DEBUG << " Booking EMTopo variables. " << endreq;
  status = nt->addItem ("EMTopoClusNum", m_h_emtopon, 0,  max_emtopo_num);
  status = nt->addItem ("EMTopoClusNsamples",  m_h_emtopon, m_h_emtoponsamples);
  
  status = nt->addItem ("EMTopoClus_Time",  m_h_emtopon, m_h_emtopo_time);
  status = nt->addItem ("EMTopoCluseta",  m_h_emtopon, m_h_emtopoeta);
  status = nt->addItem ("EMTopoClusphi",  m_h_emtopon, m_h_emtopophi);
  status = nt->addItem ("EMTopoClusenergy",  m_h_emtopon, m_h_emtopoenergy);
  status = nt->addItem ("EMTopoClusinBarrel",  m_h_emtopon, m_h_emtopoinBarrel);
  status = nt->addItem ("EMTopoClusinEndcap",  m_h_emtopon, m_h_emtopoinEndcap); 

  // available for 420 and 630 (em and calocaltopo)
  status = nt->addItem ("EMTopo_first_phi",  m_h_emtopon,  m_h_emtopo_first_phi);
  status = nt->addItem ("EMTopo_first_eta",  m_h_emtopon,  m_h_emtopo_first_eta);
  status = nt->addItem ("EMTopo_second_r",  m_h_emtopon,  m_h_emtopo_second_r);
  status = nt->addItem ("EMTopo_second_lambda",  m_h_emtopon,  m_h_emtopo_second_lambda);
  status = nt->addItem ("EMTopo_center_lambda",  m_h_emtopon,  m_h_emtopo_center_lambda);
  status = nt->addItem ("EMTopo_eng_frac_em",  m_h_emtopon,  m_h_emtopo_eng_frac_em);

  // Calocaltopo cluster
  mLog << MSG::DEBUG << " Booking CaloCalTopo variables. " << endreq;
  status = nt->addItem ("CaloCalTopoClusNum", m_h_calocaltopon, 0,  max_calocaltopo_num);
  status = nt->addItem ("CaloCalTopoClusNsamples",  m_h_calocaltopon, m_h_calocaltoponsamples);

  status = nt->addItem ("CaloCalTopoClus_Time",  m_h_calocaltopon, m_h_calocaltopo_time);
  status = nt->addItem ("CaloCalTopoCluseta",  m_h_calocaltopon, m_h_calocaltopoeta);
  status = nt->addItem ("CaloCalTopoClusphi",  m_h_calocaltopon, m_h_calocaltopophi);
  status = nt->addItem ("CaloCalTopoClusenergy",  m_h_calocaltopon, m_h_calocaltopoenergy);
  status = nt->addItem ("CaloCalTopoClusinBarrel",  m_h_calocaltopon, m_h_calocaltopoinBarrel);
  status = nt->addItem ("CaloCalTopoClusinEndcap",  m_h_calocaltopon, m_h_calocaltopoinEndcap); 

  // available for 420 and 630 (em and calocaltopo)
  status = nt->addItem ("CaloCalTopo_first_phi",  m_h_calocaltopon,  m_h_calocaltopo_first_phi);
  status = nt->addItem ("CaloCalTopo_first_eta",  m_h_calocaltopon,  m_h_calocaltopo_first_eta);
  status = nt->addItem ("CaloCalTopo_second_r",  m_h_calocaltopon,  m_h_calocaltopo_second_r);
  status = nt->addItem ("CaloCalTopo_second_lambda",  m_h_calocaltopon,  m_h_calocaltopo_second_lambda);
  status = nt->addItem ("CaloCalTopo_center_lambda",  m_h_calocaltopon,  m_h_calocaltopo_center_lambda);
  status = nt->addItem ("CaloCalTopo_eng_frac_em",  m_h_calocaltopon,  m_h_calocaltopo_eng_frac_em);

  mLog << MSG::DEBUG << " Finished Booking Electron variables. " << endreq;

  return status;
}

StatusCode AODtoLiverpoolNtuple::BookPhoton(NTuplePtr nt)
{
  MsgStream mLog( messageService(), name() );
  StatusCode status = StatusCode::SUCCESS;

  // Photon 
  mLog << MSG::DEBUG << " Booking PhotonContainer variables. " << endreq;
  status = nt->addItem ("photon_num", m_h_photon_num, 0,  max_photon_num);
  status = nt->addItem ("photon_pt",  m_h_photon_num, m_h_photon_pt);
  status = nt->addItem ("photon_eta", m_h_photon_num, m_h_photon_eta);
  status = nt->addItem ("photon_phi", m_h_photon_num, m_h_photon_phi);
  status = nt->addItem ("photon_e",   m_h_photon_num, m_h_photon_e);

  //  status = nt->addItem ("photon_author",  m_h_photon_num, m_h_photon_author);
  status = nt->addItem ("photon_egamma",  m_h_photon_num, m_h_photon_egamma);
  //  status = nt->addItem ("photon_softe",  m_h_photon_num, m_h_photon_softe);

  status = nt->addItem ("photon_IsEM",  m_h_photon_num, m_h_photon_IsEM);
  status = nt->addItem ("photon_ElectronWeight",  m_h_photon_num, m_h_photon_ElectronWeight);
  status = nt->addItem ("photon_BgWeight",  m_h_photon_num, m_h_photon_BgWeight);
  status = nt->addItem ("photon_NeuralNet",  m_h_photon_num, m_h_photon_NeuralNet);
  status = nt->addItem ("photon_SofteIsEM",  m_h_photon_num, m_h_photon_SofteIsEM);
  status = nt->addItem ("photon_SofteElectronWeight",  m_h_photon_num, m_h_photon_SofteElectronWeight);
  status = nt->addItem ("photon_SofteBgWeight",  m_h_photon_num, m_h_photon_SofteBgWeight);
  status = nt->addItem ("photon_SofteNeuralNet",  m_h_photon_num, m_h_photon_SofteNeuralNet);

  status = nt->addItem ("photon_etcone",  m_h_photon_num, m_h_photon_etcone);
  status = nt->addItem ("photon_etcone20",  m_h_photon_num, m_h_photon_etcone20);
  status = nt->addItem ("photon_etcone30",  m_h_photon_num, m_h_photon_etcone30);
  status = nt->addItem ("photon_etcone40",  m_h_photon_num, m_h_photon_etcone40);
  // photon parameters 
  status = nt->addItem ("photon_etaBE2",m_h_photon_num, m_h_photon_etaBE2);
  status = nt->addItem ("photon_et37",m_h_photon_num, m_h_photon_et37);
  status = nt->addItem ("photon_e233",m_h_photon_num, m_h_photon_e233);
  status = nt->addItem ("photon_e237",m_h_photon_num, m_h_photon_e237);
  status = nt->addItem ("photon_e277",m_h_photon_num, m_h_photon_e277);
  status = nt->addItem ("photon_ethad1",m_h_photon_num, m_h_photon_ethad1);
  status = nt->addItem ("photon_weta1",m_h_photon_num, m_h_photon_weta1);
  status = nt->addItem ("photon_weta2",m_h_photon_num, m_h_photon_weta2);
  status = nt->addItem ("photon_f1",m_h_photon_num, m_h_photon_f1);
  status = nt->addItem ("photon_e2tsts1",m_h_photon_num, m_h_photon_e2tsts1);
  status = nt->addItem ("photon_emins1",m_h_photon_num, m_h_photon_emins1);
  status = nt->addItem ("photon_wtots1",m_h_photon_num, m_h_photon_wtots1);
  status = nt->addItem ("photon_fracs1",m_h_photon_num, m_h_photon_fracs1);

  status = nt->addItem ("photon_conversionAngleMatch",m_h_photon_num, m_h_photon_conversionAngleMatch);
  status = nt->addItem ("photon_conversionTrackMatch",m_h_photon_num, m_h_photon_conversionTrackMatch);
  status = nt->addItem ("photon_conversionVx_x",m_h_photon_num, m_h_photon_conversionVx_x);
  status = nt->addItem ("photon_conversionVx_y",m_h_photon_num, m_h_photon_conversionVx_y);
  status = nt->addItem ("photon_conversionVx_z",m_h_photon_num, m_h_photon_conversionVx_z);
  status = nt->addItem ("photon_conversiontrack1_pt",m_h_photon_num, m_h_photon_conversiontrack1_pt);
  status = nt->addItem ("photon_conversiontrack1_eta",m_h_photon_num, m_h_photon_conversiontrack1_eta);
  status = nt->addItem ("photon_conversiontrack1_phi",m_h_photon_num, m_h_photon_conversiontrack1_phi);
  status = nt->addItem ("photon_conversiontrack1_e",m_h_photon_num, m_h_photon_conversiontrack1_e);
  status = nt->addItem ("photon_conversiontrack1_charge",m_h_photon_num, m_h_photon_conversiontrack1_charge);

  status = nt->addItem ("photon_conversiontrack2_pt",m_h_photon_num, m_h_photon_conversiontrack2_pt);
  status = nt->addItem ("photon_conversiontrack2_eta",m_h_photon_num, m_h_photon_conversiontrack2_eta);
  status = nt->addItem ("photon_conversiontrack2_phi",m_h_photon_num, m_h_photon_conversiontrack2_phi);
  status = nt->addItem ("photon_conversiontrack2_e",m_h_photon_num, m_h_photon_conversiontrack2_e);
  status = nt->addItem ("photon_conversiontrack2_charge",m_h_photon_num, m_h_photon_conversiontrack2_charge);

  return status;
}

StatusCode AODtoLiverpoolNtuple::BookMuidMuons(NTuplePtr nt)
{
  MsgStream mLog(messageService(), name());
  StatusCode status = StatusCode::SUCCESS;

  // MUID Muons
  mLog << MSG::DEBUG << " Booking MuidMuonContainer variables. " << endreq;
  status = nt->addItem ("muidmuon_num", m_h_muidmuon_num, 0,  max_muidmuon_num); 
  status = nt->addItem ("muidmuon_pt",m_h_muidmuon_num, m_h_muidmuon_pt);
  status = nt->addItem ("muidmuon_eta",m_h_muidmuon_num, m_h_muidmuon_eta);
  status = nt->addItem ("muidmuon_phi",m_h_muidmuon_num, m_h_muidmuon_phi);
  status = nt->addItem ("muidmuon_e",m_h_muidmuon_num, m_h_muidmuon_e);
  status = nt->addItem ("muidmuon_matchChi2OverDoF",m_h_muidmuon_num, m_h_muidmuon_matchChi2OverDoF);
  status = nt->addItem ("muidmuon_fitChi2OverDoF",m_h_muidmuon_num, m_h_muidmuon_fitChi2OverDoF);
  status = nt->addItem ("muidmuon_energyLoss_Deposit",m_h_muidmuon_num, m_h_muidmuon_energyLoss_Deposit);
  status = nt->addItem ("muidmuon_energyLoss_Sigma",m_h_muidmuon_num, m_h_muidmuon_energyLoss_Sigma);
  status = nt->addItem ("muidmuon_author",m_h_muidmuon_num, m_h_muidmuon_author);
  status = nt->addItem ("muidmuon_isCombinedMuon",m_h_muidmuon_num, m_h_muidmuon_isCombinedMuon);
  status = nt->addItem ("muidmuon_isStandAloneMuon",m_h_muidmuon_num, m_h_muidmuon_isStandAloneMuon);
  status = nt->addItem ("muidmuon_isLowPtReconstructedMuon",m_h_muidmuon_num, m_h_muidmuon_isLowPtReconstructedMuon);
  status = nt->addItem ("muidmuon_hasCombinedMuon",m_h_muidmuon_num, m_h_muidmuon_hasCombinedMuon);
  status = nt->addItem ("muidmuon_hasInDetTrackParticle",m_h_muidmuon_num, m_h_muidmuon_hasInDetTrackParticle);
  status = nt->addItem ("muidmuon_hasMuonSpectrometerTrackParticle",m_h_muidmuon_num, m_h_muidmuon_hasMuonSpectrometerTrackParticle);
  status = nt->addItem ("muidmuon_hasMuonExtrapolatedTrackParticle",m_h_muidmuon_num, m_h_muidmuon_hasMuonExtrapolatedTrackParticle);
  status = nt->addItem ("muidmuon_hasCombinedMuonTrackParticle",m_h_muidmuon_num, m_h_muidmuon_hasCombinedMuonTrackParticle);
  status = nt->addItem ("muidmuon_hasCluster",m_h_muidmuon_num, m_h_muidmuon_hasCluster);
  status = nt->addItem ("muidmuon_isHighPt",m_h_muidmuon_num, m_h_muidmuon_isHighPt);
  status = nt->addItem ("muidmuon_isLowPt",m_h_muidmuon_num, m_h_muidmuon_isLowPt);
  status = nt->addItem ("muidmuon_etcone10",m_h_muidmuon_num, m_h_muidmuon_etcone10);
  status = nt->addItem ("muidmuon_etcone20",m_h_muidmuon_num, m_h_muidmuon_etcone20);
  status = nt->addItem ("muidmuon_etcone30",m_h_muidmuon_num, m_h_muidmuon_etcone30);
  status = nt->addItem ("muidmuon_etcone40",m_h_muidmuon_num, m_h_muidmuon_etcone40);
  status = nt->addItem ("muidmuon_etcone",m_h_muidmuon_num, m_h_muidmuon_etcone);
  status = nt->addItem ("muidmuon_etcone50",m_h_muidmuon_num, m_h_muidmuon_etcone50);
  status = nt->addItem ("muidmuon_etcone60",m_h_muidmuon_num, m_h_muidmuon_etcone60);
  status = nt->addItem ("muidmuon_etcone70",m_h_muidmuon_num, m_h_muidmuon_etcone70);
  status = nt->addItem ("muidmuon_nucone10",m_h_muidmuon_num, m_h_muidmuon_nucone10);
  status = nt->addItem ("muidmuon_nucone20",m_h_muidmuon_num, m_h_muidmuon_nucone20);
  status = nt->addItem ("muidmuon_nucone30",m_h_muidmuon_num, m_h_muidmuon_nucone30);
  status = nt->addItem ("muidmuon_nucone40",m_h_muidmuon_num, m_h_muidmuon_nucone40);
  status = nt->addItem ("muidmuon_nucone",m_h_muidmuon_num, m_h_muidmuon_nucone);
  status = nt->addItem ("muidmuon_nucone50",m_h_muidmuon_num, m_h_muidmuon_nucone50);
  status = nt->addItem ("muidmuon_nucone60",m_h_muidmuon_num, m_h_muidmuon_nucone60);
  status = nt->addItem ("muidmuon_nucone70",m_h_muidmuon_num, m_h_muidmuon_nucone70);
  status = nt->addItem ("muidmuon_numberOfBLayerHits",m_h_muidmuon_num, m_h_muidmuon_numberOfBLayerHits);
  status = nt->addItem ("muidmuon_numberOfPixelHits",m_h_muidmuon_num, m_h_muidmuon_numberOfPixelHits);
  status = nt->addItem ("muidmuon_numberOfSCTHits",m_h_muidmuon_num, m_h_muidmuon_numberOfSCTHits);
  status = nt->addItem ("muidmuon_numberOfTRTHits",m_h_muidmuon_num, m_h_muidmuon_numberOfTRTHits);
  status = nt->addItem ("muidmuon_numberOfTRTHighThresholdHits",m_h_muidmuon_num, m_h_muidmuon_numberOfTRTHighThresholdHits);
  status = nt->addItem ("muidmuon_numberOfMDTHits",m_h_muidmuon_num, m_h_muidmuon_numberOfMDTHits);
  status = nt->addItem ("muidmuon_numberOfCSCEtaHits",m_h_muidmuon_num, m_h_muidmuon_numberOfCSCEtaHits);
  status = nt->addItem ("muidmuon_numberOfCSCPhiHits",m_h_muidmuon_num, m_h_muidmuon_numberOfCSCPhiHits);
  status = nt->addItem ("muidmuon_numberOfRPCEtaHits",m_h_muidmuon_num, m_h_muidmuon_numberOfRPCEtaHits);
  status = nt->addItem ("muidmuon_numberOfRPCPhiHits",m_h_muidmuon_num, m_h_muidmuon_numberOfRPCPhiHits);
  status = nt->addItem ("muidmuon_numberOfTGCEtaHits",m_h_muidmuon_num, m_h_muidmuon_numberOfTGCEtaHits);
  status = nt->addItem ("muidmuon_numberOfTGCPhiHits",m_h_muidmuon_num, m_h_muidmuon_numberOfTGCPhiHits);
  status = nt->addItem ("muidmuon_track_pt",m_h_muidmuon_num, m_h_muidmuon_track_pt);
  status = nt->addItem ("muidmuon_track_eta",m_h_muidmuon_num, m_h_muidmuon_track_eta);
  status = nt->addItem ("muidmuon_track_phi",m_h_muidmuon_num, m_h_muidmuon_track_phi);
  status = nt->addItem ("muidmuon_track_e",m_h_muidmuon_num, m_h_muidmuon_track_e);
  status = nt->addItem ("muidmuon_track_d0",m_h_muidmuon_num, m_h_muidmuon_track_d0);
  status = nt->addItem ("muidmuon_track_z0",m_h_muidmuon_num, m_h_muidmuon_track_z0);
  status = nt->addItem ("muidmuon_track_charge",m_h_muidmuon_num, m_h_muidmuon_track_charge);

  return status;
}

StatusCode AODtoLiverpoolNtuple::BookStacoMuons(NTuplePtr nt)
{
  MsgStream mLog(messageService(), name());
  StatusCode status = StatusCode::SUCCESS;

  // Staco Muons
  mLog << MSG::DEBUG << " Booking StacoMuonContainer variables. " << endreq;

  mLog << MSG::DEBUG << " Booking StacoMuonContainer variables. " << endreq;
  status = nt->addItem ("stacomuon_num", m_h_stacomuon_num, 0,  max_stacomuon_num); 
  status = nt->addItem ("stacomuon_pt",m_h_stacomuon_num, m_h_stacomuon_pt);
  status = nt->addItem ("stacomuon_eta",m_h_stacomuon_num, m_h_stacomuon_eta);
  status = nt->addItem ("stacomuon_phi",m_h_stacomuon_num, m_h_stacomuon_phi);
  status = nt->addItem ("stacomuon_e",m_h_stacomuon_num, m_h_stacomuon_e);
  status = nt->addItem ("stacomuon_matchChi2OverDoF",m_h_stacomuon_num, m_h_stacomuon_matchChi2OverDoF);
  status = nt->addItem ("stacomuon_fitChi2OverDoF",m_h_stacomuon_num, m_h_stacomuon_fitChi2OverDoF);
  status = nt->addItem ("stacomuon_energyLoss_Deposit",m_h_stacomuon_num, m_h_stacomuon_energyLoss_Deposit);
  status = nt->addItem ("stacomuon_energyLoss_Sigma",m_h_stacomuon_num, m_h_stacomuon_energyLoss_Sigma);
  status = nt->addItem ("stacomuon_author",m_h_stacomuon_num, m_h_stacomuon_author);
  status = nt->addItem ("stacomuon_isCombinedMuon",m_h_stacomuon_num, m_h_stacomuon_isCombinedMuon);
  status = nt->addItem ("stacomuon_isStandAloneMuon",m_h_stacomuon_num, m_h_stacomuon_isStandAloneMuon);
  status = nt->addItem ("stacomuon_isLowPtReconstructedMuon",m_h_stacomuon_num, m_h_stacomuon_isLowPtReconstructedMuon);
  status = nt->addItem ("stacomuon_hasCombinedMuon",m_h_stacomuon_num, m_h_stacomuon_hasCombinedMuon);
  status = nt->addItem ("stacomuon_hasInDetTrackParticle",m_h_stacomuon_num, m_h_stacomuon_hasInDetTrackParticle);
  status = nt->addItem ("stacomuon_hasMuonSpectrometerTrackParticle",m_h_stacomuon_num, m_h_stacomuon_hasMuonSpectrometerTrackParticle);
  status = nt->addItem ("stacomuon_hasMuonExtrapolatedTrackParticle",m_h_stacomuon_num, m_h_stacomuon_hasMuonExtrapolatedTrackParticle);
  status = nt->addItem ("stacomuon_hasCombinedMuonTrackParticle",m_h_stacomuon_num, m_h_stacomuon_hasCombinedMuonTrackParticle);
  status = nt->addItem ("stacomuon_hasCluster",m_h_stacomuon_num, m_h_stacomuon_hasCluster);
  status = nt->addItem ("stacomuon_isHighPt",m_h_stacomuon_num, m_h_stacomuon_isHighPt);
  status = nt->addItem ("stacomuon_isLowPt",m_h_stacomuon_num, m_h_stacomuon_isLowPt);
  status = nt->addItem ("stacomuon_etcone10",m_h_stacomuon_num, m_h_stacomuon_etcone10);
  status = nt->addItem ("stacomuon_etcone20",m_h_stacomuon_num, m_h_stacomuon_etcone20);
  status = nt->addItem ("stacomuon_etcone30",m_h_stacomuon_num, m_h_stacomuon_etcone30);
  status = nt->addItem ("stacomuon_etcone40",m_h_stacomuon_num, m_h_stacomuon_etcone40);
  status = nt->addItem ("stacomuon_etcone",m_h_stacomuon_num, m_h_stacomuon_etcone);
  status = nt->addItem ("stacomuon_etcone50",m_h_stacomuon_num, m_h_stacomuon_etcone50);
  status = nt->addItem ("stacomuon_etcone60",m_h_stacomuon_num, m_h_stacomuon_etcone60);
  status = nt->addItem ("stacomuon_etcone70",m_h_stacomuon_num, m_h_stacomuon_etcone70);
  status = nt->addItem ("stacomuon_nucone10",m_h_stacomuon_num, m_h_stacomuon_nucone10);
  status = nt->addItem ("stacomuon_nucone20",m_h_stacomuon_num, m_h_stacomuon_nucone20);
  status = nt->addItem ("stacomuon_nucone30",m_h_stacomuon_num, m_h_stacomuon_nucone30);
  status = nt->addItem ("stacomuon_nucone40",m_h_stacomuon_num, m_h_stacomuon_nucone40);
  status = nt->addItem ("stacomuon_nucone",m_h_stacomuon_num, m_h_stacomuon_nucone);
  status = nt->addItem ("stacomuon_nucone50",m_h_stacomuon_num, m_h_stacomuon_nucone50);
  status = nt->addItem ("stacomuon_nucone60",m_h_stacomuon_num, m_h_stacomuon_nucone60);
  status = nt->addItem ("stacomuon_nucone70",m_h_stacomuon_num, m_h_stacomuon_nucone70);
  status = nt->addItem ("stacomuon_numberOfBLayerHits",m_h_stacomuon_num, m_h_stacomuon_numberOfBLayerHits);
  status = nt->addItem ("stacomuon_numberOfPixelHits",m_h_stacomuon_num, m_h_stacomuon_numberOfPixelHits);
  status = nt->addItem ("stacomuon_numberOfSCTHits",m_h_stacomuon_num, m_h_stacomuon_numberOfSCTHits);
  status = nt->addItem ("stacomuon_numberOfTRTHits",m_h_stacomuon_num, m_h_stacomuon_numberOfTRTHits);
  status = nt->addItem ("stacomuon_numberOfTRTHighThresholdHits",m_h_stacomuon_num, m_h_stacomuon_numberOfTRTHighThresholdHits);
  status = nt->addItem ("stacomuon_numberOfMDTHits",m_h_stacomuon_num, m_h_stacomuon_numberOfMDTHits);
  status = nt->addItem ("stacomuon_numberOfCSCEtaHits",m_h_stacomuon_num, m_h_stacomuon_numberOfCSCEtaHits);
  status = nt->addItem ("stacomuon_numberOfCSCPhiHits",m_h_stacomuon_num, m_h_stacomuon_numberOfCSCPhiHits);
  status = nt->addItem ("stacomuon_numberOfRPCEtaHits",m_h_stacomuon_num, m_h_stacomuon_numberOfRPCEtaHits);
  status = nt->addItem ("stacomuon_numberOfRPCPhiHits",m_h_stacomuon_num, m_h_stacomuon_numberOfRPCPhiHits);
  status = nt->addItem ("stacomuon_numberOfTGCEtaHits",m_h_stacomuon_num, m_h_stacomuon_numberOfTGCEtaHits);
  status = nt->addItem ("stacomuon_numberOfTGCPhiHits",m_h_stacomuon_num, m_h_stacomuon_numberOfTGCPhiHits);
  status = nt->addItem ("stacomuon_track_pt",m_h_stacomuon_num, m_h_stacomuon_track_pt);
  status = nt->addItem ("stacomuon_track_eta",m_h_stacomuon_num, m_h_stacomuon_track_eta);
  status = nt->addItem ("stacomuon_track_phi",m_h_stacomuon_num, m_h_stacomuon_track_phi);
  status = nt->addItem ("stacomuon_track_e",m_h_stacomuon_num, m_h_stacomuon_track_e);
  status = nt->addItem ("stacomuon_track_d0",m_h_stacomuon_num, m_h_stacomuon_track_d0);
  status = nt->addItem ("stacomuon_track_z0",m_h_stacomuon_num, m_h_stacomuon_track_z0);
  status = nt->addItem ("stacomuon_track_charge",m_h_stacomuon_num, m_h_stacomuon_track_charge);

  return status;
}

StatusCode AODtoLiverpoolNtuple::BookCTP_Decision(NTuplePtr nt)
{
  MsgStream mLog( messageService(), name() );
  StatusCode status = StatusCode::SUCCESS;
  // CTP_Decision
  mLog << MSG::DEBUG << " Booking CTP_Decision variables. " << endreq;
  status = nt->addItem ("CTP_Decision_Word0", m_h_ctpdecision_word0);
  status = nt->addItem ("CTP_Decision_Word1", m_h_ctpdecision_word1);
  status = nt->addItem ("CTP_Decision_Word2", m_h_ctpdecision_word2);

  return status;
}

StatusCode AODtoLiverpoolNtuple::BookTriggerDecision(NTuplePtr nt)
{
  MsgStream mLog( messageService(), name() );
  StatusCode status = StatusCode::SUCCESS;
  // TriggerDecision
  mLog << MSG::DEBUG << " Booking TriggerDecision variables. " << endreq;
  // for 12.0.31
  //  status = nt->addItem ("TriggerDecision_passed_L1", m_h_passed_L1);
  //  status = nt->addItem ("TriggerDecision_passed_L2", m_h_passed_L2);
  //  status = nt->addItem ("TriggerDecision_passed_EF", m_h_passed_EF);
  //  status = nt->addItem ("TriggerDecision_L1_EM01", m_h_L1_EM01);
  //  status = nt->addItem ("TriggerDecision_L1_2EM15I", m_h_L1_2EM15I);
  //  status = nt->addItem ("TriggerDecision_L1_EM25I", m_h_L1_EM25I);
  //  status = nt->addItem ("TriggerDecision_L1_EM60", m_h_L1_EM60);
  //  status = nt->addItem ("TriggerDecision_L1_MU06", m_h_L1_MU06);
  //  status = nt->addItem ("TriggerDecision_L1_2MU06", m_h_L1_2MU06);
  //  status = nt->addItem ("TriggerDecision_L1_MU08", m_h_L1_MU08);
  //  status = nt->addItem ("TriggerDecision_L1_MU10", m_h_L1_MU10);
  //  status = nt->addItem ("TriggerDecision_L1_MU11", m_h_L1_MU11);
  //  status = nt->addItem ("TriggerDecision_L1_MU20", m_h_L1_MU20);
  //  status = nt->addItem ("TriggerDecision_L1_MU40", m_h_L1_MU40);
  //  status = nt->addItem ("TriggerDecision_L1_HA01", m_h_L1_HA01);
  //  status = nt->addItem ("TriggerDecision_L1_HA07", m_h_L1_HA07);
  //  status = nt->addItem ("TriggerDecision_L1_HA10I", m_h_L1_HA10I);
  //  status = nt->addItem ("TriggerDecision_L1_HA15I", m_h_L1_HA15I);
  //  status = nt->addItem ("TriggerDecision_L1_HA35I", m_h_L1_HA35I);

  status = nt->addItem ("TriggerDecision_L1_EM5", m_h_L1_EM5);
  status = nt->addItem ("TriggerDecision_L1_2EM15", m_h_L1_2EM15);
  status = nt->addItem ("TriggerDecision_L1_2EM15I", m_h_L1_2EM15I);
  status = nt->addItem ("TriggerDecision_L1_EM25", m_h_L1_EM25);
  status = nt->addItem ("TriggerDecision_L1_EM25I", m_h_L1_EM25I);
  status = nt->addItem ("TriggerDecision_L1_EM60", m_h_L1_EM60);
  status = nt->addItem ("TriggerDecision_L1_TAU05", m_h_L1_TAU05);
  status = nt->addItem ("TriggerDecision_L1_TAU10", m_h_L1_TAU10);
  status = nt->addItem ("TriggerDecision_L1_TAU10I", m_h_L1_TAU10I);
  status = nt->addItem ("TriggerDecision_L1_TAU15", m_h_L1_TAU15);
  status = nt->addItem ("TriggerDecision_L1_TAU15I", m_h_L1_TAU15I);
  status = nt->addItem ("TriggerDecision_L1_TAU20I", m_h_L1_TAU20I);
  status = nt->addItem ("TriggerDecision_L1_TAU25I", m_h_L1_TAU25I);
  status = nt->addItem ("TriggerDecision_L1_TAU35I", m_h_L1_TAU35I);
  status = nt->addItem ("TriggerDecision_L1_XE20", m_h_L1_XE20);
  status = nt->addItem ("TriggerDecision_L1_XE30", m_h_L1_XE30);
  status = nt->addItem ("TriggerDecision_L1_XE40", m_h_L1_XE40);
  status = nt->addItem ("TriggerDecision_L1_XE50", m_h_L1_XE50);
  status = nt->addItem ("TriggerDecision_L1_XE100", m_h_L1_XE100);
  status = nt->addItem ("TriggerDecision_L1_XE200", m_h_L1_XE200);
  status = nt->addItem ("TriggerDecision_L1_MU06", m_h_L1_MU06);
  status = nt->addItem ("TriggerDecision_L1_2MU06", m_h_L1_2MU06);
  status = nt->addItem ("TriggerDecision_L1_MU08", m_h_L1_MU08);
  status = nt->addItem ("TriggerDecision_L1_MU10", m_h_L1_MU10);
  status = nt->addItem ("TriggerDecision_L1_MU11", m_h_L1_MU11);
  status = nt->addItem ("TriggerDecision_L1_MU20", m_h_L1_MU20);
  status = nt->addItem ("TriggerDecision_L1_MU40", m_h_L1_MU40);
  status = nt->addItem ("TriggerDecision_L1_J35", m_h_L1_J35);
  status = nt->addItem ("TriggerDecision_L1_J45", m_h_L1_J45);
  status = nt->addItem ("TriggerDecision_L1_2J45", m_h_L1_2J45);
  status = nt->addItem ("TriggerDecision_L1_3J45", m_h_L1_3J45);
  status = nt->addItem ("TriggerDecision_L1_4J45", m_h_L1_4J45);
  status = nt->addItem ("TriggerDecision_L1_FJ30", m_h_L1_FJ30);
  status = nt->addItem ("TriggerDecision_L1_BJT15", m_h_L1_BJT15);
  
    
  status = nt->addItem ("TriggerDecision_L2_e10", m_h_L2_e10);
  status = nt->addItem ("TriggerDecision_L2_e10TRTxK", m_h_L2_e10TRTxK); 	
  status = nt->addItem ("TriggerDecision_L2_e10L2_e10", m_h_L2_e10L2_e10); 	
  status = nt->addItem ("TriggerDecision_L2_e15iL2_e15i", m_h_L2_e15iL2_e15i);	
  status = nt->addItem ("TriggerDecision_L2_e25i", m_h_L2_e25i); 	
  status = nt->addItem ("TriggerDecision_L2_e60", m_h_L2_e60);	
  status = nt->addItem ("TriggerDecision_L2_g10", m_h_L2_g10);	
  status = nt->addItem ("TriggerDecision_L2_g20iL2_g20i", m_h_L2_g20iL2_g20i);	
  status = nt->addItem ("TriggerDecision_L2_g60", m_h_L2_g60); 	
  status = nt->addItem ("TriggerDecision_L2_Ze10e10", m_h_L2_Ze10e10); 	
  status = nt->addItem ("TriggerDecision_L2_tau10", m_h_L2_tau10); 	
  status = nt->addItem ("TriggerDecision_L2_tau10i", m_h_L2_tau10i); 	
  status = nt->addItem ("TriggerDecision_L2_tau15", m_h_L2_tau15);  	
  status = nt->addItem ("TriggerDecision_L2_tau15i", m_h_L2_tau15i);  	
  status = nt->addItem ("TriggerDecision_L2_tau20i", m_h_L2_tau20i);  	
  status = nt->addItem ("TriggerDecision_L2_tau25i", m_h_L2_tau25i);  	
  status = nt->addItem ("TriggerDecision_L2_tau35i", m_h_L2_tau35i);  	
  status = nt->addItem ("TriggerDecision_L2_tauNoCut", m_h_L2_tauNoCut);  	
  status = nt->addItem ("TriggerDecision_L2_mu6", m_h_L2_mu6);  	
  status = nt->addItem ("TriggerDecision_L2_mu6l", m_h_L2_mu6l);  	
  status = nt->addItem ("TriggerDecision_L2_mu20i", m_h_L2_mu20i); 	
  status = nt->addItem ("TriggerDecision_L2_jet20kt", m_h_L2_jet20kt);  	
  status = nt->addItem ("TriggerDecision_L2_jet20", m_h_L2_jet20); 	
  status = nt->addItem ("TriggerDecision_L2_jet400", m_h_L2_jet400); 	
  status = nt->addItem ("TriggerDecision_L2_jet350L2_jet350", m_h_L2_jet350L2_jet350); 	
  status = nt->addItem ("TriggerDecision_L2_jet165L2_jet165L2_jet165", m_h_L2_jet165L2_jet165L2_jet165);  	
  status = nt->addItem ("TriggerDecision_L2_jet110L2_jet110L2_jet110L2_jet110", m_h_L2_jet110L2_jet110L2_jet110L2_jet110); 
  
  status = nt->addItem ("TriggerDecision_EF_MuonTRTExt_mu6l", m_h_EF_MuonTRTExt_mu6l);
  status = nt->addItem ("TriggerDecision_EF_e10", m_h_EF_e10);
  status = nt->addItem ("TriggerDecision_EF_e10TRTxK", m_h_EF_e10TRTxK);
  status = nt->addItem ("TriggerDecision_EF_e15iEF_e15i", m_h_EF_e15iEF_e15i);
  status = nt->addItem ("TriggerDecision_EF_e25i", m_h_EF_e25i);
  status = nt->addItem ("TriggerDecision_EF_e60", m_h_EF_e60);
  status = nt->addItem ("TriggerDecision_EF_g10", m_h_EF_g10);
  status = nt->addItem ("TriggerDecision_EF_g20iEF_g20i", m_h_EF_g20iEF_g20i);
  status = nt->addItem ("TriggerDecision_EF_g60", m_h_EF_g60);
  status = nt->addItem ("TriggerDecision_EF_jet110EF_jet110EF_jet110EF_jet110", m_h_EF_jet110EF_jet110EF_jet110EF_jet110);
  status = nt->addItem ("TriggerDecision_EF_jet165EF_jet165EF_jet165", m_h_EF_jet165EF_jet165EF_jet165);
  status = nt->addItem ("TriggerDecision_EF_jet20", m_h_EF_jet20);
  status = nt->addItem ("TriggerDecision_EF_jet20kt", m_h_EF_jet20kt);
  status = nt->addItem ("TriggerDecision_EF_jet350EF_jet350", m_h_EF_jet350EF_jet350);
  status = nt->addItem ("TriggerDecision_EF_jet400", m_h_EF_jet400);
  status = nt->addItem ("TriggerDecision_EF_mu20i", m_h_EF_mu20i);
  status = nt->addItem ("TriggerDecision_EF_mu6", m_h_EF_mu6);
  status = nt->addItem ("TriggerDecision_EF_mu6l", m_h_EF_mu6l);
  status = nt->addItem ("TriggerDecision_EF_tau10", m_h_EF_tau10);
  status = nt->addItem ("TriggerDecision_EF_tau10i", m_h_EF_tau10i);
  status = nt->addItem ("TriggerDecision_EF_tau15", m_h_EF_tau15);
  status = nt->addItem ("TriggerDecision_EF_tau15i", m_h_EF_tau15i);
  status = nt->addItem ("TriggerDecision_EF_tau20i", m_h_EF_tau20i);
  status = nt->addItem ("TriggerDecision_EF_tau25i", m_h_EF_tau25i);
  status = nt->addItem ("TriggerDecision_EF_tau35i", m_h_EF_tau35i);
  status = nt->addItem ("TriggerDecision_EF_tauNoCut", m_h_EF_tauNoCut);

  return status;
}


StatusCode AODtoLiverpoolNtuple::BookEFegamma(NTuplePtr nt)
{
  MsgStream mLog( messageService(), name() );
  StatusCode status = StatusCode::SUCCESS;
  // TrigElectron
  mLog << MSG::DEBUG << " Booking egammaContainer variables. " << endreq;
  // egamma combined result (Tracker + Calo)
  status = nt->addItem ("EFegamma_num",m_h_egamma_num, 0,  max_egamma_num);
  status = nt->addItem ("EFegamma_e",m_h_egamma_num,m_h_egamma_e);
  status = nt->addItem ("EFegamma_eta",m_h_egamma_num,m_h_egamma_eta);
  status = nt->addItem ("EFegamma_phi",m_h_egamma_num,m_h_egamma_phi);
  status = nt->addItem ("EFegamma_IsEM",m_h_egamma_num,m_h_egamma_isem);
  status = nt->addItem ("EFegamma_NeuralNet",m_h_egamma_num,m_h_egamma_epiANN);
  status = nt->addItem ("EFegamma_ElectronWeight",m_h_egamma_num,m_h_egamma_EMWeight);
  status = nt->addItem ("EFegamma_BgWeight",m_h_egamma_num,m_h_egamma_PionWeight);
  status = nt->addItem ("EFegamma_SofteIsEM",m_h_egamma_num,m_h_egamma_isemse);
  status = nt->addItem ("EFegamma_SofteNeuralNet",m_h_egamma_num,m_h_egamma_epiANNse);
  status = nt->addItem ("EFegamma_SofteElectronWeight",m_h_egamma_num,m_h_egamma_EMWeightse);
  status = nt->addItem ("EFegamma_SofteBgWeight",m_h_egamma_num,m_h_egamma_PionWeightse);
  
  // declare cluster variables
  // for explanations of ntuple variables see Liquid Argon reconstruction page
  
  status = nt->addItem ("EFegamma_cluster_et",m_h_egamma_num,m_h_egamma_cl_et);
  status = nt->addItem ("EFegamma_cluster_eta",m_h_egamma_num,m_h_egamma_cl_eta);
  status = nt->addItem ("EFegamma_cluster_phi",m_h_egamma_num,m_h_egamma_cl_phi);
  
  status = nt->addItem ("EFegamma_cluster_e0",m_h_egamma_num,m_h_egamma_e0);
  status = nt->addItem ("EFegamma_cluster_e1",m_h_egamma_num,m_h_egamma_e1);
  status = nt->addItem ("EFegamma_cluster_e2",m_h_egamma_num,m_h_egamma_e2);
  status = nt->addItem ("EFegamma_cluster_e3",m_h_egamma_num,m_h_egamma_e3);
  status = nt->addItem ("EFegamma_cluster_eta0",m_h_egamma_num,m_h_egamma_eta0);
  status = nt->addItem ("EFegamma_cluster_eta1",m_h_egamma_num,m_h_egamma_eta1);
  status = nt->addItem ("EFegamma_cluster_eta2",m_h_egamma_num,m_h_egamma_eta2);
  status = nt->addItem ("EFegamma_cluster_eta3",m_h_egamma_num,m_h_egamma_eta3);
  status = nt->addItem ("EFegamma_cluster_phi0",m_h_egamma_num,m_h_egamma_phi0);
  status = nt->addItem ("EFegamma_cluster_phi1",m_h_egamma_num,m_h_egamma_phi1);
  status = nt->addItem ("EFegamma_cluster_phi2",m_h_egamma_num,m_h_egamma_phi2);
  status = nt->addItem ("EFegamma_cluster_phi3",m_h_egamma_num,m_h_egamma_phi3);

  // Output from special tools
  status = nt->addItem ("EFegamma_clusterpoint_etap",m_h_egamma_num,m_h_egamma_etap);
  status = nt->addItem ("EFegamma_clusterpoint_zvertex",m_h_egamma_num,m_h_egamma_zvertex);
  status = nt->addItem ("EFegamma_clusterpoint_errz",m_h_egamma_num,m_h_egamma_errz);
  status = nt->addItem ("EFegamma_clusterdepth_depth",m_h_egamma_num,m_h_egamma_depth);

  // declare shower shape variables
  status = nt->addItem ("EFegamma_shower_Ethad1",m_h_egamma_num,m_h_egamma_ethad1);
  status = nt->addItem ("EFegamma_shower_Ethad",m_h_egamma_num,m_h_egamma_ethad);
  status = nt->addItem ("EFegamma_shower_Ehad1",m_h_egamma_num,m_h_egamma_ehad1);
  status = nt->addItem ("EFegamma_shower_F1",m_h_egamma_num,m_h_egamma_f1);
  status = nt->addItem ("EFegamma_shower_F3",m_h_egamma_num,m_h_egamma_f3);
  status = nt->addItem ("EFegamma_shower_E233",m_h_egamma_num,m_h_egamma_e233);
  status = nt->addItem ("EFegamma_shower_E237",m_h_egamma_num,m_h_egamma_e237);
  status = nt->addItem ("EFegamma_shower_E277",m_h_egamma_num,m_h_egamma_e277);
  status = nt->addItem ("EFegamma_shower_Weta1",m_h_egamma_num,m_h_egamma_weta1);
  status = nt->addItem ("EFegamma_shower_Weta2",m_h_egamma_num,m_h_egamma_weta2);
  status = nt->addItem ("EFegamma_shower_E2ts1",m_h_egamma_num,m_h_egamma_e2ts1);
  status = nt->addItem ("EFegamma_shower_E2tsts1",m_h_egamma_num,m_h_egamma_e2tsts1);
  status = nt->addItem ("EFegamma_shower_Widths1",m_h_egamma_num,m_h_egamma_widths1);
  status = nt->addItem ("EFegamma_shower_Widths2",m_h_egamma_num,m_h_egamma_widths2);
  status = nt->addItem ("EFegamma_shower_poscs1",m_h_egamma_num,m_h_egamma_poscs1);
  status = nt->addItem ("EFegamma_shower_poscs2",m_h_egamma_num,m_h_egamma_poscs2);
  status = nt->addItem ("EFegamma_shower_Barys1",m_h_egamma_num,m_h_egamma_barys1);
  status = nt->addItem ("EFegamma_shower_Wtots1",m_h_egamma_num,m_h_egamma_wtots1);
  status = nt->addItem ("EFegamma_shower_Emins1",m_h_egamma_num,m_h_egamma_emins1);
  status = nt->addItem ("EFegamma_shower_Emaxs1",m_h_egamma_num,m_h_egamma_emaxs1);
  status = nt->addItem ("EFegamma_shower_Fracs1",m_h_egamma_num,m_h_egamma_fracs1);
  status = nt->addItem ("EFegamma_shower_EtCone45",m_h_egamma_num,m_h_egamma_etcone);
  status = nt->addItem ("EFegamma_shower_EtCone20",m_h_egamma_num,m_h_egamma_etcone20);
  status = nt->addItem ("EFegamma_shower_EtCone30",m_h_egamma_num,m_h_egamma_etcone30);
  status = nt->addItem ("EFegamma_shower_EtCone40",m_h_egamma_num,m_h_egamma_etcone40);
  
  // shower shape for soft e-id
  status = nt->addItem ("EFegamma_shower_F1core",m_h_egamma_num,m_h_egamma_f1core);
  status = nt->addItem ("EFegamma_shower_F3core",m_h_egamma_num,m_h_egamma_f3core);
  status = nt->addItem ("EFegamma_shower_Asy1",m_h_egamma_num,m_h_egamma_asy1);
  status = nt->addItem ("EFegamma_shower_Pos7",m_h_egamma_num,m_h_egamma_pos7);
  status = nt->addItem ("EFegamma_shower_Isol",m_h_egamma_num,m_h_egamma_isol);
  status = nt->addItem ("EFegamma_shower_Ecore",m_h_egamma_num,m_h_egamma_ecore);
  
  // individual energy deposition in calorimeters
  status = nt->addItem ("EFegamma_shower_E011",m_h_egamma_num,m_h_egamma_e011);
  status = nt->addItem ("EFegamma_shower_E131",m_h_egamma_num,m_h_egamma_e131);
  status = nt->addItem ("EFegamma_shower_E235",m_h_egamma_num,m_h_egamma_e235);
  status = nt->addItem ("EFegamma_shower_E335",m_h_egamma_num,m_h_egamma_e335);
    
  // declare track match quantities for TrackParticle
  status = nt->addItem ("EFegamma_trackmatch_primary",m_h_egamma_num,m_h_egamma_primary);
  status = nt->addItem ("EFegamma_trackmatch_numtracks",m_h_egamma_num,m_h_egamma_numtracks);
  status = nt->addItem ("EFegamma_trackmatch_eoverp",m_h_egamma_num,m_h_egamma_eoverp);
  status = nt->addItem ("EFegamma_trackmatch_etacorr",m_h_egamma_num,m_h_egamma_etacorr);
  // delta eta and phi of track extrapolated into calorimeter position S1/S2
  status = nt->addItem ("EFegamma_trackmatch_deta0",m_h_egamma_num,m_h_egamma_deta0);
  status = nt->addItem ("EFegamma_trackmatch_dphi0",m_h_egamma_num,m_h_egamma_dphi0);
  status = nt->addItem ("EFegamma_trackmatch_deta1",m_h_egamma_num,m_h_egamma_deta1);
  status = nt->addItem ("EFegamma_trackmatch_dphi1",m_h_egamma_num,m_h_egamma_dphi1);
  status = nt->addItem ("EFegamma_trackmatch_deta2",m_h_egamma_num,m_h_egamma_deta2);
  status = nt->addItem ("EFegamma_trackmatch_dphi2",m_h_egamma_num,m_h_egamma_dphi2);
  status = nt->addItem ("EFegamma_trackmatch_deta3",m_h_egamma_num,m_h_egamma_deta3);
  status = nt->addItem ("EFegamma_trackmatch_dphi3",m_h_egamma_num,m_h_egamma_dphi3);

  // TRT information from TrackSummary
  status = nt->addItem ("EFegamma_tracksummary_nTRThits",m_h_egamma_num,m_h_egamma_tracksummary_nTRThits);  
  status = nt->addItem ("EFegamma_tracksummary_nTRThitsHT",m_h_egamma_num,m_h_egamma_tracksummary_nTRThitsHT);  

  return status;
}

StatusCode AODtoLiverpoolNtuple::BookTrigElectron(NTuplePtr nt)
{
  MsgStream mLog( messageService(), name() );
  StatusCode status = StatusCode::SUCCESS;
  // TrigElectron
  mLog << MSG::DEBUG << " Booking TrigElectronContainer variables. " << endreq;
  status = nt->addItem ("L2TrigElectron_num",         m_h_trigelectron_num, 0,  max_trigelectron_num);
  status = nt->addItem ("L2TrigElectron_container",   m_h_trigelectron_num, m_h_trigelectron_container);
  status = nt->addItem ("L2TrigElectron_pt",          m_h_trigelectron_num, m_h_trigelectron_pt);
  status = nt->addItem ("L2TrigElectron_eta",         m_h_trigelectron_num, m_h_trigelectron_eta);
  status = nt->addItem ("L2TrigElectron_phi",         m_h_trigelectron_num, m_h_trigelectron_phi);
  status = nt->addItem ("L2TrigElectron_m",           m_h_trigelectron_num, m_h_trigelectron_m);
  status = nt->addItem ("L2TrigElectron_isvalid",     m_h_trigelectron_num, m_h_trigelectron_isvalid);
  status = nt->addItem ("L2TrigElectron_roiID",       m_h_trigelectron_num, m_h_trigelectron_roiID);
  status = nt->addItem ("L2TrigElectron_zvtx",        m_h_trigelectron_num, m_h_trigelectron_zvtx);
  status = nt->addItem ("L2TrigElectron_err_eta",     m_h_trigelectron_num, m_h_trigelectron_err_eta);
  status = nt->addItem ("L2TrigElectron_err_phi",     m_h_trigelectron_num, m_h_trigelectron_err_phi);
  status = nt->addItem ("L2TrigElectron_err_zvtx",    m_h_trigelectron_num, m_h_trigelectron_err_zvtx);
  status = nt->addItem ("L2TrigElectron_err_pt",      m_h_trigelectron_num, m_h_trigelectron_err_pt);
  status = nt->addItem ("L2TrigElectron_trkclusdeta", m_h_trigelectron_num, m_h_trigelectron_trkclusdeta);
  status = nt->addItem ("L2TrigElectron_trkclusdphi", m_h_trigelectron_num, m_h_trigelectron_trkclusdphi);
  //new variables
  status = nt->addItem ("L2TrigElectron_track_algID",       m_h_trigelectron_num, m_h_trigelectron_track_algoID);
  status = nt->addItem ("L2TrigElectron_track_chi2",        m_h_trigelectron_num, m_h_trigelectron_track_chi2);
  status = nt->addItem ("L2TrigElectron_track_NStraw",      m_h_trigelectron_num, m_h_trigelectron_track_NStraw);
  status = nt->addItem ("L2TrigElectron_track_NStrawHits",  m_h_trigelectron_num, m_h_trigelectron_track_NStrawHits);
  status = nt->addItem ("L2TrigElectron_track_NStrawTime",  m_h_trigelectron_num, m_h_trigelectron_track_NStrawTime);
  status = nt->addItem ("L2TrigElectron_track_NTRHits",     m_h_trigelectron_num, m_h_trigelectron_track_NTRHits); 
  status = nt->addItem ("L2TrigElectron_track_param_a0",    m_h_trigelectron_num, m_h_trigelectron_track_param_a0); 
  status = nt->addItem ("L2TrigElectron_track_param_z0",    m_h_trigelectron_num, m_h_trigelectron_track_param_z0); 
  status = nt->addItem ("L2TrigElectron_track_param_phi0",  m_h_trigelectron_num, m_h_trigelectron_track_param_phi0); 
  status = nt->addItem ("L2TrigElectron_track_param_eta",   m_h_trigelectron_num, m_h_trigelectron_track_param_eta); 
  status = nt->addItem ("L2TrigElectron_track_param_pt",    m_h_trigelectron_num, m_h_trigelectron_track_param_pt); 
  status = nt->addItem ("L2TrigElectron_track_param_ea0",    m_h_trigelectron_num, m_h_trigelectron_track_param_ea0); 
  status = nt->addItem ("L2TrigElectron_track_param_ez0",    m_h_trigelectron_num, m_h_trigelectron_track_param_ez0); 
  status = nt->addItem ("L2TrigElectron_track_param_ephi0",  m_h_trigelectron_num, m_h_trigelectron_track_param_ephi0); 
  status = nt->addItem ("L2TrigElectron_track_param_eeta",   m_h_trigelectron_num, m_h_trigelectron_track_param_eeta); 
  status = nt->addItem ("L2TrigElectron_track_param_ept",    m_h_trigelectron_num, m_h_trigelectron_track_param_ept); 
  status = nt->addItem ("L2TrigElectron_track_endparam_a0",    m_h_trigelectron_num, m_h_trigelectron_track_endparam_a0); 
  status = nt->addItem ("L2TrigElectron_track_endparam_z0",    m_h_trigelectron_num, m_h_trigelectron_track_endparam_z0); 
  status = nt->addItem ("L2TrigElectron_track_endparam_phi0",  m_h_trigelectron_num, m_h_trigelectron_track_endparam_phi0); 
  status = nt->addItem ("L2TrigElectron_track_endparam_eta",   m_h_trigelectron_num, m_h_trigelectron_track_endparam_eta); 
  status = nt->addItem ("L2TrigElectron_track_endparam_pt",    m_h_trigelectron_num, m_h_trigelectron_track_endparam_pt); 
  status = nt->addItem ("L2TrigElectron_track_endparam_ea0",    m_h_trigelectron_num, m_h_trigelectron_track_endparam_ea0); 
  status = nt->addItem ("L2TrigElectron_track_endparam_ez0",    m_h_trigelectron_num, m_h_trigelectron_track_endparam_ez0); 
  status = nt->addItem ("L2TrigElectron_track_endparam_ephi0",  m_h_trigelectron_num, m_h_trigelectron_track_endparam_ephi0); 
  status = nt->addItem ("L2TrigElectron_track_endparam_eeta",   m_h_trigelectron_num, m_h_trigelectron_track_endparam_eeta); 
  status = nt->addItem ("L2TrigElectron_track_endparam_ept",    m_h_trigelectron_num, m_h_trigelectron_track_endparam_ept); 

  status = nt->addItem ("L2TrigElectron_cluster_e",       m_h_trigelectron_num, m_h_trigelectron_cluster_e);
  status = nt->addItem ("L2TrigElectron_cluster_et",      m_h_trigelectron_num, m_h_trigelectron_cluster_et);
  status = nt->addItem ("L2TrigElectron_cluster_eta",     m_h_trigelectron_num, m_h_trigelectron_cluster_eta);
  status = nt->addItem ("L2TrigElectron_cluster_phi",     m_h_trigelectron_num, m_h_trigelectron_cluster_phi);
  status = nt->addItem ("L2TrigElectron_cluster_e237",    m_h_trigelectron_num, m_h_trigelectron_cluster_e237);
  status = nt->addItem ("L2TrigElectron_cluster_e277",    m_h_trigelectron_num, m_h_trigelectron_cluster_e277);
  status = nt->addItem ("L2TrigElectron_cluster_e2tsts1", m_h_trigelectron_num, m_h_trigelectron_cluster_e2tsts1);
  status = nt->addItem ("L2TrigElectron_cluster_ehad1",   m_h_trigelectron_num, m_h_trigelectron_cluster_ehad1);
  status = nt->addItem ("L2TrigElectron_cluster_emaxs1",  m_h_trigelectron_num, m_h_trigelectron_cluster_emaxs1);
  status = nt->addItem ("L2TrigElectron_cluster_efracs1", m_h_trigelectron_num, m_h_trigelectron_cluster_fracs1);
  status = nt->addItem ("L2TrigElectron_cluster_weta2",   m_h_trigelectron_num, m_h_trigelectron_cluster_weta2);
 
  return status;
}


StatusCode AODtoLiverpoolNtuple::BookL1EMTauObject(NTuplePtr nt)
{
  MsgStream mLog( messageService(), name() );
  StatusCode status = StatusCode::SUCCESS;
  // L1EMTauObject
  mLog << MSG::DEBUG << " Booking L1EMTauObjectContainer variables. " << endreq;
  status = nt->addItem ("L1EMTauObject_Num", m_h_l1emtauobject_num, 0,  max_l1emtauobject_num);
  status = nt->addItem ("L1EMTauObject_EmCore",m_h_l1emtauobject_num, m_h_l1emtauobject_emcore);
  status = nt->addItem ("L1EMTauObject_HdCore",m_h_l1emtauobject_num, m_h_l1emtauobject_hdcore);
  status = nt->addItem ("L1EMTauObject_EmClus",m_h_l1emtauobject_num, m_h_l1emtauobject_emclus);
  status = nt->addItem ("L1EMTauObject_TauClus",m_h_l1emtauobject_num, m_h_l1emtauobject_tauclus);
  status = nt->addItem ("L1EMTauObject_EmIsol",m_h_l1emtauobject_num, m_h_l1emtauobject_emisol);
  status = nt->addItem ("L1EMTauObject_HdIsol",m_h_l1emtauobject_num, m_h_l1emtauobject_hdisol);
  status = nt->addItem ("L1EMTauObject_eta",m_h_l1emtauobject_num, m_h_l1emtauobject_eta);
  status = nt->addItem ("L1EMTauObject_phi",m_h_l1emtauobject_num, m_h_l1emtauobject_phi);
  return status;
}

StatusCode AODtoLiverpoolNtuple::BookL1JetObject(NTuplePtr nt)
{
  MsgStream mLog( messageService(), name() );
  StatusCode status = StatusCode::SUCCESS;
  // L1JetObject
  mLog << MSG::DEBUG << " Booking L1JetObjectContainer variables. " << endreq;
  status = nt->addItem ("L1JetObject_Num", m_h_l1jetobject_num, 0,  max_l1jetobject_num);
  status = nt->addItem ("L1JetObject_L1Jet_ET4x4",m_h_l1jetobject_num, m_h_l1jetobject_L1Jet_ET4x4);
  status = nt->addItem ("L1JetObject_L1Jet_ET6x6",m_h_l1jetobject_num, m_h_l1jetobject_L1Jet_ET6x6);
  status = nt->addItem ("L1JetObject_L1Jet_ET8x8",m_h_l1jetobject_num, m_h_l1jetobject_L1Jet_ET8x8);
  status = nt->addItem ("L1JetObject_L1Jet_eta",m_h_l1jetobject_num, m_h_l1jetobject_L1Jet_eta);
  status = nt->addItem ("L1JetObject_L1Jet_phi",m_h_l1jetobject_num, m_h_l1jetobject_L1Jet_phi);

  return status;
}

StatusCode AODtoLiverpoolNtuple::BookL1EtmissObject(NTuplePtr nt)
{
  MsgStream mLog( messageService(), name() );
  StatusCode status = StatusCode::SUCCESS;
  // L1EtmissObject
  mLog << MSG::DEBUG << " Booking L1EtmissObjectContainer variables. " << endreq;
  status = nt->addItem ("L1EtmissObject_L1ETM_Ex",m_h_l1etmissobject_L1ETM_Ex);
  status = nt->addItem ("L1EtmissObject_L1ETM_Ey",m_h_l1etmissobject_L1ETM_Ey);
  status = nt->addItem ("L1EtmissObject_L1ETM_EtMiss",m_h_l1etmissobject_L1ETM_EtMiss);
  status = nt->addItem ("L1EtmissObject_L1ETM_EtSum",m_h_l1etmissobject_L1ETM_EtSum);

  return status;
}



StatusCode AODtoLiverpoolNtuple::BookEtMiss(NTuplePtr nt)
{
  MsgStream mLog( messageService(), name() );
  StatusCode status = StatusCode::SUCCESS;
  
  // EtMiss
  mLog << MSG::DEBUG << " Booking EtMissContainer variables. " << endreq;

  status = nt->addItem ("etmiss_final_sumet" , m_h_etmiss_final_sumet);
  status = nt->addItem ("etmiss_final_ex"    , m_h_etmiss_final_ex);        
  status = nt->addItem ("etmiss_final_ey"    , m_h_etmiss_final_ey);        
  status = nt->addItem ("etmiss_final_phi"   , m_h_etmiss_final_phi);        
  status = nt->addItem ("etmiss_final_et"    , m_h_etmiss_final_et);        
  status = nt->addItem ("etmiss_final_source", m_h_etmiss_final_source);        
  status = nt->addItem ("etmiss_final_region", m_h_etmiss_final_region);        
  status = nt->addItem ("etmiss_final_region_ex", m_h_etmiss_final_region_ex);        
  status = nt->addItem ("etmiss_final_region_ey", m_h_etmiss_final_region_ey);        
  status = nt->addItem ("etmiss_final_region_etsum", m_h_etmiss_final_region_etsum);        

  return status;
}

StatusCode AODtoLiverpoolNtuple::BookEtMissRefFinal(NTuplePtr nt)//ED start
{
  MsgStream mLog( messageService(), name() );
  StatusCode status = StatusCode::SUCCESS;
  
  // EtMiss
  mLog << MSG::DEBUG << " Booking EtMissRefFinalContainer variables. " << endreq;

  status = nt->addItem ("etmiss_reffinal_sumet" , m_h_etmiss_reffinal_sumet);
  status = nt->addItem ("etmiss_reffinal_ex"    , m_h_etmiss_reffinal_ex);        
  status = nt->addItem ("etmiss_reffinal_ey"    , m_h_etmiss_reffinal_ey);            
  status = nt->addItem ("etmiss_reffinal_phi"   , m_h_etmiss_reffinal_phi);        
  status = nt->addItem ("etmiss_reffinal_et"    , m_h_etmiss_reffinal_et);
  return status;
}

StatusCode AODtoLiverpoolNtuple::BookEtMissLocHadTopoObj(NTuplePtr nt)//ED
{
  MsgStream mLog( messageService(), name() );
  StatusCode status = StatusCode::SUCCESS;
  
  // EtMiss
  mLog << MSG::DEBUG << " Booking EtMissLocHadTopoObjContainer variables. " << endreq;

  status = nt->addItem ("etmiss_lochadtopoobj_sumet" , m_h_etmiss_lochadtopoobj_sumet);
  status = nt->addItem ("etmiss_lochadtopoobj_ex"    , m_h_etmiss_lochadtopoobj_ex);        
  status = nt->addItem ("etmiss_lochadtopoobj_ey"    , m_h_etmiss_lochadtopoobj_ey);  
  status = nt->addItem ("etmiss_lochadtopoobj_phi"   , m_h_etmiss_lochadtopoobj_phi);        
  status = nt->addItem ("etmiss_lochadtopoobj_et"    , m_h_etmiss_lochadtopoobj_et);          
  return status;
}

StatusCode AODtoLiverpoolNtuple::BookEtMissObjFinal(NTuplePtr nt)//ED
{
  MsgStream mLog( messageService(), name() );
  StatusCode status = StatusCode::SUCCESS;
  
  // EtMiss
  mLog << MSG::DEBUG << " Booking EtMissObjFinalContainer variables. " << endreq;

  status = nt->addItem ("etmiss_objfinal_sumet" , m_h_etmiss_objfinal_sumet);
  status = nt->addItem ("etmiss_objfinal_ex"    , m_h_etmiss_objfinal_ex);        
  status = nt->addItem ("etmiss_objfinal_ey"    , m_h_etmiss_objfinal_ey);          
  status = nt->addItem ("etmiss_objfinal_phi"   , m_h_etmiss_objfinal_phi);        
  status = nt->addItem ("etmiss_objfinal_et"    , m_h_etmiss_objfinal_et);               
  return status;
}//ED end

StatusCode AODtoLiverpoolNtuple::BookTauJet(NTuplePtr nt)
{
  MsgStream mLog( messageService(), name() );
  StatusCode status = StatusCode::SUCCESS;
  
  // TauJet
  mLog << MSG::DEBUG << " Booking TauJetContainer variables. " << endreq;
  status = nt->addItem ("TauJet_num", m_h_taujet_num, 0, max_taujet_num );
  status = nt->addItem ("TauJet_Pt",m_h_taujet_num, m_h_taujet_pt);
  status = nt->addItem ("TauJet_Eta",m_h_taujet_num, m_h_taujet_eta);
  status = nt->addItem ("TauJet_Phi",m_h_taujet_num, m_h_taujet_phi);
  status = nt->addItem ("TauJet_E",m_h_taujet_num, m_h_taujet_e); 
  status = nt->addItem ("TauJet_numTrack",m_h_taujet_num, m_h_taujet_numTrack); 
  status = nt->addItem ("TauJet_author",m_h_taujet_num, m_h_taujet_author); 
  status = nt->addItem ("TauJet_etHadCalib",m_h_taujet_num, m_h_taujet_etHadCalib); 
  status = nt->addItem ("TauJet_etEMCalib",m_h_taujet_num, m_h_taujet_etEMCalib); 
  status = nt->addItem ("TauJet_emRadius",m_h_taujet_num, m_h_taujet_emRadius); 
  status = nt->addItem ("TauJet_isolationFraction",m_h_taujet_num, m_h_taujet_isolationFraction); 
  status = nt->addItem ("TauJet_centralityFraction",m_h_taujet_num, m_h_taujet_centralityFraction); 
  status = nt->addItem ("TauJet_stripWidth2",m_h_taujet_num, m_h_taujet_stripWidth2); 
  status = nt->addItem ("TauJet_nStripCells",m_h_taujet_num, m_h_taujet_nStripCells); 
  status = nt->addItem ("TauJet_logLikelihoodRatio",m_h_taujet_num, m_h_taujet_logLikelihoodRatio); 
  status = nt->addItem ("TauJet_lowPtTauJetDiscriminant",m_h_taujet_num, m_h_taujet_lowPtTauJetDiscriminant); 
  status = nt->addItem ("TauJet_lowPtTauEleDiscriminant",m_h_taujet_num, m_h_taujet_lowPtTauEleDiscriminant); 
  status = nt->addItem ("TauJet_tauJetNeuralnetwork",m_h_taujet_num, m_h_taujet_tauJetNeuralnetwork); 
  status = nt->addItem ("TauJet_tauENeuralNetwork",m_h_taujet_num, m_h_taujet_tauENeuralNetwork); 
  status = nt->addItem ("TauJet_annularIsolationFraction",m_h_taujet_num, m_h_taujet_annularIsolationFraction); 
  status = nt->addItem ("TauJet_etCaloAtEMScale",m_h_taujet_num, m_h_taujet_etCaloAtEMScale); 
  status = nt->addItem ("TauJet_etChargedHadCells",m_h_taujet_num, m_h_taujet_etChargedHadCells); 
  status = nt->addItem ("TauJet_etOtherEMCells",m_h_taujet_num, m_h_taujet_etOtherEMCells); 
  status = nt->addItem ("TauJet_etOtherHadCells",m_h_taujet_num, m_h_taujet_etOtherHadCells); 
  status = nt->addItem ("TauJet_discriminant",m_h_taujet_num, m_h_taujet_discriminant); 
  status = nt->addItem ("TauJet_discPDERS",m_h_taujet_num, m_h_taujet_discPDERS); 
  status = nt->addItem ("TauJet_discNN",m_h_taujet_num, m_h_taujet_discNN); 
  status = nt->addItem ("TauJet_sumPTtracks",m_h_taujet_num, m_h_taujet_sumPTtracks); 
  status = nt->addItem ("TauJet_nAssocTracksCore",m_h_taujet_num, m_h_taujet_nAssocTracksCore); 
  status = nt->addItem ("TauJet_nAssocTracksIsol",m_h_taujet_num, m_h_taujet_nAssocTracksIsol); 
  status = nt->addItem ("TauJet_rWidth2Trk3P",m_h_taujet_num, m_h_taujet_rWidth2Trk3P); 
  status = nt->addItem ("TauJet_massTrk3P",m_h_taujet_num, m_h_taujet_massTrk3P); 
  status = nt->addItem ("TauJet_signDOTrk3P",m_h_taujet_num, m_h_taujet_signDOTrk3P); 


  return status;
}







StatusCode AODtoLiverpoolNtuple::BookTauJet1p3p(NTuplePtr nt)
{
  MsgStream mLog( messageService(), name() );
  StatusCode status = StatusCode::SUCCESS;
  
  // TauJet
  mLog << MSG::DEBUG << " Booking TauJet1p3pContainer variables. " << endreq;

  status = nt->addItem ("TauJet1p3p_num", m_h_taujet1p3p_num, 0, max_taujet1p3p_num );
  status = nt->addItem ("TauJet1p3p_Pt",m_h_taujet1p3p_num, m_h_taujet1p3p_pt);
  status = nt->addItem ("TauJet1p3p_Eta",m_h_taujet1p3p_num, m_h_taujet1p3p_eta);
  status = nt->addItem ("TauJet1p3p_Phi",m_h_taujet1p3p_num, m_h_taujet1p3p_phi);
  status = nt->addItem ("TauJet1p3p_E",m_h_taujet1p3p_num, m_h_taujet1p3p_e); 
  status = nt->addItem ("TauJet1p3p_numTrack",m_h_taujet1p3p_num, m_h_taujet1p3p_numTrack); 
  status = nt->addItem ("TauJet1p3p_author",m_h_taujet1p3p_num, m_h_taujet1p3p_author); 
  status = nt->addItem ("TauJet1p3p_etHadCalib",m_h_taujet1p3p_num, m_h_taujet1p3p_etHadCalib); 
  status = nt->addItem ("TauJet1p3p_etEMCalib",m_h_taujet1p3p_num, m_h_taujet1p3p_etEMCalib); 
  status = nt->addItem ("TauJet1p3p_emRadius",m_h_taujet1p3p_num, m_h_taujet1p3p_emRadius); 
  status = nt->addItem ("TauJet1p3p_isolationFraction",m_h_taujet1p3p_num, m_h_taujet1p3p_isolationFraction); 
  status = nt->addItem ("TauJet1p3p_centralityFraction",m_h_taujet1p3p_num, m_h_taujet1p3p_centralityFraction); 
  status = nt->addItem ("TauJet1p3p_stripWidth2",m_h_taujet1p3p_num, m_h_taujet1p3p_stripWidth2); 
  status = nt->addItem ("TauJet1p3p_nStripCells",m_h_taujet1p3p_num, m_h_taujet1p3p_nStripCells); 
  status = nt->addItem ("TauJet1p3p_logLikelihoodRatio",m_h_taujet1p3p_num, m_h_taujet1p3p_logLikelihoodRatio); 
  status = nt->addItem ("TauJet1p3p_lowPtTauJetDiscriminant",m_h_taujet1p3p_num, m_h_taujet1p3p_lowPtTauJetDiscriminant); 
  status = nt->addItem ("TauJet1p3p_lowPtTauEleDiscriminant",m_h_taujet1p3p_num, m_h_taujet1p3p_lowPtTauEleDiscriminant); 
  status = nt->addItem ("TauJet1p3p_tauJetNeuralnetwork",m_h_taujet1p3p_num, m_h_taujet1p3p_tauJetNeuralnetwork); 
  status = nt->addItem ("TauJet1p3p_tauENeuralNetwork",m_h_taujet1p3p_num, m_h_taujet1p3p_tauENeuralNetwork); 
  status = nt->addItem ("TauJet1p3p_annularIsolationFraction",m_h_taujet1p3p_num, m_h_taujet1p3p_annularIsolationFraction); 
  status = nt->addItem ("TauJet1p3p_etCaloAtEMScale",m_h_taujet1p3p_num, m_h_taujet1p3p_etCaloAtEMScale); 
  status = nt->addItem ("TauJet1p3p_etChargedHadCells",m_h_taujet1p3p_num, m_h_taujet1p3p_etChargedHadCells); 
  status = nt->addItem ("TauJet1p3p_etOtherEMCells",m_h_taujet1p3p_num, m_h_taujet1p3p_etOtherEMCells); 
  status = nt->addItem ("TauJet1p3p_etOtherHadCells",m_h_taujet1p3p_num, m_h_taujet1p3p_etOtherHadCells); 
  status = nt->addItem ("TauJet1p3p_discriminant",m_h_taujet1p3p_num, m_h_taujet1p3p_discriminant); 
  status = nt->addItem ("TauJet1p3p_discPDERS",m_h_taujet1p3p_num, m_h_taujet1p3p_discPDERS); 
  status = nt->addItem ("TauJet1p3p_discNN",m_h_taujet1p3p_num, m_h_taujet1p3p_discNN); 
  status = nt->addItem ("TauJet1p3p_sumPTtracks",m_h_taujet1p3p_num, m_h_taujet1p3p_sumPTtracks); 
  status = nt->addItem ("TauJet1p3p_nAssocTracksCore",m_h_taujet1p3p_num, m_h_taujet1p3p_nAssocTracksCore); 
  status = nt->addItem ("TauJet1p3p_nAssocTracksIsol",m_h_taujet1p3p_num, m_h_taujet1p3p_nAssocTracksIsol); 
  status = nt->addItem ("TauJet1p3p_rWidth2Trk3P",m_h_taujet1p3p_num, m_h_taujet1p3p_rWidth2Trk3P); 
  status = nt->addItem ("TauJet1p3p_massTrk3P",m_h_taujet1p3p_num, m_h_taujet1p3p_massTrk3P); 
  status = nt->addItem ("TauJet1p3p_signDOTrk3P",m_h_taujet1p3p_num, m_h_taujet1p3p_signDOTrk3P); 


  return status;
}








StatusCode AODtoLiverpoolNtuple::BookTrack(NTuplePtr nt)
{
  MsgStream mLog( messageService(), name() );
  StatusCode status = StatusCode::SUCCESS;

  // TrackParticle
  mLog << MSG::DEBUG << " Booking TrackParticleContainer variables. " << endreq;
  status = nt->addItem ("track_num", m_h_track_num, 0, max_track_num);
  //status = nt->addItem ("track_px",m_h_track_num, m_h_track_px);
  //status = nt->addItem ("track_py",m_h_track_num, m_h_track_py);
  //status = nt->addItem ("track_pz",m_h_track_num, m_h_track_pz);
  //status = nt->addItem ("track_p",m_h_track_num, m_h_track_p);
  status = nt->addItem ("track_pt",m_h_track_num, m_h_track_pt);  
  status = nt->addItem ("track_charge",m_h_track_num, m_h_track_charge);  
  status = nt->addItem ("track_eta",m_h_track_num, m_h_track_eta);  
  status = nt->addItem ("track_phi",m_h_track_num, m_h_track_phi);  
  status = nt->addItem ("track_ndf",m_h_track_num, m_h_track_ndf);  
  status = nt->addItem ("track_chi2",m_h_track_num, m_h_track_chi2);  
  status = nt->addItem ("track_d0",m_h_track_num, m_h_track_d0);  
  status = nt->addItem ("track_z0",m_h_track_num, m_h_track_z0);  
  status = nt->addItem ("track_perigee_par0", m_h_track_num, m_h_track_perigee_par0);	 
  status = nt->addItem ("track_perigee_par1", m_h_track_num, m_h_track_perigee_par1);	 
  status = nt->addItem ("track_perigee_par2", m_h_track_num, m_h_track_perigee_par2);	 
  status = nt->addItem ("track_perigee_par3", m_h_track_num, m_h_track_perigee_par3);	 
  status = nt->addItem ("track_perigee_par4", m_h_track_num, m_h_track_perigee_par4);	 
  status = nt->addItem ("track_perigee_error11", m_h_track_num, m_h_track_perigee_error11);
  status = nt->addItem ("track_perigee_error22", m_h_track_num, m_h_track_perigee_error22);
  status = nt->addItem ("track_perigee_error33", m_h_track_num, m_h_track_perigee_error33);
  status = nt->addItem ("track_perigee_error44", m_h_track_num, m_h_track_perigee_error44);
  status = nt->addItem ("track_perigee_error55", m_h_track_num, m_h_track_perigee_error55);
  status = nt->addItem ("track_origin",m_h_track_num,m_h_track_origin);
  status = nt->addItem ("track_NumBLayerHits",m_h_track_num, m_h_track_NumBLayerHits);  
  status = nt->addItem ("track_NumPixelHits" ,m_h_track_num, m_h_track_NumPixelHits);
  status = nt->addItem ("track_NumPixelHoles",m_h_track_num, m_h_track_NumPixelHoles);
  status = nt->addItem ("track_NumSCTHits" ,m_h_track_num, m_h_track_NumSCTHits);
  status = nt->addItem ("track_NumSCTHoles",m_h_track_num, m_h_track_NumSCTHoles);
  status = nt->addItem ("track_NumTRTHits" ,m_h_track_num, m_h_track_NumTRTHits);
  status = nt->addItem ("track_NumTRTHighThreshHits",m_h_track_num, m_h_track_NumTRTHighThreshHits);
  status = nt->addItem ("track_NumTRTHoles",m_h_track_num, m_h_track_NumTRTHoles);
  status = nt->addItem ("track_hitMask",m_h_track_num, m_h_track_hitMask);
  return status;
}

StatusCode AODtoLiverpoolNtuple::BookJet(NTuplePtr nt)
{
  MsgStream mLog( messageService(), name() );
  StatusCode status = StatusCode::SUCCESS;
  
  // Jet
  mLog << MSG::DEBUG << " Booking JetContainer variables. " << endreq;

  // Cone
  status = nt->addItem ("cone_num", m_h_cone_num, 0,  max_jet_num);
  status = nt->addItem ("cone_pt",m_h_cone_num, m_h_cone_pt);
  status = nt->addItem ("cone_eta",m_h_cone_num, m_h_cone_eta);
  status = nt->addItem ("cone_phi",m_h_cone_num, m_h_cone_phi);
  
  status = nt->addItem ("cone_lhsig", m_h_cone_num, m_h_cone_lhsig);     
  status = nt->addItem ("cone_w", m_h_cone_num, m_h_cone_w);     
  status = nt->addItem ("cone_wIP2D", m_h_cone_num, m_h_cone_wIP2D); 
  status = nt->addItem ("cone_wIP3D", m_h_cone_num, m_h_cone_wIP3D); 
  status = nt->addItem ("cone_wSV1", m_h_cone_num, m_h_cone_wSV1);  
  status = nt->addItem ("cone_wSV2", m_h_cone_num, m_h_cone_wSV2);  
  status = nt->addItem ("cone_wLT1D", m_h_cone_num, m_h_cone_wLT1D); 
  status = nt->addItem ("cone_wLT2D", m_h_cone_num, m_h_cone_wLT2D); 
  //status = nt->addItem ("cone_wLT3D", m_h_cone_num, m_h_cone_wLT3D);
  status = nt->addItem ("cone_wSVBU", m_h_cone_num, m_h_cone_SVBU);  
  //status = nt->addItem ("cone_wSVTD", m_h_cone_num, m_h_cone_SVTD);
  status = nt->addItem ("cone_wLHSig", m_h_cone_num, m_h_cone_wLHSig);
  status = nt->addItem ("cone_wSoftMu", m_h_cone_num, m_h_cone_wSoftMu);
  status = nt->addItem ("cone_wSoftElec", m_h_cone_num, m_h_cone_wSoftElec);
  status = nt->addItem ("cone_wJP", m_h_cone_num, m_h_cone_wJP);


  status = nt->addItem ("cone_ntracks", m_h_cone_num, m_h_cone_ntracks);
  status = nt->addItem ("cone_trackidx", m_h_cone_num, m_h_cone_trackidx, max_jettrack_num);
  status = nt->addItem ("cone_truth", m_h_cone_num, m_h_cone_truth);

  status = nt->addItem ("cone_sv1_pu", m_h_cone_num, m_h_cone_sv1_pu);		
  status = nt->addItem ("cone_sv1_pb", m_h_cone_num, m_h_cone_sv1_pb);		
  status = nt->addItem ("cone_sv2_pu", m_h_cone_num, m_h_cone_sv2_pu);		
  status = nt->addItem ("cone_sv2_pb", m_h_cone_num, m_h_cone_sv2_pb);		
  status = nt->addItem ("cone_svplus_n2t", m_h_cone_num, m_h_cone_svplus_n2t); 	
  status = nt->addItem ("cone_svplus_eFrac", m_h_cone_num, m_h_cone_svplus_eFrac);	
  status = nt->addItem ("cone_svplus_mass", m_h_cone_num, m_h_cone_svplus_mass);	
  status = nt->addItem ("cone_svplus_normDist", m_h_cone_num, m_h_cone_svplus_normDist);
  status = nt->addItem ("cone_svplus_NGTracks", m_h_cone_num, m_h_cone_svplus_NGTracks);
  status = nt->addItem ("cone_svplus_chi2", m_h_cone_num, m_h_cone_svplus_chi2);	
  status = nt->addItem ("cone_svplus_ndf", m_h_cone_num, m_h_cone_svplus_ndf);	
  status = nt->addItem ("cone_svplus_x", m_h_cone_num, m_h_cone_svplus_x);	
  status = nt->addItem ("cone_svplus_y", m_h_cone_num, m_h_cone_svplus_y);	
  status = nt->addItem ("cone_svplus_z", m_h_cone_num, m_h_cone_svplus_z);	
  status = nt->addItem ("cone_svplus_ntracks", m_h_cone_num, m_h_cone_svplus_ntracks);	   
  status = nt->addItem ("cone_svplus_trackidx", m_h_cone_num, m_h_cone_svplus_trackidx, max_jettrack_num);
  status = nt->addItem ("cone_svbu_pu", m_h_cone_num, m_h_cone_svbu_pu);	
  status = nt->addItem ("cone_svbu_pb", m_h_cone_num, m_h_cone_svbu_pb);	
  status = nt->addItem ("cone_svbu_eFrac", m_h_cone_num, m_h_cone_svbu_eFrac);	
  status = nt->addItem ("cone_svbu_mass", m_h_cone_num, m_h_cone_svbu_mass);	
  status = nt->addItem ("cone_svbu_normDist", m_h_cone_num, m_h_cone_svbu_normDist);	
  status = nt->addItem ("cone_svbu_mult", m_h_cone_num, m_h_cone_svbu_mult);	
  status = nt->addItem ("cone_svbu_chi2", m_h_cone_num, m_h_cone_svbu_chi2);	
  status = nt->addItem ("cone_svbu_ndf", m_h_cone_num, m_h_cone_svbu_ndf);	
  status = nt->addItem ("cone_svbu_x", m_h_cone_num, m_h_cone_svbu_x);		
  status = nt->addItem ("cone_svbu_y", m_h_cone_num, m_h_cone_svbu_y);		
  status = nt->addItem ("cone_svbu_z", m_h_cone_num, m_h_cone_svbu_z);         

  status = nt->addItem ("cone_ip2d_pu", m_h_cone_num, m_h_cone_ip2d_pu);	   
  status = nt->addItem ("cone_ip2d_pb", m_h_cone_num, m_h_cone_ip2d_pb);	   
  status = nt->addItem ("cone_ip2d_numTracks", m_h_cone_num, m_h_cone_ip2d_numTracks);	   
  status = nt->addItem ("cone_ip3d_pu", m_h_cone_num, m_h_cone_ip3d_pu);	   
  status = nt->addItem ("cone_ip3d_pb", m_h_cone_num, m_h_cone_ip3d_pb);	   
  status = nt->addItem ("cone_ip3d_numTracks", m_h_cone_num, m_h_cone_ip3d_numTracks);	   
  			                          
  status = nt->addItem ("cone_ipplus_d0", m_h_cone_num, m_h_cone_ipplus_d0, max_jettrack_num);	   
  status = nt->addItem ("cone_ipplus_d0Sig", m_h_cone_num, m_h_cone_ipplus_d0Sig, max_jettrack_num);	   
  status = nt->addItem ("cone_ipplus_z0", m_h_cone_num, m_h_cone_ipplus_z0, max_jettrack_num);	   
  status = nt->addItem ("cone_ipplus_z0Sig", m_h_cone_num, m_h_cone_ipplus_z0Sig, max_jettrack_num);	   
  status = nt->addItem ("cone_ipplus_trackW2D", m_h_cone_num, m_h_cone_ipplus_trackW2D, max_jettrack_num);  
  status = nt->addItem ("cone_ipplus_trackW3D", m_h_cone_num, m_h_cone_ipplus_trackW3D, max_jettrack_num);  
  status = nt->addItem ("cone_ipplus_isFromV0", m_h_cone_num, m_h_cone_ipplus_isFromV0, max_jettrack_num);  
  status = nt->addItem ("cone_ipplus_trackJP", m_h_cone_num, m_h_cone_ipplus_trackJP, max_jettrack_num);   
  status = nt->addItem ("cone_ipplus_trackGrade", m_h_cone_num, m_h_cone_ipplus_trackGrade, max_jettrack_num);
  status = nt->addItem ("cone_ipplus_trackidx", m_h_cone_num, m_h_cone_ipplus_trackidx, max_jettrack_num);

  status = nt->addItem ("cone_lt1d_numTracks", m_h_cone_num, m_h_cone_lt1d_numTracks);	   
  status = nt->addItem ("cone_lt1d_z0", m_h_cone_num, m_h_cone_lt1d_z0, max_jettrack_num);	   
  status = nt->addItem ("cone_lt1d_z0Sig", m_h_cone_num, m_h_cone_lt1d_z0Sig, max_jettrack_num);	   
  status = nt->addItem ("cone_lt2d_numTracks", m_h_cone_num, m_h_cone_lt2d_numTracks);	   
  status = nt->addItem ("cone_lt2d_d0", m_h_cone_num, m_h_cone_lt2d_d0, max_jettrack_num);	   
  status = nt->addItem ("cone_lt2d_d0Sig", m_h_cone_num, m_h_cone_lt2d_d0Sig, max_jettrack_num);	  

  status = nt->addItem ("cone_softmu_numTracks", m_h_cone_num, m_h_cone_softmu_numTracks);	   
  status = nt->addItem ("cone_softmu_d0", m_h_cone_num, m_h_cone_softmu_d0, max_jettrack_num);
  status = nt->addItem ("cone_softmu_ptrel", m_h_cone_num, m_h_cone_softmu_ptrel, max_jettrack_num);
  status = nt->addItem ("cone_softelec_numTracks", m_h_cone_num, m_h_cone_softelec_numTracks);	   
  status = nt->addItem ("cone_softelec_d0", m_h_cone_num, m_h_cone_softelec_d0, max_jettrack_num);
  status = nt->addItem ("cone_softelec_ptrel", m_h_cone_num, m_h_cone_softelec_ptrel, max_jettrack_num);

  // Cone 4
  status = nt->addItem ("cone4_num", m_h_cone4_num, 0,  max_jet_num);
  status = nt->addItem ("cone4_pt",m_h_cone4_num, m_h_cone4_pt);
  status = nt->addItem ("cone4_eta",m_h_cone4_num, m_h_cone4_eta);
  status = nt->addItem ("cone4_phi",m_h_cone4_num, m_h_cone4_phi);
  
  status = nt->addItem ("cone4_lhsig", m_h_cone4_num, m_h_cone4_lhsig);     
  status = nt->addItem ("cone4_w", m_h_cone4_num, m_h_cone4_w);     
  status = nt->addItem ("cone4_wIP2D", m_h_cone4_num, m_h_cone4_wIP2D); 
  status = nt->addItem ("cone4_wIP3D", m_h_cone4_num, m_h_cone4_wIP3D); 
  status = nt->addItem ("cone4_wSV1", m_h_cone4_num, m_h_cone4_wSV1);  
  status = nt->addItem ("cone4_wSV2", m_h_cone4_num, m_h_cone4_wSV2);  
  status = nt->addItem ("cone4_wLT1D", m_h_cone4_num, m_h_cone4_wLT1D); 
  status = nt->addItem ("cone4_wLT2D", m_h_cone4_num, m_h_cone4_wLT2D); 
  //status = nt->addItem ("cone4_wLT3D", m_h_cone4_num, m_h_cone4_wLT3D);
  status = nt->addItem ("cone4_wSVBU", m_h_cone4_num, m_h_cone4_SVBU);  
  //status = nt->addItem ("cone4_wSVTD", m_h_cone4_num, m_h_cone4_SVTD);
  status = nt->addItem ("cone4_wLHSig", m_h_cone4_num, m_h_cone4_wLHSig);
  status = nt->addItem ("cone4_wSoftMu", m_h_cone4_num, m_h_cone4_wSoftMu);
  status = nt->addItem ("cone4_wSoftElec", m_h_cone4_num, m_h_cone4_wSoftElec);
  status = nt->addItem ("cone4_wJP", m_h_cone4_num, m_h_cone4_wJP);

  status = nt->addItem ("cone4_ntracks", m_h_cone4_num, m_h_cone4_ntracks);
  status = nt->addItem ("cone4_trackidx", m_h_cone4_num, m_h_cone4_trackidx, max_jettrack_num);
  status = nt->addItem ("cone4_truth", m_h_cone4_num, m_h_cone4_truth);

  status = nt->addItem ("cone4_sv1_pu", m_h_cone4_num, m_h_cone4_sv1_pu);		
  status = nt->addItem ("cone4_sv1_pb", m_h_cone4_num, m_h_cone4_sv1_pb);		
  status = nt->addItem ("cone4_sv2_pu", m_h_cone4_num, m_h_cone4_sv2_pu);		
  status = nt->addItem ("cone4_sv2_pb", m_h_cone4_num, m_h_cone4_sv2_pb);		
  status = nt->addItem ("cone4_svplus_n2t", m_h_cone4_num, m_h_cone4_svplus_n2t); 	
  status = nt->addItem ("cone4_svplus_eFrac", m_h_cone4_num, m_h_cone4_svplus_eFrac);	
  status = nt->addItem ("cone4_svplus_mass", m_h_cone4_num, m_h_cone4_svplus_mass);	
  status = nt->addItem ("cone4_svplus_normDist", m_h_cone4_num, m_h_cone4_svplus_normDist);
  status = nt->addItem ("cone4_svplus_NGTracks", m_h_cone4_num, m_h_cone4_svplus_NGTracks);
  status = nt->addItem ("cone4_svplus_chi2", m_h_cone4_num, m_h_cone4_svplus_chi2);	
  status = nt->addItem ("cone4_svplus_ndf", m_h_cone4_num, m_h_cone4_svplus_ndf);	
  status = nt->addItem ("cone4_svplus_x", m_h_cone4_num, m_h_cone4_svplus_x);	
  status = nt->addItem ("cone4_svplus_y", m_h_cone4_num, m_h_cone4_svplus_y);	
  status = nt->addItem ("cone4_svplus_z", m_h_cone4_num, m_h_cone4_svplus_z);
  status = nt->addItem ("cone4_svplus_ntracks", m_h_cone4_num, m_h_cone4_svplus_ntracks);	   
  status = nt->addItem ("cone4_svplus_trackidx", m_h_cone4_num, m_h_cone4_svplus_trackidx, max_jettrack_num);
  status = nt->addItem ("cone4_svbu_pu", m_h_cone4_num, m_h_cone4_svbu_pu);	
  status = nt->addItem ("cone4_svbu_pb", m_h_cone4_num, m_h_cone4_svbu_pb);	
  status = nt->addItem ("cone4_svbu_eFrac", m_h_cone4_num, m_h_cone4_svbu_eFrac);	
  status = nt->addItem ("cone4_svbu_mass", m_h_cone4_num, m_h_cone4_svbu_mass);	
  status = nt->addItem ("cone4_svbu_normDist", m_h_cone4_num, m_h_cone4_svbu_normDist);	
  status = nt->addItem ("cone4_svbu_mult", m_h_cone4_num, m_h_cone4_svbu_mult);	
  status = nt->addItem ("cone4_svbu_chi2", m_h_cone4_num, m_h_cone4_svbu_chi2);	
  status = nt->addItem ("cone4_svbu_ndf", m_h_cone4_num, m_h_cone4_svbu_ndf);	
  status = nt->addItem ("cone4_svbu_x", m_h_cone4_num, m_h_cone4_svbu_x);		
  status = nt->addItem ("cone4_svbu_y", m_h_cone4_num, m_h_cone4_svbu_y);		
  status = nt->addItem ("cone4_svbu_z", m_h_cone4_num, m_h_cone4_svbu_z);         

  status = nt->addItem ("cone4_ip2d_pu", m_h_cone4_num, m_h_cone4_ip2d_pu);	   
  status = nt->addItem ("cone4_ip2d_pb", m_h_cone4_num, m_h_cone4_ip2d_pb);	   
  status = nt->addItem ("cone4_ip2d_numTracks", m_h_cone4_num, m_h_cone4_ip2d_numTracks);	   
  status = nt->addItem ("cone4_ip3d_pu", m_h_cone4_num, m_h_cone4_ip3d_pu);	   
  status = nt->addItem ("cone4_ip3d_pb", m_h_cone4_num, m_h_cone4_ip3d_pb);	   
  status = nt->addItem ("cone4_ip3d_numTracks", m_h_cone4_num, m_h_cone4_ip3d_numTracks);	   
  			                          
  status = nt->addItem ("cone4_ipplus_d0", m_h_cone4_num, m_h_cone4_ipplus_d0, max_jettrack_num);	   
  status = nt->addItem ("cone4_ipplus_d0Sig", m_h_cone4_num, m_h_cone4_ipplus_d0Sig, max_jettrack_num);	   
  status = nt->addItem ("cone4_ipplus_z0", m_h_cone4_num, m_h_cone4_ipplus_z0, max_jettrack_num);	   
  status = nt->addItem ("cone4_ipplus_z0Sig", m_h_cone4_num, m_h_cone4_ipplus_z0Sig, max_jettrack_num);	   
  status = nt->addItem ("cone4_ipplus_trackW2D", m_h_cone4_num, m_h_cone4_ipplus_trackW2D, max_jettrack_num);  
  status = nt->addItem ("cone4_ipplus_trackW3D", m_h_cone4_num, m_h_cone4_ipplus_trackW3D, max_jettrack_num);  
  status = nt->addItem ("cone4_ipplus_isFromV0", m_h_cone4_num, m_h_cone4_ipplus_isFromV0, max_jettrack_num);  
  status = nt->addItem ("cone4_ipplus_trackJP", m_h_cone4_num, m_h_cone4_ipplus_trackJP, max_jettrack_num);   
  status = nt->addItem ("cone4_ipplus_trackGrade", m_h_cone4_num, m_h_cone4_ipplus_trackGrade, max_jettrack_num);
  status = nt->addItem ("cone4_ipplus_trackidx", m_h_cone4_num, m_h_cone4_ipplus_trackidx, max_jettrack_num);

  status = nt->addItem ("cone4_lt1d_numTracks", m_h_cone4_num, m_h_cone4_lt1d_numTracks);	   
  status = nt->addItem ("cone4_lt1d_z0", m_h_cone4_num, m_h_cone4_lt1d_z0, max_jettrack_num);	   
  status = nt->addItem ("cone4_lt1d_z0Sig", m_h_cone4_num, m_h_cone4_lt1d_z0Sig, max_jettrack_num);	   
  status = nt->addItem ("cone4_lt2d_numTracks", m_h_cone4_num, m_h_cone4_lt2d_numTracks);	   
  status = nt->addItem ("cone4_lt2d_d0", m_h_cone4_num, m_h_cone4_lt2d_d0, max_jettrack_num);	   
  status = nt->addItem ("cone4_lt2d_d0Sig", m_h_cone4_num, m_h_cone4_lt2d_d0Sig, max_jettrack_num);	  

  status = nt->addItem ("cone4_softmu_numTracks", m_h_cone4_num, m_h_cone4_softmu_numTracks);	   
  status = nt->addItem ("cone4_softmu_d0", m_h_cone4_num, m_h_cone4_softmu_d0, max_jettrack_num);
  status = nt->addItem ("cone4_softmu_ptrel", m_h_cone4_num, m_h_cone4_softmu_ptrel, max_jettrack_num);
  status = nt->addItem ("cone4_softelec_numTracks", m_h_cone4_num, m_h_cone4_softelec_numTracks);	   
  status = nt->addItem ("cone4_softelec_d0", m_h_cone4_num, m_h_cone4_softelec_d0, max_jettrack_num);
  status = nt->addItem ("cone4_softelec_ptrel", m_h_cone4_num, m_h_cone4_softelec_ptrel, max_jettrack_num);
  
  // Kt
  status = nt->addItem ("kt4jet_num", m_h_kt4jet_num, 0,  max_jet_num);
  status = nt->addItem ("kt4jet_pt",m_h_kt4jet_num, m_h_kt4jet_pt);
  status = nt->addItem ("kt4jet_eta",m_h_kt4jet_num, m_h_kt4jet_eta);
  status = nt->addItem ("kt4jet_phi",m_h_kt4jet_num, m_h_kt4jet_phi);

  status = nt->addItem ("kt4jet_lhsig", m_h_kt4jet_num, m_h_kt4jet_lhsig);     
  status = nt->addItem ("kt4jet_w", m_h_kt4jet_num, m_h_kt4jet_w);     
  status = nt->addItem ("kt4jet_wIP2D", m_h_kt4jet_num, m_h_kt4jet_wIP2D); 
  status = nt->addItem ("kt4jet_wIP3D", m_h_kt4jet_num, m_h_kt4jet_wIP3D); 
  status = nt->addItem ("kt4jet_wSV1", m_h_kt4jet_num, m_h_kt4jet_wSV1);  
  status = nt->addItem ("kt4jet_wSV2", m_h_kt4jet_num, m_h_kt4jet_wSV2);  
  status = nt->addItem ("kt4jet_wLT1D", m_h_kt4jet_num, m_h_kt4jet_wLT1D); 
  status = nt->addItem ("kt4jet_wLT2D", m_h_kt4jet_num, m_h_kt4jet_wLT2D); 
  //status = nt->addItem ("kt4jet_wLT3D", m_h_kt4jet_num, m_h_kt4jet_wLT3D);
  status = nt->addItem ("kt4jet_wSVBU", m_h_kt4jet_num, m_h_kt4jet_SVBU);  
  //status = nt->addItem ("kt4jet_wSVTD", m_h_kt4jet_num, m_h_kt4jet_SVTD);
  status = nt->addItem ("kt4jet_wLHSig", m_h_kt4jet_num, m_h_kt4jet_wLHSig);
  status = nt->addItem ("kt4jet_wSoftMu", m_h_kt4jet_num, m_h_kt4jet_wSoftMu);
  status = nt->addItem ("kt4jet_wSoftElec", m_h_kt4jet_num, m_h_kt4jet_wSoftElec);
  status = nt->addItem ("kt4jet_wJP", m_h_kt4jet_num, m_h_kt4jet_wJP);

  status = nt->addItem ("kt4jet_ntracks", m_h_kt4jet_num, m_h_kt4jet_ntracks);
  status = nt->addItem ("kt4jet_trackidx", m_h_kt4jet_num, m_h_kt4jet_trackidx, max_jettrack_num);

  status = nt->addItem ("kt4jet_truth", m_h_kt4jet_num, m_h_kt4jet_truth);

  status = nt->addItem ("kt4jet_sv1_pu", m_h_kt4jet_num, m_h_kt4jet_sv1_pu);		
  status = nt->addItem ("kt4jet_sv1_pb", m_h_kt4jet_num, m_h_kt4jet_sv1_pb);		
  status = nt->addItem ("kt4jet_sv2_pu", m_h_kt4jet_num, m_h_kt4jet_sv2_pu);		
  status = nt->addItem ("kt4jet_sv2_pb", m_h_kt4jet_num, m_h_kt4jet_sv2_pb);		
  status = nt->addItem ("kt4jet_svplus_n2t", m_h_kt4jet_num, m_h_kt4jet_svplus_n2t); 	
  status = nt->addItem ("kt4jet_svplus_eFrac", m_h_kt4jet_num, m_h_kt4jet_svplus_eFrac);	
  status = nt->addItem ("kt4jet_svplus_mass", m_h_kt4jet_num, m_h_kt4jet_svplus_mass);	
  status = nt->addItem ("kt4jet_svplus_normDist", m_h_kt4jet_num, m_h_kt4jet_svplus_normDist);
  status = nt->addItem ("kt4jet_svplus_NGTracks", m_h_kt4jet_num, m_h_kt4jet_svplus_NGTracks);
  status = nt->addItem ("kt4jet_svplus_chi2", m_h_kt4jet_num, m_h_kt4jet_svplus_chi2);	
  status = nt->addItem ("kt4jet_svplus_ndf", m_h_kt4jet_num, m_h_kt4jet_svplus_ndf);	
  status = nt->addItem ("kt4jet_svplus_x", m_h_kt4jet_num, m_h_kt4jet_svplus_x);	
  status = nt->addItem ("kt4jet_svplus_y", m_h_kt4jet_num, m_h_kt4jet_svplus_y);	
  status = nt->addItem ("kt4jet_svplus_z", m_h_kt4jet_num, m_h_kt4jet_svplus_z);	
  status = nt->addItem ("kt4jet_svplus_ntracks", m_h_kt4jet_num, m_h_kt4jet_svplus_ntracks);	   
  status = nt->addItem ("kt4jet_svplus_trackidx", m_h_kt4jet_num, m_h_kt4jet_svplus_trackidx, max_jettrack_num);

  status = nt->addItem ("kt4jet_svbu_pu", m_h_kt4jet_num, m_h_kt4jet_svbu_pu);	
  status = nt->addItem ("kt4jet_svbu_pb", m_h_kt4jet_num, m_h_kt4jet_svbu_pb);	
  status = nt->addItem ("kt4jet_svbu_eFrac", m_h_kt4jet_num, m_h_kt4jet_svbu_eFrac);	
  status = nt->addItem ("kt4jet_svbu_mass", m_h_kt4jet_num, m_h_kt4jet_svbu_mass);	
  status = nt->addItem ("kt4jet_svbu_normDist", m_h_kt4jet_num, m_h_kt4jet_svbu_normDist);	
  status = nt->addItem ("kt4jet_svbu_mult", m_h_kt4jet_num, m_h_kt4jet_svbu_mult);	
  status = nt->addItem ("kt4jet_svbu_chi2", m_h_kt4jet_num, m_h_kt4jet_svbu_chi2);	
  status = nt->addItem ("kt4jet_svbu_ndf", m_h_kt4jet_num, m_h_kt4jet_svbu_ndf);	
  status = nt->addItem ("kt4jet_svbu_x", m_h_kt4jet_num, m_h_kt4jet_svbu_x);		
  status = nt->addItem ("kt4jet_svbu_y", m_h_kt4jet_num, m_h_kt4jet_svbu_y);		
  status = nt->addItem ("kt4jet_svbu_z", m_h_kt4jet_num, m_h_kt4jet_svbu_z);         

  status = nt->addItem ("kt4jet_ip2d_pu", m_h_kt4jet_num, m_h_kt4jet_ip2d_pu);	   
  status = nt->addItem ("kt4jet_ip2d_pb", m_h_kt4jet_num, m_h_kt4jet_ip2d_pb);	   
  status = nt->addItem ("kt4jet_ip2d_numTracks", m_h_kt4jet_num, m_h_kt4jet_ip2d_numTracks);	   
  status = nt->addItem ("kt4jet_ip3d_pu", m_h_kt4jet_num, m_h_kt4jet_ip3d_pu);	   
  status = nt->addItem ("kt4jet_ip3d_pb", m_h_kt4jet_num, m_h_kt4jet_ip3d_pb);	   
  status = nt->addItem ("kt4jet_ip3d_numTracks", m_h_kt4jet_num, m_h_kt4jet_ip3d_numTracks);	   
  			                          
  status = nt->addItem ("kt4jet_ipplus_d0", m_h_kt4jet_num, m_h_kt4jet_ipplus_d0, max_jettrack_num);	   
  status = nt->addItem ("kt4jet_ipplus_d0Sig", m_h_kt4jet_num, m_h_kt4jet_ipplus_d0Sig, max_jettrack_num);	   
  status = nt->addItem ("kt4jet_ipplus_z0", m_h_kt4jet_num, m_h_kt4jet_ipplus_z0, max_jettrack_num);	   
  status = nt->addItem ("kt4jet_ipplus_z0Sig", m_h_kt4jet_num, m_h_kt4jet_ipplus_z0Sig, max_jettrack_num);	   
  status = nt->addItem ("kt4jet_ipplus_trackW2D", m_h_kt4jet_num, m_h_kt4jet_ipplus_trackW2D, max_jettrack_num);  
  status = nt->addItem ("kt4jet_ipplus_trackW3D", m_h_kt4jet_num, m_h_kt4jet_ipplus_trackW3D, max_jettrack_num);  
  status = nt->addItem ("kt4jet_ipplus_isFromV0", m_h_kt4jet_num, m_h_kt4jet_ipplus_isFromV0, max_jettrack_num);  
  status = nt->addItem ("kt4jet_ipplus_trackJP", m_h_kt4jet_num, m_h_kt4jet_ipplus_trackJP, max_jettrack_num);   
  status = nt->addItem ("kt4jet_ipplus_trackGrade", m_h_kt4jet_num, m_h_kt4jet_ipplus_trackGrade, max_jettrack_num);
  status = nt->addItem ("kt4jet_ipplus_trackidx", m_h_kt4jet_num, m_h_kt4jet_ipplus_trackidx, max_jettrack_num);

  status = nt->addItem ("kt4jet_lt1d_numTracks", m_h_kt4jet_num, m_h_kt4jet_lt1d_numTracks);	   
  status = nt->addItem ("kt4jet_lt1d_z0", m_h_kt4jet_num, m_h_kt4jet_lt1d_z0, max_jettrack_num);	   
  status = nt->addItem ("kt4jet_lt1d_z0Sig", m_h_kt4jet_num, m_h_kt4jet_lt1d_z0Sig, max_jettrack_num);	   
  status = nt->addItem ("kt4jet_lt2d_numTracks", m_h_kt4jet_num, m_h_kt4jet_lt2d_numTracks);	   
  status = nt->addItem ("kt4jet_lt2d_d0", m_h_kt4jet_num, m_h_kt4jet_lt2d_d0, max_jettrack_num);	   
  status = nt->addItem ("kt4jet_lt2d_d0Sig", m_h_kt4jet_num, m_h_kt4jet_lt2d_d0Sig, max_jettrack_num);	  

  status = nt->addItem ("kt4jet_softmu_numTracks", m_h_kt4jet_num, m_h_kt4jet_softmu_numTracks);	   
  status = nt->addItem ("kt4jet_softmu_d0", m_h_kt4jet_num, m_h_kt4jet_softmu_d0, max_jettrack_num);
  status = nt->addItem ("kt4jet_softmu_ptrel", m_h_kt4jet_num, m_h_kt4jet_softmu_ptrel, max_jettrack_num);
  status = nt->addItem ("kt4jet_softelec_numTracks", m_h_kt4jet_num, m_h_kt4jet_softelec_numTracks);	   
  status = nt->addItem ("kt4jet_softelec_d0", m_h_kt4jet_num, m_h_kt4jet_softelec_d0, max_jettrack_num);
  status = nt->addItem ("kt4jet_softelec_ptrel", m_h_kt4jet_num, m_h_kt4jet_softelec_ptrel, max_jettrack_num);

  return status;
}  

StatusCode AODtoLiverpoolNtuple::BookTruthParticle(NTuplePtr nt)
{
  MsgStream mLog( messageService(), name() );
  StatusCode status = StatusCode::SUCCESS;

  // Truth Particle
  status = nt->addItem ("truth_num", m_h_truth_num, 0,  max_truth_num);
  status = nt->addItem ("truth_pt",m_h_truth_num, m_h_truth_pt);
  status = nt->addItem ("truth_eta",m_h_truth_num, m_h_truth_eta);
  status = nt->addItem ("truth_phi",m_h_truth_num, m_h_truth_phi);
  status = nt->addItem ("truth_e",m_h_truth_num, m_h_truth_e);
  status = nt->addItem ("truth_pdgId",m_h_truth_num, m_h_truth_pdgId);
  status = nt->addItem ("truth_Barcode",m_h_truth_num, m_h_truth_Barcode);
  status = nt->addItem ("truth_Status",m_h_truth_num, m_h_truth_Status);
  status = nt->addItem ("truth_etisolation_cone10",m_h_truth_num, m_h_truth_etisolation_cone10);
  status = nt->addItem ("truth_etisolation_cone20",m_h_truth_num, m_h_truth_etisolation_cone20);
  status = nt->addItem ("truth_etisolation_cone30",m_h_truth_num, m_h_truth_etisolation_cone30);
  status = nt->addItem ("truth_etisolation_cone40",m_h_truth_num, m_h_truth_etisolation_cone40);
  status = nt->addItem ("truth_etisolation_cone45",m_h_truth_num, m_h_truth_etisolation_cone45);
  status = nt->addItem ("truth_etisolation_cone50",m_h_truth_num, m_h_truth_etisolation_cone50);
  status = nt->addItem ("truth_etisolation_cone60",m_h_truth_num, m_h_truth_etisolation_cone60);
  status = nt->addItem ("truth_etisolation_cone70",m_h_truth_num, m_h_truth_etisolation_cone70);
  //  status = nt->addItem ("truth_NbrOfCones",m_h_truth_num, m_h_truth_NbrOfCones);

  status = nt->addItem ("truth_hasProductionVertex",m_h_truth_num, m_h_truth_hasProductionVertex);
  status = nt->addItem ("truth_productionVertex_x",m_h_truth_num, m_h_truth_productionVertex_x);
  status = nt->addItem ("truth_productionVertex_y",m_h_truth_num, m_h_truth_productionVertex_y);
  status = nt->addItem ("truth_productionVertex_z",m_h_truth_num, m_h_truth_productionVertex_z);
  status = nt->addItem ("truth_productionVertex_t",m_h_truth_num, m_h_truth_productionVertex_t);
  status = nt->addItem ("truth_productionVertex_perp",m_h_truth_num, m_h_truth_productionVertex_perp);
  status = nt->addItem ("truth_productionVertex_eta",m_h_truth_num, m_h_truth_productionVertex_eta);
  status = nt->addItem ("truth_productionVertex_phi",m_h_truth_num, m_h_truth_productionVertex_phi);
  status = nt->addItem ("truth_productionVertex_r",m_h_truth_num, m_h_truth_productionVertex_r);

  status = nt->addItem ("truth_hasDecayVertex",m_h_truth_num, m_h_truth_hasDecayVertex);
  status = nt->addItem ("truth_decayVertex_x",m_h_truth_num, m_h_truth_decayVertex_x);
  status = nt->addItem ("truth_decayVertex_y",m_h_truth_num, m_h_truth_decayVertex_y);
  status = nt->addItem ("truth_decayVertex_z",m_h_truth_num, m_h_truth_decayVertex_z);
  status = nt->addItem ("truth_decayVertex_t",m_h_truth_num, m_h_truth_decayVertex_t);
  status = nt->addItem ("truth_decayVertex_perp",m_h_truth_num, m_h_truth_decayVertex_perp);
  status = nt->addItem ("truth_decayVertex_eta",m_h_truth_num, m_h_truth_decayVertex_eta);
  status = nt->addItem ("truth_decayVertex_phi",m_h_truth_num, m_h_truth_decayVertex_phi);
  status = nt->addItem ("truth_decayVertex_r",m_h_truth_num, m_h_truth_decayVertex_r);

  status = nt->addItem ("truth_child_num",m_h_truth_num, m_h_truth_child_num);
  status = nt->addItem ("truth_child_Barcode",m_h_truth_num, m_h_truth_child_Barcode, 200);
  status = nt->addItem ("truth_child_pdgId",m_h_truth_num, m_h_truth_child_pdgId, 200);
  status = nt->addItem ("truth_parent_num",m_h_truth_num, m_h_truth_parent_num);
  status = nt->addItem ("truth_parent_Barcode",m_h_truth_num, m_h_truth_parent_Barcode, 200);
  status = nt->addItem ("truth_parent_pdgId",m_h_truth_num, m_h_truth_parent_pdgId, 200);

  return status;
}

StatusCode AODtoLiverpoolNtuple::BookTruthJet(NTuplePtr nt)
{
  MsgStream mLog( messageService(), name() );
  StatusCode status = StatusCode::SUCCESS;

  // Truth Jet

  status = nt->addItem ("cone_true_num", m_h_cone_true_num, 0,  max_jet_num);
  status = nt->addItem ("cone_true_pt",m_h_cone_true_num, m_h_cone_true_pt);
  status = nt->addItem ("cone_true_eta",m_h_cone_true_num, m_h_cone_true_eta);
  status = nt->addItem ("cone_true_phi",m_h_cone_true_num, m_h_cone_true_phi);

  status = nt->addItem ("cone4_true_num", m_h_cone4_true_num, 0,  max_jet_num);
  status = nt->addItem ("cone4_true_pt",m_h_cone4_true_num, m_h_cone4_true_pt);
  status = nt->addItem ("cone4_true_eta",m_h_cone4_true_num, m_h_cone4_true_eta);
  status = nt->addItem ("cone4_true_phi",m_h_cone4_true_num, m_h_cone4_true_phi);

  status = nt->addItem ("kt4jet_true_num", m_h_kt4jet_true_num, 0,  max_jet_num);
  status = nt->addItem ("kt4jet_true_pt",m_h_kt4jet_true_num, m_h_kt4jet_true_pt);
  status = nt->addItem ("kt4jet_true_eta",m_h_kt4jet_true_num, m_h_kt4jet_true_eta);
  status = nt->addItem ("kt4jet_true_phi",m_h_kt4jet_true_num, m_h_kt4jet_true_phi);
  
  return status;
}    

StatusCode AODtoLiverpoolNtuple::BookTruthEtMiss(NTuplePtr nt)
{
  MsgStream mLog( messageService(), name() );
  StatusCode status = StatusCode::SUCCESS;

  // EtMiss Truth
  status = nt->addItem ("etmiss_true_sumet", m_h_etmiss_true_sumet);
  status = nt->addItem ("etmiss_true_ex"   , m_h_etmiss_true_ex);       
  status = nt->addItem ("etmiss_true_ey"   , m_h_etmiss_true_ey);      
  status = nt->addItem ("etmiss_true_phi"   , m_h_etmiss_true_phi);    
  status = nt->addItem ("etmiss_true_et"   , m_h_etmiss_true_et);      

  return status;
}  

StatusCode AODtoLiverpoolNtuple::BookPrimaryVertex(NTuplePtr nt)
{
  MsgStream mLog( messageService(), name() );
  StatusCode status = StatusCode::SUCCESS;
  
  // Primary Vertex
  status = nt->addItem ("primaryvtx_num", m_h_primaryvtx_num, 0,  100);
  status = nt->addItem ("primaryvtx_x" , m_h_primaryvtx_num, m_h_primaryvtx_x);     
  status = nt->addItem ("primaryvtx_y" , m_h_primaryvtx_num, m_h_primaryvtx_y);
  status = nt->addItem ("primaryvtx_z" , m_h_primaryvtx_num, m_h_primaryvtx_z);
  status = nt->addItem ("primaryvtx_dX" , m_h_primaryvtx_num, m_h_primaryvtx_dX);
  status = nt->addItem ("primaryvtx_dY" , m_h_primaryvtx_num, m_h_primaryvtx_dY);
  status = nt->addItem ("primaryvtx_dZ" , m_h_primaryvtx_num, m_h_primaryvtx_dZ);
  status = nt->addItem ("primaryvtx_chi2" , m_h_primaryvtx_num, m_h_primaryvtx_chi2);
  status = nt->addItem ("primaryvtx_ndf" , m_h_primaryvtx_num , m_h_primaryvtx_ndf);
  
  return status;
}  
	 
StatusCode AODtoLiverpoolNtuple::finalize() 
{
  MsgStream mLog( messageService(), name() );
  mLog << MSG::DEBUG << "finalize()" << endreq;
  return StatusCode::SUCCESS;

  // Delete any memory allocation on heap (i.e. with new)
}

StatusCode AODtoLiverpoolNtuple::execute() {
  MsgStream mLog( messageService(), name() );
  mLog << MSG::DEBUG << "execute()" << endreq;
  StatusCode status = StatusCode::SUCCESS;

  // version
  m_h_version_h = 1.3;
  m_h_version_c = 1.8;

  // event
  const EventInfo* eventInfo;
  status = m_storeGate->retrieve(eventInfo);
  if (status.isFailure()) {
    mLog << MSG::WARNING << "Could not retrieve event info" << endreq;
  }
  
  mLog << MSG::DEBUG << "Event info succesfully retrieved" << endreq;
  EventID* eventID = eventInfo->event_ID();
  if (eventID) {
    m_h_event_runnum   = eventID->run_number();
    m_h_event_eventnum = eventID->event_number();
  } else {
    m_h_event_runnum   = -999;
    m_h_event_eventnum = -999;
  }

  status = UnpackElectron();         
  status = UnpackPhoton();
  status = UnpackMuidMuons();
  status = UnpackStacoMuons();
  // used by anyone?
  // status = UnpackCTP_Decision();
  status = UnpackTriggerDecision();
  if ( m_doElectronTrigger )
    {
      status = UnpackEFegamma();
      status = UnpackTrigElectron();
    }
  status = UnpackL1EMTauObject();
  status = UnpackL1JetObject();
  status = UnpackL1EtmissObject();
  status = UnpackEtMiss();
  status = UnpackEtMissRefFinal();//ED
  status = UnpackEtMissLocHadTopoObj();//ED
  status = UnpackEtMissObjFinal();//ED
  status = UnpackTauJet();
  status = UnpackTauJet1p3p();
  status = UnpackTrack();
  status = UnpackJet();
  status = UnpackTruthParticle();
  status = UnpackTruthJet();
  status = UnpackTruthEtMiss();
  status = UnpackPrimaryVertex();

  mLog << MSG::DEBUG << "Writing ntuple record" << endreq;
  if (!(ntupleSvc()->writeRecord("/NTUPLES/FILE1/Ntuple1").isSuccess())){
    mLog << MSG::DEBUG << "problems writing ntuple record" << endreq;
  } else {
    mLog << MSG::DEBUG << "writing ntuple record" << endreq;
  }

  mLog << MSG::DEBUG << "execute() succeeded" << endreq;
  return StatusCode::SUCCESS;
}

StatusCode AODtoLiverpoolNtuple::UnpackElectron()
  //
  // https://twiki.cern.ch/twiki/bin/view/Atlas/CaloEvolution1200Persist
  //
{
  MsgStream mLog(messageService(), name());
  StatusCode status = StatusCode::SUCCESS;
  mLog << MSG::INFO << "Starting UnpackElectron" << endreq; 

  // Electron
  const ElectronContainer* elecTES;
  if (m_fastsim == 1) {
    status=m_storeGate->retrieve( elecTES, "AtlfastElectronCollection");
    if ( status.isFailure()  ||  !elecTES ) {
      mLog << MSG::WARNING << "No Atlfast AOD electron container found in TDS" << endreq; 
      return StatusCode::SUCCESS;
    }
  } else {
    status=m_storeGate->retrieve( elecTES, "ElectronCollection");
    if( status.isFailure()  ||  !elecTES ) {
      mLog << MSG::WARNING << "No AOD electron container found in TDS" << endreq; 
      return StatusCode::SUCCESS;
    }
  }

  mLog << MSG::DEBUG << "Electrons successfully retrieved" << endreq;

  ElectronContainer::const_iterator elecItr  = (*elecTES).begin();
  ElectronContainer::const_iterator elecItrE = (*elecTES).end();   

  // Electron
  int Nelec = 0;
  for (; elecItr != elecItrE; ++elecItr) {
    Nelec++ ;

    if (Nelec < max_electron_num) {

      m_h_electron_pt[Nelec-1]         = (*elecItr)->pt();
      m_h_electron_eta[Nelec-1]        = (*elecItr)->eta();
      m_h_electron_phi[Nelec-1]        = (*elecItr)->phi();
      m_h_electron_e[Nelec-1]          = (*elecItr)->e(); 
      if (m_fastsim == 1) m_h_electron_track_charge[Nelec-1]= (*elecItr)->charge();
      
      // for Atlfast data we are done now
      if (m_fastsim == 0) {

	m_h_electron_hasTrack[Nelec-1]  = (*elecItr)->hasTrack();
	if (m_h_electron_hasTrack[Nelec-1]) {
	  const Rec::TrackParticle *elTrack = (*elecItr)->track();
	  if (elTrack != 0) {
	    m_h_electron_track_pt[Nelec-1]            = elTrack->pt();
	    m_h_electron_track_eta[Nelec-1]           = elTrack->eta();
	    m_h_electron_track_phi[Nelec-1]           = elTrack->phi();
	    m_h_electron_track_e[Nelec-1]             = elTrack->e();
	    m_h_electron_track_d0[Nelec-1]            = elTrack->measuredPerigee()->parameters()[Trk::d0];
	    m_h_electron_track_z0[Nelec-1]            = elTrack->measuredPerigee()->parameters()[Trk::z0];   
	    m_h_electron_track_charge[Nelec-1]        = elTrack->charge();
	  }
	}
	  
	m_h_electron_numberOfBLayerHits[Nelec-1]           = (*elecItr)->numberOfBLayerHits();   
	m_h_electron_numberOfPixelHits[Nelec-1]            = (*elecItr)->numberOfPixelHits();  
	m_h_electron_numberOfSCTHits[Nelec-1]              = (*elecItr)->numberOfSCTHits();    
	m_h_electron_numberOfTRTHits[Nelec-1]              = (*elecItr)->numberOfTRTHits();   
	m_h_electron_numberOfTRTHighThresholdHits[Nelec-1] = (*elecItr)->numberOfTRTHighThresholdHits();         
	  
	// Electron algorithm
	m_h_electron_egamma[Nelec-1]              = (*elecItr)->author(ElectronParameters::AuthorEgamma);        
	m_h_electron_softe[Nelec-1]               = (*elecItr)->author(ElectronParameters::AuthorSofte);        
	// EgammaPID parameters
	m_h_electron_IsEM[Nelec-1]                = (*elecItr)->electronID(egammaPID::IsEM);
	m_h_electron_ElectronWeight[Nelec-1]      = (*elecItr)->electronID(egammaPID::ElectronWeight);     
	m_h_electron_BgWeight[Nelec-1]            = (*elecItr)->electronID(egammaPID::BgWeight);     
	m_h_electron_NeuralNet[Nelec-1]           = (*elecItr)->electronID(egammaPID::NeuralNet);     
	m_h_electron_Hmatrix[Nelec-1]             = (*elecItr)->electronID(egammaPID::Hmatrix);     
	m_h_electron_SofteIsEM[Nelec-1]           = (*elecItr)->electronID(egammaPID::SofteIsEM);
	m_h_electron_SofteElectronWeight[Nelec-1] = (*elecItr)->electronID(egammaPID::SofteElectronWeight);     
	m_h_electron_SofteBgWeight[Nelec-1]       = (*elecItr)->electronID(egammaPID::SofteBgWeight);     
	m_h_electron_SofteNeuralNet[Nelec-1]      = (*elecItr)->electronID(egammaPID::SofteNeuralNet);     
	  
	// Electron Parameters
	m_h_electron_EoverP[Nelec-1]        = (*elecItr)->parameter(ElectronParameters::EoverP);
	m_h_electron_etcone[Nelec-1]        = (*elecItr)->parameter(ElectronParameters::etcone);  
	m_h_electron_etcone20[Nelec-1]      = (*elecItr)->parameter(ElectronParameters::etcone20);
	m_h_electron_etcone30[Nelec-1]      = (*elecItr)->parameter(ElectronParameters::etcone30);
	m_h_electron_etcone40[Nelec-1]      = (*elecItr)->parameter(ElectronParameters::etcone40);
	  
	// for egamma electrons 
	m_h_electron_etaBE2[Nelec-1]        = (*elecItr)->parameter(ElectronParameters::etaBE2);
	m_h_electron_et37[Nelec-1]          = (*elecItr)->parameter(ElectronParameters::et37);
	m_h_electron_e233[Nelec-1]          = (*elecItr)->parameter(ElectronParameters::e233);
	m_h_electron_e237[Nelec-1]          = (*elecItr)->parameter(ElectronParameters::e237);
	m_h_electron_e277[Nelec-1]          = (*elecItr)->parameter(ElectronParameters::e277);
	m_h_electron_ethad1[Nelec-1]        = (*elecItr)->parameter(ElectronParameters::ethad1);
	m_h_electron_weta1[Nelec-1]         = (*elecItr)->parameter(ElectronParameters::weta1);
	m_h_electron_weta2[Nelec-1]         = (*elecItr)->parameter(ElectronParameters::weta2);
	m_h_electron_f1[Nelec-1]            = (*elecItr)->parameter(ElectronParameters::f1);
	m_h_electron_e2tsts1[Nelec-1]       = (*elecItr)->parameter(ElectronParameters::e2tsts1);
	m_h_electron_emins1[Nelec-1]        = (*elecItr)->parameter(ElectronParameters::emins1);
	m_h_electron_wtots1[Nelec-1]        = (*elecItr)->parameter(ElectronParameters::wtots1);
	m_h_electron_fracs1[Nelec-1]        = (*elecItr)->parameter(ElectronParameters::fracs1);
	  
	// for softe electrons
	m_h_electron_EtaCorrMag[Nelec-1]    = (*elecItr)->parameter(ElectronParameters::EtaCorrMag);
	m_h_electron_f1core[Nelec-1]        = (*elecItr)->parameter(ElectronParameters::f1core);
	m_h_electron_f3core[Nelec-1]        = (*elecItr)->parameter(ElectronParameters::f3core);
	m_h_electron_pos7[Nelec-1]          = (*elecItr)->parameter(ElectronParameters::pos7);
	m_h_electron_iso[Nelec-1]           = (*elecItr)->parameter(ElectronParameters::iso);
	m_h_electron_widths2[Nelec-1]       = (*elecItr)->parameter(ElectronParameters::widths2);
      }							
    }
  }								 
  
  m_h_electron_num = Nelec < max_electron_num ? Nelec : max_electron_num;
  mLog << MSG::INFO << "Electrons added to Ntuple. " << endreq;

  // for Atlfast data we are done now
  if (m_fastsim == 1) {
    m_h_larn = 0; // no LAr clusters in Altfast data
    m_h_emtopon = 0; // no emtopo clusters in Atlfast data 
    m_h_calocaltopon = 0; // no calocaltopo clusters in Atlfast data 
    return StatusCode::SUCCESS;
  }    

  // LArEM
  const CaloClusterContainer* larTES;
  status = m_storeGate->retrieve( larTES, "LArClusterEM");
  if (status.isFailure()  ||  !larTES) {
    mLog << MSG::WARNING << "No AOD larcluster container found in TDS" << endreq; 
    return StatusCode::SUCCESS;
  }  
  mLog << MSG::DEBUG << "LArClusters successfully retrieved" << endreq;

  CaloClusterContainer::const_iterator larItr  = (*larTES).begin();
  CaloClusterContainer::const_iterator larItrE = (*larTES).end();   

  int Nlar = 0;
  for (; larItr != larItrE; ++larItr) {
    Nlar++ ;     

    if (Nlar < max_lar_num) {
      m_h_larnsamples[Nlar-1] = (*larItr)->nSamples();      
      m_h_larinBarrel[Nlar-1] = (*larItr)->inBarrel();
      m_h_larinEndcap[Nlar-1] = (*larItr)->inEndcap();      
      m_h_lar_time[Nlar-1]    = (*larItr)->getTime();      
      m_h_larEMB0E[Nlar-1]    = (*larItr)->eSample(CaloSampling::PreSamplerB);
      m_h_larEMB1E[Nlar-1]    = (*larItr)->eSample(CaloSampling::EMB1);	  
      m_h_larEMB2E[Nlar-1]    = (*larItr)->eSample(CaloSampling::EMB2);	  
      m_h_larEMB3E[Nlar-1]    = (*larItr)->eSample(CaloSampling::EMB3);	  

      m_h_larEMB0Eta[Nlar-1] = (*larItr)->etaSample(CaloSampling::PreSamplerB);
      m_h_larEMB1Eta[Nlar-1] = (*larItr)->etaSample(CaloSampling::EMB1);	  
      m_h_larEMB2Eta[Nlar-1] = (*larItr)->etaSample(CaloSampling::EMB2);	  
      m_h_larEMB3Eta[Nlar-1] = (*larItr)->etaSample(CaloSampling::EMB3);	  

      m_h_larEMB0Phi[Nlar-1] = (*larItr)->phiSample(CaloSampling::PreSamplerB);
      m_h_larEMB1Phi[Nlar-1] = (*larItr)->phiSample(CaloSampling::EMB1);	  
      m_h_larEMB2Phi[Nlar-1] = (*larItr)->phiSample(CaloSampling::EMB2);	  
      m_h_larEMB3Phi[Nlar-1] = (*larItr)->phiSample(CaloSampling::EMB3);	  

      m_h_larEME0E[Nlar-1]   = (*larItr)->eSample(CaloSampling::PreSamplerE);	  
      m_h_larEME1E[Nlar-1]   = (*larItr)->eSample(CaloSampling::EME1);	  
      m_h_larEME2E[Nlar-1]   = (*larItr)->eSample(CaloSampling::EME2);	  
      m_h_larEME3E[Nlar-1]   = (*larItr)->eSample(CaloSampling::EME3);	  

      m_h_larEME0Eta[Nlar-1] = (*larItr)->etaSample(CaloSampling::PreSamplerE);	  
      m_h_larEME1Eta[Nlar-1] = (*larItr)->etaSample(CaloSampling::EME1);	  
      m_h_larEME2Eta[Nlar-1] = (*larItr)->etaSample(CaloSampling::EME2);	  
      m_h_larEME3Eta[Nlar-1] = (*larItr)->etaSample(CaloSampling::EME3);	  

      m_h_larEME0Phi[Nlar-1] = (*larItr)->phiSample(CaloSampling::PreSamplerE);	  
      m_h_larEME1Phi[Nlar-1] = (*larItr)->phiSample(CaloSampling::EME1);	  
      m_h_larEME2Phi[Nlar-1] = (*larItr)->phiSample(CaloSampling::EME2);	  
      m_h_larEME3Phi[Nlar-1] = (*larItr)->phiSample(CaloSampling::EME3);	  

      m_h_lareta[Nlar-1]     = (*larItr)->eta();
      m_h_larphi[Nlar-1]     = (*larItr)->phi();
      m_h_larenergy[Nlar-1]  = (*larItr)->e();
    }  
  }

  m_h_larn = Nlar < max_lar_num ? Nlar : max_lar_num;
  mLog << MSG::INFO << "LArClusters added to Ntuple. " << endreq;
    

  // EMTopo  
  const CaloClusterContainer* emtopoTES;
  status = m_storeGate->retrieve(emtopoTES, "EMTopoCluster");
  if( status.isFailure()  ||  !emtopoTES ) {
    mLog << MSG::WARNING << "No AOD EMTopo cluster container found in TDS" << endreq; 
    return StatusCode::SUCCESS;
  }  

  mLog << MSG::DEBUG << "EMTopo Clusters successfully retrieved" << endreq;

  CaloClusterContainer::const_iterator emtopoItr  = (*emtopoTES).begin();
  CaloClusterContainer::const_iterator emtopoItrE = (*emtopoTES).end();   

  int Nemtopo = 0;
  for (; emtopoItr != emtopoItrE; ++emtopoItr){
    Nemtopo++ ;     

    if(Nemtopo < max_emtopo_num) {
      m_h_emtoponsamples[Nemtopo-1] = (*emtopoItr)->nSamples();      
      m_h_emtopoinBarrel[Nemtopo-1] = (*emtopoItr)->inBarrel();
      m_h_emtopoinEndcap[Nemtopo-1] = (*emtopoItr)->inEndcap();
    
      m_h_emtopo_time[Nemtopo-1]     = (*emtopoItr)->getTime();
      m_h_emtopoeta[Nemtopo-1]     = (*emtopoItr)->eta();
      m_h_emtopophi[Nemtopo-1]     = (*emtopoItr)->phi();
      m_h_emtopoenergy[Nemtopo-1]  = (*emtopoItr)->e();

      //      m_h_emtopo_first_phi[Nemtopo-1]      = (*emtopoItr)->getMomentValue(CaloClusterMoment::FIRST_PHI);
      //      m_h_emtopo_first_eta[Nemtopo-1]      = (*emtopoItr)->getMomentValue(CaloClusterMoment::FIRST_ETA);
      //      m_h_emtopo_second_r[Nemtopo-1]       = (*emtopoItr)->getMomentValue(CaloClusterMoment::SECOND_R);
      //      m_h_emtopo_second_lambda[Nemtopo-1]  = (*emtopoItr)->getMomentValue(CaloClusterMoment::SECOND_LAMBDA);
      //      m_h_emtopo_center_lambda[Nemtopo-1]  = (*emtopoItr)->getMomentValue(CaloClusterMoment::CENTER_LAMBDA);
      //      m_h_emtopo_eng_frac_em[Nemtopo-1] = (*emtopoItr)->getMomentValue(CaloClusterMoment::ENG_FRAC_EM);
      //      m_h_emtopo_first_eng_dens[Nemtopo-1] = (*emtopoItr)->getMomentValue(CaloClusterMoment::FIRTS_ENG_DENS);

    } 
  }

  m_h_emtopon = Nemtopo < max_emtopo_num ? Nemtopo : max_emtopo_num;
  mLog << MSG::INFO << "EMTopo Clusters added to Ntuple. " << endreq;

  // CaloCalTopo  
  const CaloClusterContainer* calocaltopoTES;
  status = m_storeGate->retrieve( calocaltopoTES, "CaloCalTopoCluster");
  if(status.isFailure()  ||  !calocaltopoTES) {
    mLog << MSG::WARNING << "No AOD CaloCalTopo cluster container found in TDS" << endreq; 
    return StatusCode::SUCCESS;
  }  

  mLog << MSG::DEBUG << "CaloCalTopo Clusters successfully retrieved" << endreq;

  CaloClusterContainer::const_iterator calocaltopoItr  = (*calocaltopoTES).begin();
  CaloClusterContainer::const_iterator calocaltopoItrE = (*calocaltopoTES).end();   

  int Ncalocaltopo = 0;
  for (; calocaltopoItr != calocaltopoItrE; ++calocaltopoItr) {
    Ncalocaltopo++ ;     

    if (Ncalocaltopo < max_calocaltopo_num) {
      m_h_calocaltoponsamples[Ncalocaltopo-1] = (*calocaltopoItr)->nSamples();      
      m_h_calocaltopoinBarrel[Ncalocaltopo-1] = (*calocaltopoItr)->inBarrel();
      m_h_calocaltopoinEndcap[Ncalocaltopo-1] = (*calocaltopoItr)->inEndcap();
    
      m_h_calocaltopo_time[Ncalocaltopo-1]     = (*calocaltopoItr)->getTime();
      m_h_calocaltopoeta[Ncalocaltopo-1]     = (*calocaltopoItr)->eta();
      m_h_calocaltopophi[Ncalocaltopo-1]     = (*calocaltopoItr)->phi();
      m_h_calocaltopoenergy[Ncalocaltopo-1]  = (*calocaltopoItr)->e();

      //      m_h_calocaltopo_first_phi[Ncalocaltopo-1]      = (*calocaltopoItr)->getMomentValue(CaloClusterMoment::FIRST_PHI);
      //      m_h_calocaltopo_first_eta[Ncalocaltopo-1]      = (*calocaltopoItr)->getMomentValue(CaloClusterMoment::FIRST_ETA);
      //      m_h_calocaltopo_second_r[Ncalocaltopo-1]       = (*calocaltopoItr)->getMomentValue(CaloClusterMoment::SECOND_R);
      //      m_h_calocaltopo_second_lambda[Ncalocaltopo-1]  = (*calocaltopoItr)->getMomentValue(CaloClusterMoment::SECOND_LAMBDA);
      //      m_h_calocaltopo_center_lambda[Ncalocaltopo-1]  = (*calocaltopoItr)->getMomentValue(CaloClusterMoment::CENTER_LAMBDA);
      //      m_h_calocaltopo_eng_frac_em[Ncalocaltopo-1] = (*calocaltopoItr)->getMomentValue(CaloClusterMoment::ENG_FRAC_EM);
    } 
  }

  m_h_calocaltopon = Ncalocaltopo < max_calocaltopo_num ? Ncalocaltopo : max_calocaltopo_num;
  mLog << MSG::INFO << "CaloCalTopo Clusters added to Ntuple. " << endreq;

  return StatusCode::SUCCESS;
}

StatusCode AODtoLiverpoolNtuple::UnpackPhoton()
{
  MsgStream mLog(messageService(), name());
  StatusCode status = StatusCode::SUCCESS;

  const PhotonContainer* photonTES;
  if (m_fastsim == 1) {
    status = m_storeGate->retrieve( photonTES, "AtlfastPhotonCollection");
    if (status.isFailure()  ||  !photonTES) {
      mLog << MSG::WARNING << "No Atlfast AOD Photon container found in TDS" << endreq;
      return StatusCode::SUCCESS;
    }
  } else {
    status = m_storeGate->retrieve( photonTES, "PhotonCollection");
    if (status.isFailure()  ||  !photonTES) {
      mLog << MSG::WARNING << "No AOD Photon container found in TDS" << endreq;
      return StatusCode::SUCCESS;
    }
  }
  
  mLog << MSG::DEBUG << "Photons successfully retrieved" << endreq;

  PhotonContainer::const_iterator photonItr  = (*photonTES).begin();
  PhotonContainer::const_iterator photonItrE = (*photonTES).end();

  // Photon
  int Nphoton = 0;
  for (; photonItr != photonItrE; ++photonItr) {
    Nphoton++ ;

    if(Nphoton < max_photon_num) {

      m_h_photon_pt[Nphoton-1]         = (*photonItr)->pt();
      m_h_photon_eta[Nphoton-1]        = (*photonItr)->eta();
      m_h_photon_phi[Nphoton-1]        = (*photonItr)->phi();
      m_h_photon_e[Nphoton-1]          = (*photonItr)->e(); 

      // for Atlfast data we are done now
      if (m_fastsim == 0) {
	  
	// Photon algorithm
	m_h_photon_egamma[Nphoton-1]              = (*photonItr)->author(PhotonParameters::AuthorEgamma);        
	//m_h_photon_softe[Nphoton-1]               = (*photonItr)->author(PhotonParameters::AuthorSofte);        
	// EgammaPID parameters
	m_h_photon_IsEM[Nphoton-1]                = (*photonItr)->photonID(egammaPID::IsEM);
	m_h_photon_ElectronWeight[Nphoton-1]      = (*photonItr)->photonID(egammaPID::ElectronWeight);     
	m_h_photon_BgWeight[Nphoton-1]            = (*photonItr)->photonID(egammaPID::BgWeight);     
	m_h_photon_NeuralNet[Nphoton-1]           = (*photonItr)->photonID(egammaPID::NeuralNet);     
	m_h_photon_SofteIsEM[Nphoton-1]           = (*photonItr)->photonID(egammaPID::SofteIsEM);
	m_h_photon_SofteElectronWeight[Nphoton-1] = (*photonItr)->photonID(egammaPID::SofteElectronWeight);     
	m_h_photon_SofteBgWeight[Nphoton-1]       = (*photonItr)->photonID(egammaPID::SofteBgWeight);     
	m_h_photon_SofteNeuralNet[Nphoton-1]      = (*photonItr)->photonID(egammaPID::SofteNeuralNet);     
	  
	// Photon Parameters
	m_h_photon_etcone[Nphoton-1]        = (*photonItr)->parameter(PhotonParameters::etcone);  
	m_h_photon_etcone20[Nphoton-1]      = (*photonItr)->parameter(PhotonParameters::etcone20);
	m_h_photon_etcone30[Nphoton-1]      = (*photonItr)->parameter(PhotonParameters::etcone30);
	m_h_photon_etcone40[Nphoton-1]      = (*photonItr)->parameter(PhotonParameters::etcone40); 
	m_h_photon_etaBE2[Nphoton-1]        = (*photonItr)->parameter(PhotonParameters::etaBE2);
	m_h_photon_et37[Nphoton-1]          = (*photonItr)->parameter(PhotonParameters::et37);
	m_h_photon_e233[Nphoton-1]          = (*photonItr)->parameter(PhotonParameters::e233);
	m_h_photon_e237[Nphoton-1]          = (*photonItr)->parameter(PhotonParameters::e237);
	m_h_photon_e277[Nphoton-1]          = (*photonItr)->parameter(PhotonParameters::e277);
	m_h_photon_ethad1[Nphoton-1]        = (*photonItr)->parameter(PhotonParameters::ethad1);
	m_h_photon_weta1[Nphoton-1]         = (*photonItr)->parameter(PhotonParameters::weta1);
	m_h_photon_weta2[Nphoton-1]         = (*photonItr)->parameter(PhotonParameters::weta2);
	m_h_photon_f1[Nphoton-1]            = (*photonItr)->parameter(PhotonParameters::f1);
	m_h_photon_e2tsts1[Nphoton-1]       = (*photonItr)->parameter(PhotonParameters::e2tsts1);
	m_h_photon_emins1[Nphoton-1]        = (*photonItr)->parameter(PhotonParameters::emins1);
	m_h_photon_wtots1[Nphoton-1]        = (*photonItr)->parameter(PhotonParameters::wtots1);
	m_h_photon_fracs1[Nphoton-1]        = (*photonItr)->parameter(PhotonParameters::fracs1);
	
	m_h_photon_conversionAngleMatch[Nphoton-1]         = (*photonItr)->conversionAngleMatch();
	m_h_photon_conversionTrackMatch[Nphoton-1]         = (*photonItr)->conversionTrackMatch();


	const Trk::RecVertex *convVx = (*photonItr)->conversionVx();
	if (convVx != 0) {
	  m_h_photon_conversionVx_x[Nphoton-1] = convVx->position().x();
	  m_h_photon_conversionVx_y[Nphoton-1] = convVx->position().y();
	  m_h_photon_conversionVx_z[Nphoton-1] = convVx->position().z();
	}	  

	const Rec::TrackParticle *convTrack1 = (*photonItr)->track(0);
	if (convTrack1 != 0) {
	  m_h_photon_conversiontrack1_pt[Nphoton-1]           = convTrack1->pt();
	  m_h_photon_conversiontrack1_eta[Nphoton-1]          = convTrack1->eta();
	  m_h_photon_conversiontrack1_phi[Nphoton-1]          = convTrack1->phi();
	  m_h_photon_conversiontrack1_e[Nphoton-1]            = convTrack1->e();
	  m_h_photon_conversiontrack1_charge[Nphoton-1]       = convTrack1->charge();
	}

	const Rec::TrackParticle *convTrack2 = (*photonItr)->track(1);
	if (convTrack2 != 0) {
	  m_h_photon_conversiontrack2_pt[Nphoton-1]           = convTrack2->pt();
	  m_h_photon_conversiontrack2_eta[Nphoton-1]          = convTrack2->eta();
	  m_h_photon_conversiontrack2_phi[Nphoton-1]          = convTrack2->phi();
	  m_h_photon_conversiontrack2_e[Nphoton-1]            = convTrack2->e();
	  m_h_photon_conversiontrack2_charge[Nphoton-1]       = convTrack2->charge();
	}
      }							
    }
  }								 

  m_h_photon_num = Nphoton < max_photon_num ? Nphoton : max_photon_num;
  mLog << MSG::INFO << "Photons added to Ntuple. " << endreq;

  return StatusCode::SUCCESS;
}

StatusCode AODtoLiverpoolNtuple::UnpackMuidMuons()
{
  MsgStream mLog(messageService(), name());
  StatusCode status = StatusCode::SUCCESS;

  const MuonContainer* muonTES;
  // we use the muid bank to store Atlfast muon data 
  if (m_fastsim == 1) {
    status = m_storeGate->retrieve( muonTES, "AtlfastMuonCollection");
    if (status.isFailure()  ||  !muonTES) {
      mLog << MSG::WARNING << "No Atlfast AOD muon container found in TDS" << endreq;
      return StatusCode::SUCCESS;
    }
  } else {  
    status = m_storeGate->retrieve( muonTES, "MuidMuonCollection");
    if (status.isFailure()  ||  !muonTES) {
      mLog << MSG::WARNING << "No AOD MUID muon container found in TDS" << endreq;
      return StatusCode::SUCCESS;
    }
  }
  
  mLog << MSG::DEBUG << "Muons successfully retrieved" << endreq;

  MuonContainer::const_iterator muonItr  = (*muonTES).begin();
  MuonContainer::const_iterator muonItrE = (*muonTES).end();

  // Muon
  int Nmuon = 0;
  for (; muonItr != muonItrE; ++muonItr) {   
    Nmuon++ ;
    if(Nmuon <  max_muidmuon_num) {
      m_h_muidmuon_pt[Nmuon-1]         = (*muonItr)->pt();
      m_h_muidmuon_eta[Nmuon-1]        = (*muonItr)->eta();
      m_h_muidmuon_phi[Nmuon-1]        = (*muonItr)->phi();
      m_h_muidmuon_e[Nmuon-1]          = (*muonItr)->e(); 
      if (m_fastsim == 1) m_h_muidmuon_track_charge[Nmuon-1]      = (*muonItr)->charge();
 
      if (m_fastsim == 0) {
        const Rec::TrackParticle *muTrack = (*muonItr)->track();

	if (muTrack != 0) {
	  m_h_muidmuon_track_pt[Nmuon-1]          = muTrack->pt();
	  m_h_muidmuon_track_eta[Nmuon-1]         = muTrack->eta();
	  m_h_muidmuon_track_phi[Nmuon-1]         = muTrack->phi();
	  m_h_muidmuon_track_e[Nmuon-1]           = muTrack->e();
	  m_h_muidmuon_track_d0[Nmuon-1]          = muTrack->measuredPerigee()->parameters()[Trk::d0];
	  m_h_muidmuon_track_z0[Nmuon-1]          = muTrack->measuredPerigee()->parameters()[Trk::z0];   
	  m_h_muidmuon_track_charge[Nmuon-1]      = muTrack->charge();
	}
	
	m_h_muidmuon_numberOfBLayerHits[Nmuon-1]  = (*muonItr)->numberOfBLayerHits();   
	m_h_muidmuon_numberOfPixelHits[Nmuon-1]   = (*muonItr)->numberOfPixelHits();  
	m_h_muidmuon_numberOfSCTHits[Nmuon-1]     = (*muonItr)->numberOfSCTHits();    
	m_h_muidmuon_numberOfTRTHits[Nmuon-1]     = (*muonItr)->numberOfTRTHits();   
	m_h_muidmuon_numberOfTRTHighThresholdHits[Nmuon-1] = (*muonItr)->numberOfTRTHighThresholdHits();         	  
  
	// this doesn't work for now due to a bug in MuonIDEvent/Muon.cxx 
  	//m_h_muidmuon_numberOfMDTHits[Nmuon-1]     = (*muonItr)->numberOfMDTHits();         
	//m_h_muidmuon_numberOfCSCEtaHits[Nmuon-1]  = (*muonItr)->numberOfCSCEtaHits();         
	//m_h_muidmuon_numberOfCSCPhiHits[Nmuon-1]  = (*muonItr)->numberOfCSCPhiHits();         
	//m_h_muidmuon_numberOfRPCEtaHits[Nmuon-1]  = (*muonItr)->numberOfRPCEtaHits();         
	//m_h_muidmuon_numberOfRPCPhiHits[Nmuon-1]  = (*muonItr)->numberOfRPCPhiHits();         
	//m_h_muidmuon_numberOfTGCEtaHits[Nmuon-1]  = (*muonItr)->numberOfTGCEtaHits();         
	//m_h_muidmuon_numberOfTGCPhiHits[Nmuon-1]  = (*muonItr)->numberOfTGCPhiHits();         
	// temporary fix 
	if ( (*muonItr)->hasCombinedMuonTrackParticle() )
	  {
	    const Rec::TrackParticle *muTrack = (*muonItr)->combinedMuonTrackParticle();
	    m_h_muidmuon_numberOfMDTHits[Nmuon-1]     = muTrack->trackSummary()->get( Trk::numberOfMdtHits );
	    m_h_muidmuon_numberOfCSCEtaHits[Nmuon-1]  = muTrack->trackSummary()->get( Trk::numberOfCscEtaHits );         
	    m_h_muidmuon_numberOfCSCPhiHits[Nmuon-1]  = muTrack->trackSummary()->get( Trk::numberOfCscPhiHits );      
	    m_h_muidmuon_numberOfRPCEtaHits[Nmuon-1]  = muTrack->trackSummary()->get( Trk::numberOfRpcEtaHits );         
	    m_h_muidmuon_numberOfRPCPhiHits[Nmuon-1]  = muTrack->trackSummary()->get( Trk::numberOfRpcPhiHits );         
	    m_h_muidmuon_numberOfTGCEtaHits[Nmuon-1]  = muTrack->trackSummary()->get( Trk::numberOfTgcEtaHits );         
	    m_h_muidmuon_numberOfTGCPhiHits[Nmuon-1]  = muTrack->trackSummary()->get( Trk::numberOfTgcPhiHits );         
	  } 
	else if ( (*muonItr)->hasMuonExtrapolatedTrackParticle() )
	  {
	    const Rec::TrackParticle *muTrack = (*muonItr)->muonExtrapolatedTrackParticle();
	    m_h_muidmuon_numberOfMDTHits[Nmuon-1]     = muTrack->trackSummary()->get( Trk::numberOfMdtHits );
	    m_h_muidmuon_numberOfCSCEtaHits[Nmuon-1]  = muTrack->trackSummary()->get( Trk::numberOfCscEtaHits );         
	    m_h_muidmuon_numberOfCSCPhiHits[Nmuon-1]  = muTrack->trackSummary()->get( Trk::numberOfCscPhiHits );      
	    m_h_muidmuon_numberOfRPCEtaHits[Nmuon-1]  = muTrack->trackSummary()->get( Trk::numberOfRpcEtaHits );         
	    m_h_muidmuon_numberOfRPCPhiHits[Nmuon-1]  = muTrack->trackSummary()->get( Trk::numberOfRpcPhiHits );         
	    m_h_muidmuon_numberOfTGCEtaHits[Nmuon-1]  = muTrack->trackSummary()->get( Trk::numberOfTgcEtaHits );         
	    m_h_muidmuon_numberOfTGCPhiHits[Nmuon-1]  = muTrack->trackSummary()->get( Trk::numberOfTgcPhiHits );         
	  } 
	else if ( (*muonItr)->hasMuonSpectrometerTrackParticle() )
	  {
	    const Rec::TrackParticle *muTrack = (*muonItr)->muonSpectrometerTrackParticle ();
	    m_h_muidmuon_numberOfMDTHits[Nmuon-1]     = muTrack->trackSummary()->get( Trk::numberOfMdtHits );
	    m_h_muidmuon_numberOfCSCEtaHits[Nmuon-1]  = muTrack->trackSummary()->get( Trk::numberOfCscEtaHits );         
	    m_h_muidmuon_numberOfCSCPhiHits[Nmuon-1]  = muTrack->trackSummary()->get( Trk::numberOfCscPhiHits );      
	    m_h_muidmuon_numberOfRPCEtaHits[Nmuon-1]  = muTrack->trackSummary()->get( Trk::numberOfRpcEtaHits );         
	    m_h_muidmuon_numberOfRPCPhiHits[Nmuon-1]  = muTrack->trackSummary()->get( Trk::numberOfRpcPhiHits );         
	    m_h_muidmuon_numberOfTGCEtaHits[Nmuon-1]  = muTrack->trackSummary()->get( Trk::numberOfTgcEtaHits );         
	    m_h_muidmuon_numberOfTGCPhiHits[Nmuon-1]  = muTrack->trackSummary()->get( Trk::numberOfTgcPhiHits );         
	  }
	else if ( (*muonItr)->hasInDetTrackParticle() )
	  {
	    if ( (*muonItr)->associatedEtaDigits().size() > 0)
	      {
		m_h_muidmuon_numberOfMDTHits[Nmuon-1]     = (*muonItr)->associatedEtaDigits()[0];
		m_h_muidmuon_numberOfCSCEtaHits[Nmuon-1]  = (*muonItr)->associatedEtaDigits()[1];         
		m_h_muidmuon_numberOfRPCEtaHits[Nmuon-1]  = (*muonItr)->associatedEtaDigits()[2];        
		m_h_muidmuon_numberOfTGCEtaHits[Nmuon-1]  = (*muonItr)->associatedEtaDigits()[3];         
	      } 
	    else
	      {
		m_h_muidmuon_numberOfMDTHits[Nmuon-1]     = -1;
		m_h_muidmuon_numberOfCSCEtaHits[Nmuon-1]  = -1;         
		m_h_muidmuon_numberOfRPCEtaHits[Nmuon-1]  = -1;        
		m_h_muidmuon_numberOfTGCEtaHits[Nmuon-1]  = -1;         
	      }
	    if ( (*muonItr)->associatedPhiDigits().size() > 0)
	      {
		m_h_muidmuon_numberOfCSCPhiHits[Nmuon-1]  = (*muonItr)->associatedPhiDigits()[1];      
		m_h_muidmuon_numberOfRPCPhiHits[Nmuon-1]  = (*muonItr)->associatedPhiDigits()[2];        
		m_h_muidmuon_numberOfTGCPhiHits[Nmuon-1]  = (*muonItr)->associatedPhiDigits()[3];
	      } 
	    else
	      {
		m_h_muidmuon_numberOfCSCPhiHits[Nmuon-1]  = -1;      
		m_h_muidmuon_numberOfRPCPhiHits[Nmuon-1]  = -1;        
		m_h_muidmuon_numberOfTGCPhiHits[Nmuon-1]  = -1;
	      }

	  }
	else 
	  {
	    m_h_muidmuon_numberOfMDTHits[Nmuon-1]     = -1;
	    m_h_muidmuon_numberOfCSCEtaHits[Nmuon-1]  = -1;         
	    m_h_muidmuon_numberOfCSCPhiHits[Nmuon-1]  = -1;      
	    m_h_muidmuon_numberOfRPCEtaHits[Nmuon-1]  = -1;        
	    m_h_muidmuon_numberOfRPCPhiHits[Nmuon-1]  = -1;        
	    m_h_muidmuon_numberOfTGCEtaHits[Nmuon-1]  = -1;         
	    m_h_muidmuon_numberOfTGCPhiHits[Nmuon-1]  = -1;
	  }
	// end of temporary patch
	

	m_h_muidmuon_etcone10[Nmuon-1]            = (*muonItr)->parameter(MuonParameters::etcone10);  
	m_h_muidmuon_etcone20[Nmuon-1]            = (*muonItr)->parameter(MuonParameters::etcone20);  
	m_h_muidmuon_etcone30[Nmuon-1]            = (*muonItr)->parameter(MuonParameters::etcone30);  
	m_h_muidmuon_etcone40[Nmuon-1]            = (*muonItr)->parameter(MuonParameters::etcone40);  
	m_h_muidmuon_etcone[Nmuon-1]              = (*muonItr)->parameter(MuonParameters::etcone);  
	m_h_muidmuon_etcone50[Nmuon-1]            = (*muonItr)->parameter(MuonParameters::etcone50);  
	m_h_muidmuon_etcone60[Nmuon-1]            = (*muonItr)->parameter(MuonParameters::etcone60);  
	m_h_muidmuon_etcone70[Nmuon-1]            = (*muonItr)->parameter(MuonParameters::etcone70);  
	m_h_muidmuon_nucone10[Nmuon-1]            = (*muonItr)->parameter(MuonParameters::nucone10);  
	m_h_muidmuon_nucone20[Nmuon-1]            = (*muonItr)->parameter(MuonParameters::nucone20);  
	m_h_muidmuon_nucone30[Nmuon-1]            = (*muonItr)->parameter(MuonParameters::nucone30);  
	m_h_muidmuon_nucone40[Nmuon-1]            = (*muonItr)->parameter(MuonParameters::nucone40);  
	m_h_muidmuon_nucone[Nmuon-1]              = (*muonItr)->parameter(MuonParameters::nucone);  
	m_h_muidmuon_nucone50[Nmuon-1]            = (*muonItr)->parameter(MuonParameters::nucone50);  
	m_h_muidmuon_nucone60[Nmuon-1]            = (*muonItr)->parameter(MuonParameters::nucone60);  
	m_h_muidmuon_nucone70[Nmuon-1]            = (*muonItr)->parameter(MuonParameters::nucone70);  


	m_h_muidmuon_matchChi2OverDoF[Nmuon-1]    = (*muonItr)->matchChi2OverDoF(); 
	m_h_muidmuon_fitChi2OverDoF[Nmuon-1]      = (*muonItr)->fitChi2OverDoF();
	//	const CaloEnergy *ELoss = (*muonItr)->caloEnergyLoss();
	//	if (ELoss != 0)
	//	{
	//	  m_h_muidmuon_energyLoss_Deposit[Nmuon-1]        = ELoss.energyDeposit(); 
	//	  m_h_muidmuon_energyLoss_Sigma[Nmuon-1]          = ELoss.energySigma();
	//	} 
	m_h_muidmuon_author[Nmuon-1]                    = (*muonItr)->author();
	m_h_muidmuon_isCombinedMuon[Nmuon-1]            = (*muonItr)->isCombinedMuon();
	m_h_muidmuon_isStandAloneMuon[Nmuon-1]          = (*muonItr)->isStandAloneMuon();
	m_h_muidmuon_isLowPtReconstructedMuon[Nmuon-1]  = (*muonItr)->isLowPtReconstructedMuon();
	m_h_muidmuon_hasCombinedMuon[Nmuon-1]           = (*muonItr)->hasCombinedMuon();
	m_h_muidmuon_hasInDetTrackParticle[Nmuon-1]     = (*muonItr)->hasInDetTrackParticle();
	m_h_muidmuon_hasMuonSpectrometerTrackParticle[Nmuon-1]        = (*muonItr)->hasMuonSpectrometerTrackParticle();
	m_h_muidmuon_hasMuonExtrapolatedTrackParticle[Nmuon-1]        = (*muonItr)->hasMuonExtrapolatedTrackParticle();
	m_h_muidmuon_hasCombinedMuonTrackParticle[Nmuon-1]            = (*muonItr)->hasCombinedMuonTrackParticle();
	m_h_muidmuon_hasCluster[Nmuon-1]                = (*muonItr)->hasCluster();
	m_h_muidmuon_isHighPt[Nmuon-1]                  = (*muonItr)->isHighPt(); 
	m_h_muidmuon_isLowPt[Nmuon-1]                   = (*muonItr)->isLowPt(); 
      }
    }
  }

  m_h_muidmuon_num =  Nmuon < max_muidmuon_num ? Nmuon : max_muidmuon_num;
  mLog << MSG::INFO << "Muons added to Ntuple. " << endreq;

  return StatusCode::SUCCESS;
}

StatusCode AODtoLiverpoolNtuple::UnpackStacoMuons()
{
  MsgStream mLog(messageService(), name());
  StatusCode status = StatusCode::SUCCESS;

  // for Atlfast data nothing to do here
  if (m_fastsim == 1) {
    m_h_stacomuon_num = 0; // no Staco muons in Altfast data
    return StatusCode::SUCCESS;
  }    
      
  const MuonContainer* stacoTES;
  status = m_storeGate->retrieve( stacoTES, "StacoMuonCollection");
  if (status.isFailure()  ||  !stacoTES) {
    mLog << MSG::WARNING << "No AOD MUID muon container found in TDS" << endreq;
    return StatusCode::SUCCESS;
  }

  mLog << MSG::DEBUG << "Muons successfully retrieved" << endreq;

  MuonContainer::const_iterator stacoItr  = (*stacoTES).begin();
  MuonContainer::const_iterator stacoItrE = (*stacoTES).end();

  // Muon
  int Nmuon = 0;
  for (; stacoItr != stacoItrE; ++stacoItr) {   
    Nmuon++ ;

    if (Nmuon <  max_stacomuon_num) {
      m_h_stacomuon_pt[Nmuon-1]                       = (*stacoItr)->pt();
      m_h_stacomuon_eta[Nmuon-1]                      = (*stacoItr)->eta();
      m_h_stacomuon_phi[Nmuon-1]                      = (*stacoItr)->phi();
      m_h_stacomuon_e[Nmuon-1]                        = (*stacoItr)->e(); 
       
      const Rec::TrackParticle *muTrack = (*stacoItr)->track();
      
      if (muTrack != 0) {
	m_h_stacomuon_track_pt[Nmuon-1]                = muTrack->pt();
	m_h_stacomuon_track_eta[Nmuon-1]               = muTrack->eta();
	m_h_stacomuon_track_phi[Nmuon-1]               = muTrack->phi();
	m_h_stacomuon_track_e[Nmuon-1]                 = muTrack->e();
	m_h_stacomuon_track_d0[Nmuon-1]                = muTrack->measuredPerigee()->parameters()[Trk::d0];
	m_h_stacomuon_track_z0[Nmuon-1]                = muTrack->measuredPerigee()->parameters()[Trk::z0];   
	m_h_stacomuon_track_charge[Nmuon-1]            = muTrack->charge();
      }
	
      m_h_stacomuon_numberOfBLayerHits[Nmuon-1]        = (*stacoItr)->numberOfBLayerHits();   
      m_h_stacomuon_numberOfPixelHits[Nmuon-1]         = (*stacoItr)->numberOfPixelHits();  
      m_h_stacomuon_numberOfSCTHits[Nmuon-1]           = (*stacoItr)->numberOfSCTHits();    
      m_h_stacomuon_numberOfTRTHits[Nmuon-1]           = (*stacoItr)->numberOfTRTHits();   
      m_h_stacomuon_numberOfTRTHighThresholdHits[Nmuon-1] = (*stacoItr)->numberOfTRTHighThresholdHits();         	  
      m_h_stacomuon_numberOfMDTHits[Nmuon-1]           = (*stacoItr)->numberOfMDTHits();         
      m_h_stacomuon_numberOfCSCEtaHits[Nmuon-1]        = (*stacoItr)->numberOfCSCEtaHits();         
      m_h_stacomuon_numberOfCSCPhiHits[Nmuon-1]        = (*stacoItr)->numberOfCSCPhiHits();         
      m_h_stacomuon_numberOfRPCEtaHits[Nmuon-1]        = (*stacoItr)->numberOfRPCEtaHits();         
      m_h_stacomuon_numberOfRPCPhiHits[Nmuon-1]        = (*stacoItr)->numberOfRPCPhiHits();         
      m_h_stacomuon_numberOfTGCEtaHits[Nmuon-1]        = (*stacoItr)->numberOfTGCEtaHits();         
      m_h_stacomuon_numberOfTGCPhiHits[Nmuon-1]        = (*stacoItr)->numberOfTGCPhiHits();         

      m_h_stacomuon_etcone10[Nmuon-1]                  = (*stacoItr)->parameter(MuonParameters::etcone10);  
      m_h_stacomuon_etcone20[Nmuon-1]                  = (*stacoItr)->parameter(MuonParameters::etcone20);  
      m_h_stacomuon_etcone30[Nmuon-1]                  = (*stacoItr)->parameter(MuonParameters::etcone30);  
      m_h_stacomuon_etcone40[Nmuon-1]                  = (*stacoItr)->parameter(MuonParameters::etcone40);  
      m_h_stacomuon_etcone[Nmuon-1]                    = (*stacoItr)->parameter(MuonParameters::etcone);  
      m_h_stacomuon_etcone50[Nmuon-1]                  = (*stacoItr)->parameter(MuonParameters::etcone50);  
      m_h_stacomuon_etcone60[Nmuon-1]                  = (*stacoItr)->parameter(MuonParameters::etcone60);  
      m_h_stacomuon_etcone70[Nmuon-1]                  = (*stacoItr)->parameter(MuonParameters::etcone70);  
      m_h_stacomuon_nucone10[Nmuon-1]                  = (*stacoItr)->parameter(MuonParameters::nucone10);  
      m_h_stacomuon_nucone20[Nmuon-1]                  = (*stacoItr)->parameter(MuonParameters::nucone20);  
      m_h_stacomuon_nucone30[Nmuon-1]                  = (*stacoItr)->parameter(MuonParameters::nucone30);  
      m_h_stacomuon_nucone40[Nmuon-1]                  = (*stacoItr)->parameter(MuonParameters::nucone40);  
      m_h_stacomuon_nucone[Nmuon-1]                    = (*stacoItr)->parameter(MuonParameters::nucone);  
      m_h_stacomuon_nucone50[Nmuon-1]                  = (*stacoItr)->parameter(MuonParameters::nucone50);  
      m_h_stacomuon_nucone60[Nmuon-1]                  = (*stacoItr)->parameter(MuonParameters::nucone60);  
      m_h_stacomuon_nucone70[Nmuon-1]                  = (*stacoItr)->parameter(MuonParameters::nucone70);  

      m_h_stacomuon_matchChi2OverDoF[Nmuon-1]          = (*stacoItr)->matchChi2OverDoF(); 
      m_h_stacomuon_fitChi2OverDoF[Nmuon-1]            = (*stacoItr)->fitChi2OverDoF();
      m_h_stacomuon_author[Nmuon-1]                    = (*stacoItr)->author();
      m_h_stacomuon_isCombinedMuon[Nmuon-1]            = (*stacoItr)->isCombinedMuon();
      m_h_stacomuon_isStandAloneMuon[Nmuon-1]          = (*stacoItr)->isStandAloneMuon();
      m_h_stacomuon_isLowPtReconstructedMuon[Nmuon-1]  = (*stacoItr)->isLowPtReconstructedMuon();
      m_h_stacomuon_hasCombinedMuon[Nmuon-1]           = (*stacoItr)->hasCombinedMuon();
      m_h_stacomuon_hasInDetTrackParticle[Nmuon-1]     = (*stacoItr)->hasInDetTrackParticle();
      m_h_stacomuon_hasMuonSpectrometerTrackParticle[Nmuon-1]        = (*stacoItr)->hasMuonSpectrometerTrackParticle();
      m_h_stacomuon_hasMuonExtrapolatedTrackParticle[Nmuon-1]        = (*stacoItr)->hasMuonExtrapolatedTrackParticle();
      m_h_stacomuon_hasCombinedMuonTrackParticle[Nmuon-1]            = (*stacoItr)->hasCombinedMuonTrackParticle();
      m_h_stacomuon_hasCluster[Nmuon-1]                = (*stacoItr)->hasCluster();
      m_h_stacomuon_isHighPt[Nmuon-1]                  = (*stacoItr)->isHighPt(); 
      m_h_stacomuon_isLowPt[Nmuon-1]                   = (*stacoItr)->isLowPt(); 
    }
  }

  m_h_stacomuon_num =  Nmuon < max_stacomuon_num ? Nmuon : max_stacomuon_num;
  mLog << MSG::INFO << "Muons added to Ntuple. " << endreq;

  return StatusCode::SUCCESS;
}




StatusCode AODtoLiverpoolNtuple::UnpackCTP_Decision()
{
  MsgStream mLog(messageService(), name());
  StatusCode status = StatusCode::SUCCESS;

  if (m_fastsim == 1)
    {
      return StatusCode::SUCCESS;
    }

  const CTP_Decision *ctpdecision;
  status=m_storeGate->retrieve(ctpdecision , "CTP_Decision");
  if( status.isFailure()  ||  !ctpdecision ) 
    {
      mLog << MSG::WARNING << "No AOD CTP_Decision found in TDS" << endreq;
      return StatusCode::SUCCESS;
    }
   
   
  mLog << MSG::DEBUG << "CTP_Decision successfully retrieved" << endreq;

  m_h_ctpdecision_word0 = ctpdecision->getWord0();
  m_h_ctpdecision_word1 = ctpdecision->getWord1();
  m_h_ctpdecision_word2 = ctpdecision->getWord2();

  mLog << MSG::INFO << "CTP_Decision added to Ntuple. " << endreq;


  return StatusCode::SUCCESS;
}







StatusCode AODtoLiverpoolNtuple::UnpackTriggerDecision()
  // --------------
  // See:
  // https://twiki.cern.ch/twiki/bin/view/Atlas/TriggerAnalysisTutorial
  // https://twiki.cern.ch/twiki/bin/view/Atlas/TriggerMenuVersions
  //--------------
{
  MsgStream mLog(messageService(), name());
  StatusCode status = StatusCode::SUCCESS;

  if (m_fastsim == 1)
    {
      return StatusCode::SUCCESS;
    }

  const TriggerDecision *triggerdecision;
  status=m_storeGate->retrieve(triggerdecision , "MyTriggerDecision");
  if( status.isFailure()  ||  !triggerdecision ) 
    {
      mLog << MSG::WARNING << "No AOD TriggerDecision found in TDS" << endreq;
      return StatusCode::SUCCESS;
    }
      
  mLog << MSG::DEBUG << "TriggerDecision successfully retrieved" << endreq;
  // for 12.0.31
  //   m_h_passed_L1 = triggerdecision->isDefinedL1() ? triggerdecision->isPassedL1() : -999;
  //   m_h_passed_L2 = triggerdecision->isDefinedL2() ? triggerdecision->isPassedL2() : -999;
  //   m_h_passed_EF = triggerdecision->isDefinedEF() ? triggerdecision->isPassedEF() : -999;
  //   m_h_L1_EM01   = triggerdecision->isDefined("iEM01") ? triggerdecision->isTriggered("iEM01") : -999;
  //   m_h_L1_2EM15I  = triggerdecision->isDefined("iL1_2EM15I") ? triggerdecision->isTriggered("iL1_2EM15I") : -999; 	
  //   m_h_L1_EM25I  = triggerdecision->isDefined("iL1_EM25I") ? triggerdecision->isTriggered("iL1_EM25I") : -999;
  //   m_h_L1_EM60 	 = triggerdecision->isDefined("iL1_EM60") ? triggerdecision->isTriggered("iL1_EM60") : -999;
  //   m_h_L1_HA07   = triggerdecision->isDefined("iHA01") ? triggerdecision->isTriggered("iHA01") : -999;
  //   m_h_L1_HA10I   = triggerdecision->isDefined("iL1_HA10I") ? triggerdecision->isTriggered("iL1_HA10I") : -999;
  //   m_h_L1_HA15I   = triggerdecision->isDefined("iL1_HA15I") ? triggerdecision->isTriggered("iL1_HA15I") : -999;
  //   m_h_L1_HA35I   = triggerdecision->isDefined("iL1_HA35I") ? triggerdecision->isTriggered("iL1_HA35I") : -999;
  //   m_h_L1_MU06   = triggerdecision->isDefined("iMU06") ? triggerdecision->isTriggered("iMU06") : -999;
  //   m_h_L1_2MU06  = triggerdecision->isDefined("iL1_2MU6") ? triggerdecision->isTriggered("iL1_2MU6") : -999;
  //   m_h_L1_MU08   = triggerdecision->isDefined("iL1_MU8") ? triggerdecision->isTriggered("iL1_MU8") : -999;
  //   m_h_L1_MU10   = triggerdecision->isDefined("iL1_MU10") ? triggerdecision->isTriggered("iL1_MU10") : -999;
  //   m_h_L1_MU11   = triggerdecision->isDefined("iL1_MU11") ? triggerdecision->isTriggered("iL1_MU11") : -999;
  //   m_h_L1_MU20   = triggerdecision->isDefined("iL1_MU20") ? triggerdecision->isTriggered("iL1_MU20") : -999;
  //   m_h_L1_MU40   = triggerdecision->isDefined("iL1_MU40") ? triggerdecision->isTriggered("iL1_MU40") : -999;


  m_h_L1_EM5    = triggerdecision->isDefined("L1_EM5") ? triggerdecision->isTriggered("L1_EM5") : -999; 	
  m_h_L1_2EM15  = triggerdecision->isDefined("L1_2EM15") ? triggerdecision->isTriggered("L1_2EM15") : -999; 	
  m_h_L1_2EM15I = triggerdecision->isDefined("L1_2EM15I") ? triggerdecision->isTriggered("L1_2EM15I") : -999; 
  m_h_L1_EM25   = triggerdecision->isDefined("L1_EM25") ? triggerdecision->isTriggered("L1_EM25") : -999;	
  m_h_L1_EM25I  = triggerdecision->isDefined("L1_EM25I") ? triggerdecision->isTriggered("L1_EM25I") : -999;
  m_h_L1_EM60 	 = triggerdecision->isDefined("L1_EM60") ? triggerdecision->isTriggered("L1_EM60") : -999;
  m_h_L1_TAU05  = triggerdecision->isDefined("L1_TAU05") ? triggerdecision->isTriggered("L1_TAU05") : -999;
  m_h_L1_TAU10  = triggerdecision->isDefined("L1_TAU10") ? triggerdecision->isTriggered("L1_TAU10") : -999;
  m_h_L1_TAU10I = triggerdecision->isDefined("L1_TAU10I") ? triggerdecision->isTriggered("L1_TAU10I") : -999;
  m_h_L1_TAU15  = triggerdecision->isDefined("L1_TAU15") ? triggerdecision->isTriggered("L1_TAU15") : -999;
  m_h_L1_TAU15I = triggerdecision->isDefined("L1_TAU15I") ? triggerdecision->isTriggered("L1_TAU15I") : -999;
  m_h_L1_TAU20I = triggerdecision->isDefined("L1_TAU20I") ? triggerdecision->isTriggered("L1_TAU20I") : -999;
  m_h_L1_TAU25I = triggerdecision->isDefined("L1_TAU25I") ? triggerdecision->isTriggered("L1_TAU25I") : -999;
  m_h_L1_TAU35I = triggerdecision->isDefined("L1_TAU35I") ? triggerdecision->isTriggered("L1_TAU35I") : -999;
  m_h_L1_XE20   = triggerdecision->isDefined("L1_XE20") ? triggerdecision->isTriggered("L1_XE20") : -999;
  m_h_L1_XE30   = triggerdecision->isDefined("L1_XE30") ? triggerdecision->isTriggered("L1_XE30") : -999;
  m_h_L1_XE40   = triggerdecision->isDefined("L1_XE40") ? triggerdecision->isTriggered("L1_XE40") : -999;
  m_h_L1_XE50   = triggerdecision->isDefined("L1_XE50") ? triggerdecision->isTriggered("L1_XE50") : -999;
  m_h_L1_XE100  = triggerdecision->isDefined("L1_XE100") ? triggerdecision->isTriggered("L1_XE100") : -999;
  m_h_L1_XE200  = triggerdecision->isDefined("L1_XE200") ? triggerdecision->isTriggered("L1_XE200") : -999;
  m_h_L1_MU06   = triggerdecision->isDefined("L1_MU06") ? triggerdecision->isTriggered("L1_MU06") : -999;
  m_h_L1_2MU06  = triggerdecision->isDefined("L1_2MU06") ? triggerdecision->isTriggered("L1_2MU06") : -999;
  m_h_L1_MU08   = triggerdecision->isDefined("L1_MU08") ? triggerdecision->isTriggered("L1_MU08") : -999;
  m_h_L1_MU10   = triggerdecision->isDefined("L1_MU10") ? triggerdecision->isTriggered("L1_MU10") : -999;
  m_h_L1_MU11   = triggerdecision->isDefined("L1_MU11") ? triggerdecision->isTriggered("L1_MU11") : -999;
  m_h_L1_MU20   = triggerdecision->isDefined("L1_MU20") ? triggerdecision->isTriggered("L1_MU20") : -999;
  m_h_L1_MU40   = triggerdecision->isDefined("L1_MU40") ? triggerdecision->isTriggered("L1_MU40") : -999;
  m_h_L1_J35    = triggerdecision->isDefined("L1_J35") ? triggerdecision->isTriggered("L1_J35") : -999;
  m_h_L1_J45    = triggerdecision->isDefined("L1_J45") ? triggerdecision->isTriggered("L1_J45") : -999;
  m_h_L1_2J45   = triggerdecision->isDefined("L1_2J45") ? triggerdecision->isTriggered("L1_2J45") : -999;
  m_h_L1_3J45   = triggerdecision->isDefined("L1_3J45") ? triggerdecision->isTriggered("L1_3J45") : -999;
  m_h_L1_4J45   = triggerdecision->isDefined("L1_4J45") ? triggerdecision->isTriggered("L1_4J45") : -999;
  m_h_L1_FJ30   = triggerdecision->isDefined("L1_FJ30") ? triggerdecision->isTriggered("L1_FJ30") : -999;
  m_h_L1_BJT15  = triggerdecision->isDefined("L1_BJT15") ? triggerdecision->isTriggered("L1_BJT15") : -999;

   
  m_h_L2_e10  = triggerdecision->isDefined("L2_e10") ? triggerdecision->isTriggered("L2_e10") : -999;
  m_h_L2_e10TRTxK  = triggerdecision->isDefined("L2_e10TRTxK") ? triggerdecision->isTriggered("L2_e10TRTxK") : -999;
  m_h_L2_e10L2_e10  = triggerdecision->isDefined("L2_e10L2_e10") ? triggerdecision->isTriggered("L2_e10L2_e10") : -999;
   
  m_h_L2_e15iL2_e15i  = triggerdecision->isDefined("L2_e15iL2_e15i") ? triggerdecision->isTriggered("L2_e15iL2_e15i") : -999;

  m_h_L2_e25i  = triggerdecision->isDefined("L2_e25i") ? triggerdecision->isTriggered("L2_e25i") : -999;
  m_h_L2_e60  = triggerdecision->isDefined("L2_e60") ? triggerdecision->isTriggered("L2_e60") : -999;
  m_h_L2_g10  = triggerdecision->isDefined("L2_g10") ? triggerdecision->isTriggered("L2_g10") : -999;
  m_h_L2_g20iL2_g20i  = triggerdecision->isDefined("L2_g20iL2_g20i") ? triggerdecision->isTriggered("L2_g20iL2_g20i") : -999;
  m_h_L2_g60  = triggerdecision->isDefined("L2_g60") ? triggerdecision->isTriggered("L2_g60") : -999;
  m_h_L2_Ze10e10  = triggerdecision->isDefined("L2_Z(e10e10)") ? triggerdecision->isTriggered("L2_Z(e10e10)") : -999;
  m_h_L2_tau10  = triggerdecision->isDefined("L2_tau10") ? triggerdecision->isTriggered("L2_tau10") : -999;
  m_h_L2_tau10i  = triggerdecision->isDefined("L2_tau10i") ? triggerdecision->isTriggered("L2_tau10i") : -999;
  m_h_L2_tau15  = triggerdecision->isDefined("L2_tau15") ? triggerdecision->isTriggered("L2_tau15") : -999;
  m_h_L2_tau15i  = triggerdecision->isDefined("L2_tau15i") ? triggerdecision->isTriggered("L2_tau15i") : -999;
  m_h_L2_tau20i  = triggerdecision->isDefined("L2_tau20i") ? triggerdecision->isTriggered("L2_tau20i") : -999;
  m_h_L2_tau25i  = triggerdecision->isDefined("L2_tau25i") ? triggerdecision->isTriggered("L2_tau25i") : -999;
  m_h_L2_tau35i  = triggerdecision->isDefined("L2_tau35i") ? triggerdecision->isTriggered("L2_tau35i") : -999;
  m_h_L2_tauNoCut  = triggerdecision->isDefined("L2_tauNoCut") ? triggerdecision->isTriggered("L2_tauNoCut") : -999;
  m_h_L2_mu6  = triggerdecision->isDefined("L2_mu6") ? triggerdecision->isTriggered("L2_mu6") : -999;
  m_h_L2_mu6l  = triggerdecision->isDefined("L2_mu6l") ? triggerdecision->isTriggered("L2_mu6l") : -999;
  m_h_L2_mu20i  = triggerdecision->isDefined("L2_mu20i") ? triggerdecision->isTriggered("L2_mu20i") : -999;
  m_h_L2_jet20kt  = triggerdecision->isDefined("L2_jet20kt") ? triggerdecision->isTriggered("L2_jet20kt") : -999;
  m_h_L2_jet20  = triggerdecision->isDefined("L2_jet20") ? triggerdecision->isTriggered("L2_jet20") : -999;
  m_h_L2_jet400  = triggerdecision->isDefined("L2_jet400") ? triggerdecision->isTriggered("L2_jet400") : -999;
  m_h_L2_jet350L2_jet350  = 
    triggerdecision->isDefined("L2_jet350L2_jet350") 
    ? triggerdecision->isTriggered("L2_jet350L2_jet350") : -999;
  m_h_L2_jet165L2_jet165L2_jet165  = 
    triggerdecision->isDefined("L2_jet165L2_jet165L2_jet165") ? 
    triggerdecision->isTriggered("L2_jet165L2_jet165L2_jet165") : -999;
  m_h_L2_jet110L2_jet110L2_jet110L2_jet110  = 
    triggerdecision->isDefined("L2_jet110L2_jet110L2_jet110L2_jet110") ? 
    triggerdecision->isTriggered("L2_jet110L2_jet110L2_jet110L2_jet110") : -999;
   

  m_h_EF_MuonTRTExt_mu6l  = triggerdecision->isDefined("EF_MuonTRTExt_mu6l") ? triggerdecision->isTriggered("EF_MuonTRTExt_mu6l") : -999;
  m_h_EF_e10  = triggerdecision->isDefined("EF_e10") ? triggerdecision->isTriggered("EF_e10") : -999;
  m_h_EF_e10TRTxK  = triggerdecision->isDefined("EF_e10TRTxK") ? triggerdecision->isTriggered("EF_e10TRTxK") : -999;
  m_h_EF_e15iEF_e15i  = triggerdecision->isDefined("EF_e15iEF_e15i") ? triggerdecision->isTriggered("EF_e15iEF_e15i") : -999;
  m_h_EF_e25i  = triggerdecision->isDefined("EF_e25i") ? triggerdecision->isTriggered("EF_e25i") : -999;
  m_h_EF_e60  = triggerdecision->isDefined("EF_e60") ? triggerdecision->isTriggered("EF_e60") : -999;
  m_h_EF_g10  = triggerdecision->isDefined("EF_g10") ? triggerdecision->isTriggered("EF_g10") : -999;
  m_h_EF_g20iEF_g20i  = triggerdecision->isDefined("EF_g20iEF_g20i") ? triggerdecision->isTriggered("EF_g20iEF_g20i") : -999;
  m_h_EF_g60  = triggerdecision->isDefined("EF_g60") ? triggerdecision->isTriggered("EF_g60") : -999;
  m_h_EF_jet110EF_jet110EF_jet110EF_jet110  = triggerdecision->isDefined("EF_jet110EF_jet110EF_jet110EF_jet110") ? triggerdecision->isTriggered("EF_jet110EF_jet110EF_jet110EF_jet110") : -999;
  m_h_EF_jet165EF_jet165EF_jet165  = triggerdecision->isDefined("EF_jet165EF_jet165EF_jet165") ? triggerdecision->isTriggered("EF_jet165EF_jet165EF_jet165") : -999;
  m_h_EF_jet20  = triggerdecision->isDefined("EF_jet20") ? triggerdecision->isTriggered("EF_jet20") : -999;
  m_h_EF_jet20kt  = triggerdecision->isDefined("EF_jet20kt") ? triggerdecision->isTriggered("EF_jet20kt") : -999;
  m_h_EF_jet350EF_jet350  = triggerdecision->isDefined("EF_jet350EF_jet350") ? triggerdecision->isTriggered("EF_jet350EF_jet350") : -999;
  m_h_EF_jet400  = triggerdecision->isDefined("EF_jet400") ? triggerdecision->isTriggered("EF_jet400") : -999;
  m_h_EF_mu20i  = triggerdecision->isDefined("EF_mu20i") ? triggerdecision->isTriggered("EF_mu20i") : -999;
  m_h_EF_mu6  = triggerdecision->isDefined("EF_mu6") ? triggerdecision->isTriggered("EF_mu6") : -999;
  m_h_EF_mu6l  = triggerdecision->isDefined("EF_mu6l") ? triggerdecision->isTriggered("EF_mu6l") : -999;
  m_h_EF_tau10  = triggerdecision->isDefined("EF_tau10") ? triggerdecision->isTriggered("EF_tau10") : -999;
  m_h_EF_tau10i  = triggerdecision->isDefined("EF_tau10i") ? triggerdecision->isTriggered("EF_tau10i") : -999;
  m_h_EF_tau15  = triggerdecision->isDefined("EF_tau15") ? triggerdecision->isTriggered("EF_tau15") : -999;
  m_h_EF_tau15i  = triggerdecision->isDefined("EF_tau15i") ? triggerdecision->isTriggered("EF_tau15i") : -999;
  m_h_EF_tau20i  = triggerdecision->isDefined("EF_tau20i") ? triggerdecision->isTriggered("EF_tau20i") : -999;
  m_h_EF_tau25i  = triggerdecision->isDefined("EF_tau25i") ? triggerdecision->isTriggered("EF_tau25i") : -999;
  m_h_EF_tau35i  = triggerdecision->isDefined("EF_tau35i") ? triggerdecision->isTriggered("EF_tau35i") : -999;
  m_h_EF_tauNoCut  = triggerdecision->isDefined("EF_tauNoCut") ? triggerdecision->isTriggered("EF_tauNoCut") : -999;


  triggerdecision->print();
  mLog << MSG::INFO << "TriggerDecision added to Ntuple. " << endreq;


  return StatusCode::SUCCESS;
}

StatusCode AODtoLiverpoolNtuple::UnpackEFegamma()
{
  MsgStream mLog(messageService(), name());
  StatusCode status = StatusCode::SUCCESS;
  
  if (m_fastsim == 1)
    {
      m_h_egamma_num = 0; // no egammas in Altfast data
      /// I don't know if that's true
      return StatusCode::SUCCESS;
    }
  const DataHandle<egammaContainer> trigEcontIt, trigEcontEnd;

  status= m_storeGate->retrieve(trigEcontIt,trigEcontEnd);
  if (status.isFailure()) 
    {
    mLog << MSG::INFO << "Failure when retrieving egammaContainers from SG" << endreq;
    return StatusCode::SUCCESS;
    }

  // initialize number of RoI per event counter
  int nRoi = -999; //Just for logging, not in ntuple
  int n=0;        // number of egamma objects per event

  // Loop over all the egammaContainers
  for (;trigEcontIt != trigEcontEnd; ++trigEcontIt) 
    {

      // Identify the trigger egamma containers (you don't want to include offline ones)
      // they include _HLTAutoKey in their name

      std::string contName = trigEcontIt.key();
      
      mLog << MSG::INFO << "Retrieved from SG egammaContainer with key: " << contName << endreq;
      
      // find trigger containers + protection against failed find operation
      if( contName.find("_HLTAutoKey") != std::string::npos) 
	{
	  // get RoI identifier from container name if present
	  if( contName.find("_RoI") != std::string::npos) 
	    {
	      std::stringstream strm;
	      strm << (contName).substr((contName).find("_RoI",0)+4);
	      nRoi = long (std::atoi( (strm.str()).c_str() ));
	    }
	  
	  mLog << MSG::INFO
	       << "Filling CBNT trigEgamma with: " << contName
	       << " size: " << trigEcontIt->size()  
	       << " corresponding to Roi : " << nRoi
	       << endreq;

	  mLog << MSG::DEBUG
	       << "Filling CBNTAA trigEgamma with: " << contName
	       << " size: " << trigEcontIt->size()
	       << " corresponding to Roi : " << nRoi
	       << endreq;

	  // get pointer to fist and last cluster in the egamma container
	  typedef egammaContainer::const_iterator  egamma_iterator ; 
	  egamma_iterator f_eg = trigEcontIt->begin(); 
	  egamma_iterator l_eg = trigEcontIt->end(); 

	  // get pointer to egamma object
	  const egamma* eg; 
	  // pointer to cluster, shower shape and trackmatch objects which is
	  // given in egamma object
	  const CaloCluster* clus; 
	  const EMShower* shower; 
	  const EMTrackMatch* trkmatch; 
	  const Rec::TrackParticle* trackparticle;
	  const Trk::TrackSummary* tracksummary;
	  
	  mLog << MSG::DEBUG
	       << "Beginning loop over egamma objects"
	       << endreq;

	  // Loop over all the egamma objects in egamma container
	  for ( ; f_eg!=l_eg; f_eg++) 
	    {
	      n++;
	      if(n < max_egamma_num)
		{ 
		  eg = *f_eg; 
		  // the combined result
		  m_h_egamma_e[n-1]     = eg->e(); 
		  m_h_egamma_eta[n-1]   = eg->eta(); 
		  m_h_egamma_phi[n-1]   = eg->phi(); 
		  // e-hypothesis in case only calorimeter information is used 
		  m_h_egamma_isem[n-1] = (long) (eg)->isem();
		  // (1:e, 0:jet)
		  m_h_egamma_epiANN[n-1] = (eg)->get_egammaID(egammaPID::NeuralNet);
		  // pion and EMweights
		  m_h_egamma_EMWeight[n-1]   = (eg)->get_egammaID(egammaPID::ElectronWeight);
		  m_h_egamma_PionWeight[n-1] = (eg)->get_egammaID(egammaPID::BgWeight);
		  // softe ID
		  m_h_egamma_isemse[n-1] = (long) (eg)->isemse();
		  m_h_egamma_epiANNse[n-1] = (eg)->get_egammaID(egammaPID::SofteNeuralNet);
		  // pion and EMweights
		  m_h_egamma_EMWeightse[n-1]   = (eg)->get_egammaID(egammaPID::SofteElectronWeight);
		  m_h_egamma_PionWeightse[n-1] = (eg)->get_egammaID(egammaPID::SofteBgWeight);
		  
		  // access cluster information
		  clus =eg->get_Cluster();
		  
		  if (clus) 
		    {
		      mLog << MSG::DEBUG
			   << "Found cluster"
			   << endreq;
		      
		      float ETclus =(clus)->et();
		      if(ETclus != -999.) 
			{
			  mLog << MSG::DEBUG
			       << "Found good cluster"
			       << endreq;
			  
			  // corrected ET, eta, phi of cluster
			  m_h_egamma_cl_et[n-1]=(clus)->et(); 
			  m_h_egamma_cl_eta[n-1]=(clus)->eta(); 
			  m_h_egamma_cl_phi[n-1]=(clus)->phi(); 
			  
			  // energies in different layers at EM scale without any corrections 
			  // for leakage
			  m_h_egamma_e0[n-1]=(clus)->energyBE(0); 
			  m_h_egamma_e1[n-1]=(clus)->energyBE(1); 
			  m_h_egamma_e2[n-1]=(clus)->energyBE(2); 
			  m_h_egamma_e3[n-1]=(clus)->energyBE(3); 
		      
			  // uncorrected eta positions in the different em layers
			  m_h_egamma_eta0[n-1]=(clus)->etaBE(0); 
			  m_h_egamma_eta1[n-1]=(clus)->etaBE(1); 
			  m_h_egamma_eta2[n-1]=(clus)->etaBE(2); 
			  m_h_egamma_eta3[n-1]=(clus)->etaBE(3); 
		      
			  // uncorrected phi positions in the different em layers
			  m_h_egamma_phi0[n-1]=(clus)->phiBE(0); 
			  m_h_egamma_phi1[n-1]=(clus)->phiBE(1); 
			  m_h_egamma_phi2[n-1]=(clus)->phiBE(2); 
			  m_h_egamma_phi3[n-1]=(clus)->phiBE(3); 
		      
			  // get pointing information 
			  mLog << MSG::DEBUG
			       << "Getting point information"
			       << endreq;
			  status= m_egammaqpoint->execute(clus);     
			  if ( status.isFailure() ) 
			    {
			      mLog << MSG::ERROR 
				   << "failure of egammaqpoint "
				   << endreq;
			    } else {
			      mLog << MSG::DEBUG
				   << "Found egammaqpoint"
				   << endreq;
			    }
		      
			  m_h_egamma_zvertex[n-1] = m_egammaqpoint->zvertex(); 
			  m_h_egamma_errz[n-1]    = m_egammaqpoint->errz(); 
			  m_h_egamma_etap[n-1]    = m_egammaqpoint->eta(); 
			  // get shower depth
			  mLog << MSG::DEBUG
			       << "Getting shower depth information"
			       << endreq;
			  status= m_egammaqgcld->execute(clus);
			  if ( status.isFailure() ) 
			    {
			      mLog << MSG::ERROR 
				   << "failure of egammaqgcld " 
				   << endreq;
			    } else {
			      mLog << MSG::DEBUG
				   << "Found egammaqgcld"
				   << endreq;
			    }
			  m_h_egamma_depth[n-1] = m_egammaqgcld->depth();
			}
		    }
		  // now access shower shape quantities if available
		  mLog << MSG::DEBUG
		       << "Finding shower"
		       << endreq;
		  shower = eg->get_EMShower();
		  if (shower) 
		    {
		      mLog << MSG::DEBUG
			   << "Found shower"
			   << endreq;
		      // in case we keep in CBNT the high pt e/gamma-id
		      // leakage into hadronic calorimeter
		      m_h_egamma_ethad1[n-1]   = (shower)->ethad1();
		      m_h_egamma_ethad[n-1]    = (shower)->ethad();
		      m_h_egamma_ehad1[n-1]    = (shower)->ehad1();
		  
		      // useful shape quantities from the 3rd em sampling
		      m_h_egamma_f3[n-1]       = (shower)->f3(); 
		  
		      // useful shape quantities from the 2nd em sampling
		  
		      m_h_egamma_e233[n-1]     = (shower)->e233(); 
		      m_h_egamma_e237[n-1]     = (shower)->e237(); 
		      m_h_egamma_e277[n-1]     = (shower)->e277(); 
		      m_h_egamma_weta2[n-1]    = (shower)->weta2();
		      m_h_egamma_widths2[n-1]  = (shower)->widths2();
		      m_h_egamma_poscs2[n-1]   = (shower)->poscs2();
		  
		      // useful shape quantities from the 1st em sampling
		      m_h_egamma_f1[n-1]       = (shower)->f1(); 
		      m_h_egamma_weta1[n-1]    = (shower)->weta1(); 
		      m_h_egamma_e2ts1[n-1]    = (shower)->e2ts1(); 
		      m_h_egamma_e2tsts1[n-1]  = (shower)->e2tsts1(); 
		      m_h_egamma_widths1[n-1]  = (shower)->widths1();
		      m_h_egamma_poscs1[n-1]   = (shower)->poscs1();
		      m_h_egamma_fracs1[n-1]   = (shower)->fracs1(); 
		      m_h_egamma_barys1[n-1]   = (shower)->barys1();
		      m_h_egamma_wtots1[n-1]   = (shower)->wtots1(); 
		      m_h_egamma_emins1[n-1]   = (shower)->emins1();
		      m_h_egamma_emaxs1[n-1]   = (shower)->emaxs1();
		  
		      m_h_egamma_etcone[n-1]   = (shower)->etcone();
		      m_h_egamma_etcone20[n-1] = (shower)->etcone20();
		      m_h_egamma_etcone30[n-1] = (shower)->etcone30();
		      m_h_egamma_etcone40[n-1] = (shower)->etcone40();
		  
		      // in case we keep in CBNT the softe e-id
		      m_h_egamma_f1core[n-1]   = (shower)->f1core(); 
		      m_h_egamma_f3core[n-1]   = (shower)->f3core(); 
		      m_h_egamma_asy1[n-1]     = (shower)->asy1();
		      m_h_egamma_pos7[n-1]     = (shower)->pos7();
		      m_h_egamma_isol[n-1]     = (shower)->iso();
		      m_h_egamma_ecore[n-1]    = (shower)->ecore();        
		  
		      // in case we keep in the CBNT the individual energy 
		      // deposition in calorimeters
		      // useful shape quantities from the pre sampler
		  
		      m_h_egamma_e011[n-1]     = (shower)->e011(); 
		  
		      // useful shape quantities from the strip
		  
		      m_h_egamma_e131[n-1]     = (shower)->e131(); 
		  
		      // useful shape quantities from the 2nd em sampling
		  
		      m_h_egamma_e235[n-1]     = (shower)->e235(); 
		  
		      // useful shape quantities from the 3rd em sampling
		      m_h_egamma_e335[n-1]     = (shower)->e335(); 
		    }
	      
		  // now access track match quantities (if available)
		  // trackMatch for iPatRec tracks?
		  trkmatch = eg->get_EMTrackMatch();
	      
		  if (trkmatch) 
		    {
		      mLog << MSG::DEBUG
			   << "Found track match"
			   << endreq;
		      m_h_egamma_primary[n-1]   = trkmatch->isPrimary() + 1;
		      m_h_egamma_numtracks[n-1] = trkmatch->tracksInBroadWindow(); 
		      m_h_egamma_eoverp[n-1]    = trkmatch->EoverP();
		      m_h_egamma_etacorr[n-1]   = trkmatch->EtaCorrMag();
		      m_h_egamma_deta0[n-1]     = trkmatch->deltaEta(0);
		      m_h_egamma_dphi0[n-1]     = trkmatch->deltaPhi(0);
		      m_h_egamma_deta1[n-1]     = trkmatch->deltaEta(1);
		      m_h_egamma_dphi1[n-1]     = trkmatch->deltaPhi(1);
		      m_h_egamma_deta2[n-1]     = trkmatch->deltaEta(2);
		      m_h_egamma_dphi2[n-1]     = trkmatch->deltaPhi(2);
		      m_h_egamma_deta3[n-1]     = trkmatch->deltaEta(3);
		      m_h_egamma_dphi3[n-1]     = trkmatch->deltaPhi(3);
		  
		    } else {
		      //// Here, default of 0 is dangerous
		      mLog << MSG::DEBUG
			   << "No trackmatch found"
			   << endreq;
		      m_h_egamma_primary[n-1]    = 0;
		      m_h_egamma_numtracks[n-1]  = -999;
		      m_h_egamma_eoverp[n-1]     = -999.;
		      m_h_egamma_etacorr[n-1]    = -999.;
		      m_h_egamma_deta0[n-1]      = -999.;
		      m_h_egamma_dphi0[n-1]      = -999.;
		      m_h_egamma_deta1[n-1]      = -999.;
		      m_h_egamma_dphi1[n-1]      = -999.;
		      m_h_egamma_deta2[n-1]      = -999.;
		      m_h_egamma_dphi2[n-1]      = -999.;
		      m_h_egamma_deta3[n-1]      = -999.;
		      m_h_egamma_dphi3[n-1]      = -999.;
		    }
	      
		  trackparticle = eg->get_TrackParticle();
		  if ( trackparticle )
		    {
		      tracksummary = trackparticle->trackSummary();
		      if ( tracksummary )
			{
			  m_h_egamma_tracksummary_nTRThits[n-1] = tracksummary->get(Trk::numberOfTRTHits);
			  m_h_egamma_tracksummary_nTRThitsHT[n-1] = tracksummary->get(Trk::numberOfTRTHighThresholdHits);
			}
		    }
		}
	    }        // end of loop over egamma objects
	}        // end of if to select trigger egamma Containers
    }        // end of loop over egamma Containers
  
  m_h_egamma_num = n < max_egamma_num ? n : max_egamma_num; 
  
  return StatusCode::SUCCESS;
}


StatusCode AODtoLiverpoolNtuple::UnpackTrigElectron()
{
  MsgStream mLog(messageService(), name());
  StatusCode status = StatusCode::SUCCESS;
  
  if (m_fastsim == 1)
    {
      m_h_trigelectron_num = 0; // no TrigElectrons in Altfast data
      /// I don't know if that's true
      return StatusCode::SUCCESS;
    }

  /// There's usually more than one TrigElectronContainer in AOD
  /// They are identified by the DataHandle type (TrigElectronContainer)
  /// and also by their StoreGateKey ending with "_HLTAutoKey"
  const DataHandle<TrigElectronContainer> trigEcontItr, trigEcontEnd;
  status = m_storeGate->retrieve(trigEcontItr,trigEcontEnd);
  if( status.isFailure() ) 
    {
      mLog << MSG::WARNING << "No AOD TrigElectron container found in TDS" << endreq;
      return StatusCode::SUCCESS;
    }
  mLog << MSG::DEBUG << "TrigElectronContainer successfully retrieved" << endreq;
  
  // Loop over all the TrigElectronContainers
  int Ntrigelectroncontainer = 0;
  int Ntrigelectron = 0;
  for (;trigEcontItr != trigEcontEnd; ++trigEcontItr) 
    {
      std::string contName = trigEcontItr.key(); /// Ends "_HLTAutoKey"
      // debug message
      mLog << MSG::DEBUG
	   << "Retrieved TrigElectronContainer from SG with key: " << contName
	   << endreq;
      // find trigger containers + protection against failed find operation
      if( contName.find("_HLTAutoKey") != std::string::npos)
	{
	  Ntrigelectroncontainer++;
	  /// Now we have a container, let's get the TrigElectron objects from it
	  TrigElectronContainer::const_iterator trigEItr  = (*trigEcontItr).begin();
	  TrigElectronContainer::const_iterator trigEItrE = (*trigEcontItr).end();
	  
	  for (; trigEItr != trigEItrE; ++trigEItr) /// Loop over TrigElectrons
	    {
	      //// Note to self: (*track)->algorithmId() == TrigInDetTrack::SITRACKID
	      Ntrigelectron++;
	      if( Ntrigelectron < max_trigelectron_num )
		{
		  m_h_trigelectron_container[Ntrigelectron-1]   = Ntrigelectroncontainer;
		  m_h_trigelectron_pt[Ntrigelectron-1]          = (*trigEItr)->Pt();
		  m_h_trigelectron_eta[Ntrigelectron-1]         = (*trigEItr)->eta();
		  m_h_trigelectron_phi[Ntrigelectron-1]         = (*trigEItr)->phi();
		  m_h_trigelectron_m[Ntrigelectron-1]           = (*trigEItr)->m();		
		  m_h_trigelectron_isvalid[Ntrigelectron-1]     = (*trigEItr)->isValid();	
		  m_h_trigelectron_roiID[Ntrigelectron-1]       = (*trigEItr)->roiId();	
		  m_h_trigelectron_zvtx[Ntrigelectron-1]        = (*trigEItr)->Zvtx();	
		  m_h_trigelectron_err_eta[Ntrigelectron-1]     = (*trigEItr)->err_eta();	
		  m_h_trigelectron_err_phi[Ntrigelectron-1]     = (*trigEItr)->err_phi();	
		  m_h_trigelectron_err_zvtx[Ntrigelectron-1]    = (*trigEItr)->err_Zvtx();	
		  m_h_trigelectron_err_pt[Ntrigelectron-1]      = (*trigEItr)->err_Pt();	
		  m_h_trigelectron_trkclusdeta[Ntrigelectron-1] = (*trigEItr)->trkClusDeta();
		  m_h_trigelectron_trkclusdphi[Ntrigelectron-1] = (*trigEItr)->trkClusDphi();

		  const TrigInDetTrack *track                   = (*trigEItr)->track();
		  if ( track ) 
		    {
		      m_h_trigelectron_track_algoID[Ntrigelectron-1]          = track->algorithmId();
		      m_h_trigelectron_track_chi2[Ntrigelectron-1]            = track->chi2();
		      m_h_trigelectron_track_NStrawHits[Ntrigelectron-1]      = track->NStrawHits();
		      m_h_trigelectron_track_NStraw[Ntrigelectron-1]          = track->NStraw();
		      m_h_trigelectron_track_NStrawTime[Ntrigelectron-1]      = track->NStrawTime();
		      m_h_trigelectron_track_NTRHits[Ntrigelectron-1]         = track->NTRHits();
		      const TrigInDetTrackFitPar* track_param                 = track->param();
		      if ( track_param )
			{
			  m_h_trigelectron_track_param_a0[Ntrigelectron-1]        = track_param->a0();
			  m_h_trigelectron_track_param_z0[Ntrigelectron-1]        = track_param->z0();
			  m_h_trigelectron_track_param_phi0[Ntrigelectron-1]      = track_param->phi0();
			  m_h_trigelectron_track_param_eta[Ntrigelectron-1]       = track_param->eta();
			  m_h_trigelectron_track_param_pt[Ntrigelectron-1]        = track_param->pT();
			  m_h_trigelectron_track_param_ea0[Ntrigelectron-1]       = track_param->ea0();
			  m_h_trigelectron_track_param_ez0[Ntrigelectron-1]       = track_param->ez0();
			  m_h_trigelectron_track_param_ephi0[Ntrigelectron-1]     = track_param->ephi0();
			  m_h_trigelectron_track_param_eeta[Ntrigelectron-1]      = track_param->eeta();
			  m_h_trigelectron_track_param_ept[Ntrigelectron-1]       = track_param->epT();
			}
		      const TrigInDetTrackFitPar* track_endparam              = track->endParam();
		      if ( track_endparam )
			{
			  m_h_trigelectron_track_endparam_a0[Ntrigelectron-1]     = track_endparam->a0();
			  m_h_trigelectron_track_endparam_z0[Ntrigelectron-1]     = track_endparam->z0();
			  m_h_trigelectron_track_endparam_phi0[Ntrigelectron-1]   = track_endparam->phi0();
			  m_h_trigelectron_track_endparam_eta[Ntrigelectron-1]    = track_endparam->eta();
			  m_h_trigelectron_track_endparam_pt[Ntrigelectron-1]     = track_endparam->pT();
			  m_h_trigelectron_track_endparam_ea0[Ntrigelectron-1]    = track_endparam->ea0();
			  m_h_trigelectron_track_endparam_ez0[Ntrigelectron-1]    = track_endparam->ez0();
			  m_h_trigelectron_track_endparam_ephi0[Ntrigelectron-1]  = track_endparam->ephi0();
			  m_h_trigelectron_track_endparam_eeta[Ntrigelectron-1]   = track_endparam->eeta();
			  m_h_trigelectron_track_endparam_ept[Ntrigelectron-1]    = track_endparam->epT();
			}
		    }

		  const TrigEMCluster *cluster                  = (*trigEItr)->cluster();
		  if ( cluster )
		    {
		      m_h_trigelectron_cluster_e[Ntrigelectron-1]       = cluster->energy();
		      m_h_trigelectron_cluster_et[Ntrigelectron-1]      = cluster->et();
		      m_h_trigelectron_cluster_eta[Ntrigelectron-1]     = cluster->eta();
		      m_h_trigelectron_cluster_phi[Ntrigelectron-1]     = cluster->phi();
		      m_h_trigelectron_cluster_e237[Ntrigelectron-1]    = cluster->e237();
		      m_h_trigelectron_cluster_e277[Ntrigelectron-1]    = cluster->e277();
		      m_h_trigelectron_cluster_e2tsts1[Ntrigelectron-1] = cluster->e2tsts1();
		      m_h_trigelectron_cluster_ehad1[Ntrigelectron-1]   = cluster->ehad1();
		      m_h_trigelectron_cluster_emaxs1[Ntrigelectron-1]  = cluster->emaxs1();
		      m_h_trigelectron_cluster_fracs1[Ntrigelectron-1]  = cluster->fracs1();
		      m_h_trigelectron_cluster_weta2[Ntrigelectron-1]   = cluster->weta2();
		    }
		}
	    }
	  m_h_trigelectron_num = Ntrigelectron < max_trigelectron_num ? Ntrigelectron : max_trigelectron_num;
	}
    }
  
  mLog << MSG::INFO << "TrigElectrons added to Ntuple. " << endreq;
  return StatusCode::SUCCESS;
}



StatusCode AODtoLiverpoolNtuple::UnpackL1EMTauObject()
{
  MsgStream mLog(messageService(), name());
  StatusCode status = StatusCode::SUCCESS;
  
  if (m_fastsim == 1)
    {
      m_h_l1emtauobject_num = 0; // no L1EMTauObjects in Altfast data
      return StatusCode::SUCCESS;
    }
  
  const L1EMTauObjectContainer* l1emtauTES;
  status=m_storeGate->retrieve( l1emtauTES, "L1EMTauObjectContainer");
  if( status.isFailure()  ||  !l1emtauTES ) 
    {
      mLog << MSG::WARNING << "No AOD L1EMTauObject container found in TDS" << endreq;
      return StatusCode::SUCCESS;
    }
  
  
  mLog << MSG::DEBUG << "L1EMTauObjects successfully retrieved" << endreq;
  
  L1EMTauObjectContainer::const_iterator l1emItr  = (*l1emtauTES).begin();
  L1EMTauObjectContainer::const_iterator l1emItrE = (*l1emtauTES).end();
  
  // L1EMTauObject
  
  int Nl1emtauobject = 0;
  for (; l1emItr != l1emItrE; ++l1emItr) 
    {
      Nl1emtauobject++ ;
      if(Nl1emtauobject < max_l1emtauobject_num) 
	{
	  m_h_l1emtauobject_emcore[Nl1emtauobject-1]    = (*l1emItr)->L1EM_EmCore();
	  m_h_l1emtauobject_hdcore[Nl1emtauobject-1]    = (*l1emItr)->L1EM_HdCore();
	  m_h_l1emtauobject_emclus[Nl1emtauobject-1]    = (*l1emItr)->L1EM_EmClus();
	  m_h_l1emtauobject_tauclus[Nl1emtauobject-1]   = (*l1emItr)->L1EM_TauClus();
	  m_h_l1emtauobject_emisol[Nl1emtauobject-1]    = (*l1emItr)->L1EM_EmIsol();
	  m_h_l1emtauobject_hdisol[Nl1emtauobject-1]    = (*l1emItr)->L1EM_HdIsol();
	  m_h_l1emtauobject_eta[Nl1emtauobject-1]    = (*l1emItr)->L1EM_eta();
	  m_h_l1emtauobject_phi[Nl1emtauobject-1]    = (*l1emItr)->L1EM_phi();
	}
    }
  
  m_h_l1emtauobject_num = Nl1emtauobject < max_l1emtauobject_num ? Nl1emtauobject : max_l1emtauobject_num;
  
  mLog << MSG::INFO << "L1EMTauObjects added to Ntuple. " << endreq;
  return StatusCode::SUCCESS;
}



StatusCode AODtoLiverpoolNtuple::UnpackL1JetObject()
{
  MsgStream mLog(messageService(), name());
  StatusCode status = StatusCode::SUCCESS;
  
  if (m_fastsim == 1)
    {
      m_h_l1jetobject_num = 0; // no L1JetObjects in Altfast data
      return StatusCode::SUCCESS;
    }
  
  const L1JetObjectContainer* l1jetTES;
  status=m_storeGate->retrieve( l1jetTES, "L1JetObjectContainer");
  if( status.isFailure()  ||  !l1jetTES ) 
    {
      mLog << MSG::WARNING << "No AOD L1JetObject container found in TDS" << endreq;
      return StatusCode::SUCCESS;
    }
   

  mLog << MSG::DEBUG << "L1JetObjects successfully retrieved" << endreq;

  L1JetObjectContainer::const_iterator l1jetItr  = (*l1jetTES).begin();
  L1JetObjectContainer::const_iterator l1jetItrE = (*l1jetTES).end();

  // L1JetObject

  int Nl1jetobject = 0;
  for (; l1jetItr != l1jetItrE; ++l1jetItr) 
    {
      Nl1jetobject++ ;
      if(Nl1jetobject < max_l1jetobject_num) 
	{
	  m_h_l1jetobject_L1Jet_ET4x4 [Nl1jetobject-1]    = (*l1jetItr)->L1Jet_ET4x4();
	  m_h_l1jetobject_L1Jet_ET6x6[Nl1jetobject-1]    = (*l1jetItr)->L1Jet_ET6x6();
	  m_h_l1jetobject_L1Jet_ET8x8[Nl1jetobject-1]    = (*l1jetItr)->L1Jet_ET8x8();
	  m_h_l1jetobject_L1Jet_eta[Nl1jetobject-1]    = (*l1jetItr)->L1Jet_eta();
	  m_h_l1jetobject_L1Jet_phi[Nl1jetobject-1]    = (*l1jetItr)->L1Jet_phi();

	}
    }

  m_h_l1jetobject_num = Nl1jetobject < max_l1jetobject_num ? Nl1jetobject : max_l1jetobject_num;

  mLog << MSG::INFO << "L1JetObjects added to Ntuple. " << endreq;
  return StatusCode::SUCCESS;
}



StatusCode AODtoLiverpoolNtuple::UnpackL1EtmissObject()
{
  MsgStream mLog(messageService(), name());
  StatusCode status = StatusCode::SUCCESS;

  if (m_fastsim == 1)
    {
      return StatusCode::SUCCESS;
    }

  const L1EtmissObject* l1etmiss;
  status=m_storeGate->retrieve( l1etmiss, "L1EtmissObject");
  if( status.isFailure()  ||  !l1etmiss ) 
    {
      mLog << MSG::WARNING << "No AOD L1EtmissObject found in TDS" << endreq;
      return StatusCode::SUCCESS;
    }
   

  mLog << MSG::DEBUG << "L1EtmissObject successfully retrieved" << endreq;

  // L1EtmissObject

  m_h_l1etmissobject_L1ETM_Ex     = l1etmiss->L1ETM_Ex();
  m_h_l1etmissobject_L1ETM_Ey     = l1etmiss->L1ETM_Ey();
  m_h_l1etmissobject_L1ETM_EtMiss = l1etmiss->L1ETM_EtMiss();
  m_h_l1etmissobject_L1ETM_EtSum  = l1etmiss->L1ETM_EtSum();

  mLog << MSG::INFO << "L1EtmissObject added to Ntuple. " << endreq;
  return StatusCode::SUCCESS;
}




StatusCode AODtoLiverpoolNtuple::UnpackTauJet()
{
  MsgStream mLog(messageService(), name());
  StatusCode status = StatusCode::SUCCESS;
  
  const TauJetContainer* tauTES;
  if (m_fastsim==1)
    {
      status=m_storeGate->retrieve( tauTES, "AtlfastTauJetContainer");
      if( status.isFailure()  ||  !tauTES ) 
	{
	  mLog << MSG::WARNING << "No Atlfast AOD TauJet container found in TDS" << endreq;
	  return StatusCode::SUCCESS;
	}
    }
  else
    {
      status=m_storeGate->retrieve( tauTES, "TauJetCollection");
      if( status.isFailure()  ||  !tauTES ) 
	{
	  mLog << MSG::WARNING << "No AOD TauJet container found in TDS" << endreq;
	  return StatusCode::SUCCESS;
	}
    }
  
  mLog << MSG::DEBUG << "Taus successfully retrieved" << endreq;
  
  //int NTauAll = (*tauTES).size();
  TauJetContainer::const_iterator tauItr  = (*tauTES).begin();      
  TauJetContainer::const_iterator tauItrE = (*tauTES).end();     
  
  // TauJet
  int Ntaujet = 0;
  for (; tauItr != tauItrE; ++tauItr) 
    {  
      Ntaujet++ ;
      if(Ntaujet < max_taujet_num) 
	{
	  m_h_taujet_pt[Ntaujet-1]=(*tauItr)->pt();
	  m_h_taujet_eta[Ntaujet-1]=(*tauItr)->eta();
	  m_h_taujet_phi[Ntaujet-1]=(*tauItr)->phi();
	  m_h_taujet_e[Ntaujet-1]=(*tauItr)->e();
	  
	  m_h_taujet_numTrack[Ntaujet-1]=(*tauItr)->numTrack();
	  
	  if (m_fastsim==0)
	    {
	      m_h_taujet_author[Ntaujet-1]=(*tauItr)->author(); 	
	      
	      m_h_taujet_etHadCalib[Ntaujet-1]= (*tauItr)->parameter(TauJetParameters::etHadCalib); 	
	      m_h_taujet_etEMCalib[Ntaujet-1]= (*tauItr)->parameter(TauJetParameters::etEMCalib); 	
	      m_h_taujet_emRadius[Ntaujet-1]= (*tauItr)->parameter(TauJetParameters::emRadius); 	
	      m_h_taujet_isolationFraction[Ntaujet-1]= (*tauItr)->parameter(TauJetParameters::isolationFraction); 	
	      m_h_taujet_centralityFraction[Ntaujet-1]= (*tauItr)->parameter(TauJetParameters::centralityFraction); 	
	      m_h_taujet_stripWidth2[Ntaujet-1]= (*tauItr)->parameter(TauJetParameters::stripWidth2); 	
	      m_h_taujet_nStripCells[Ntaujet-1]= (*tauItr)->parameter(TauJetParameters::nStripCells);	
	      if (m_h_taujet_author[Ntaujet-1] == 1)
		{
		  m_h_taujet_logLikelihoodRatio[Ntaujet-1]= (*tauItr)->parameter(TauJetParameters::logLikelihoodRatio); 	
		  m_h_taujet_lowPtTauJetDiscriminant[Ntaujet-1]= (*tauItr)->parameter(TauJetParameters::lowPtTauJetDiscriminant); 	
		  m_h_taujet_lowPtTauEleDiscriminant[Ntaujet-1]= (*tauItr)->parameter(TauJetParameters::lowPtTauEleDiscriminant); 	
		  m_h_taujet_tauJetNeuralnetwork[Ntaujet-1]= (*tauItr)->parameter(TauJetParameters::tauJetNeuralnetwork);
		  m_h_taujet_tauENeuralNetwork[Ntaujet-1]= (*tauItr)->parameter(TauJetParameters::tauENeuralNetwork);
		}
	      else if (m_h_taujet_author[Ntaujet-1] == 2)
		{
		  m_h_taujet_annularIsolationFraction[Ntaujet-1]= (*tauItr)->parameter(TauJetParameters::annularIsolationFraction);	
		  m_h_taujet_etCaloAtEMScale[Ntaujet-1]= (*tauItr)->parameter(TauJetParameters::etCaloAtEMScale);
		  m_h_taujet_etChargedHadCells[Ntaujet-1]= (*tauItr)->parameter(TauJetParameters::etChargedHadCells);
		  m_h_taujet_etOtherEMCells[Ntaujet-1]= (*tauItr)->parameter(TauJetParameters::etOtherEMCells);
		  m_h_taujet_etOtherHadCells[Ntaujet-1]= (*tauItr)->parameter(TauJetParameters::etOtherHadCells);
		  m_h_taujet_discriminant[Ntaujet-1]= (*tauItr)->parameter(TauJetParameters::discriminant);
		  m_h_taujet_discPDERS[Ntaujet-1]= (*tauItr)->parameter(TauJetParameters::discPDERS);	
		  m_h_taujet_discNN[Ntaujet-1]= (*tauItr)->parameter(TauJetParameters::discNN);
		  m_h_taujet_sumPTtracks[Ntaujet-1]= (*tauItr)->parameter(TauJetParameters::sumPTtracks);	
		  m_h_taujet_nAssocTracksCore[Ntaujet-1]= (*tauItr)->parameter(TauJetParameters::nAssocTracksCore);	
		  m_h_taujet_nAssocTracksIsol[Ntaujet-1]= (*tauItr)->parameter(TauJetParameters::nAssocTracksIsol);	
		  m_h_taujet_rWidth2Trk3P[Ntaujet-1]= (*tauItr)->parameter(TauJetParameters::rWidth2Trk3P); 	
		  m_h_taujet_massTrk3P[Ntaujet-1]= (*tauItr)->parameter(TauJetParameters::massTrk3P);	
		  m_h_taujet_signDOTrk3P[Ntaujet-1]= (*tauItr)->parameter(TauJetParameters::signDOTrk3P);
		}
	    }
	}
    }
  m_h_taujet_num = Ntaujet < max_taujet_num ? Ntaujet : max_taujet_num;

  mLog << MSG::INFO << "TauJets added to Ntuple. " << endreq;


  return StatusCode::SUCCESS;
}





StatusCode AODtoLiverpoolNtuple::UnpackTauJet1p3p()
{
  MsgStream mLog(messageService(), name());
  StatusCode status = StatusCode::SUCCESS;
  

  if (m_fastsim==1)
    {
      return StatusCode::SUCCESS;
    }

  const TauJetContainer* tau1p3pTES;
  status=m_storeGate->retrieve( tau1p3pTES, "TauJet1p3pCollection");
  if( status.isFailure()  ||  !tau1p3pTES ) 
    {
      mLog << MSG::WARNING << "No AOD TauJet1p3p container found in TDS" << endreq;
      return StatusCode::SUCCESS;
    }
  
  mLog << MSG::DEBUG << "Taujet1p3p successfully retrieved" << endreq;
  
  //int NTauAll = (*tau1p3pTES).size();
  TauJetContainer::const_iterator tau1p3pItr  = (*tau1p3pTES).begin();      
  TauJetContainer::const_iterator tau1p3pItrE = (*tau1p3pTES).end();     
  
  // TauJet
  int Ntaujet1p3p = 0;
  for (; tau1p3pItr != tau1p3pItrE; ++tau1p3pItr) 
    {  
      Ntaujet1p3p++ ;
      if(Ntaujet1p3p < max_taujet1p3p_num) 
	{
	  m_h_taujet1p3p_pt[Ntaujet1p3p-1]=(*tau1p3pItr)->pt();
	  m_h_taujet1p3p_eta[Ntaujet1p3p-1]=(*tau1p3pItr)->eta();
	  m_h_taujet1p3p_phi[Ntaujet1p3p-1]=(*tau1p3pItr)->phi();
	  m_h_taujet1p3p_e[Ntaujet1p3p-1]=(*tau1p3pItr)->e();
	  
	  m_h_taujet1p3p_numTrack[Ntaujet1p3p-1]=(*tau1p3pItr)->numTrack();
	  
	  if (m_fastsim==0)
	    {
	      m_h_taujet1p3p_author[Ntaujet1p3p-1]=(*tau1p3pItr)->author(); 	
	      
	      m_h_taujet1p3p_etHadCalib[Ntaujet1p3p-1]= (*tau1p3pItr)->parameter(TauJetParameters::etHadCalib); 	
	      m_h_taujet1p3p_etEMCalib[Ntaujet1p3p-1]= (*tau1p3pItr)->parameter(TauJetParameters::etEMCalib); 	
	      m_h_taujet1p3p_emRadius[Ntaujet1p3p-1]= (*tau1p3pItr)->parameter(TauJetParameters::emRadius); 	
	      m_h_taujet1p3p_isolationFraction[Ntaujet1p3p-1]= (*tau1p3pItr)->parameter(TauJetParameters::isolationFraction); 	
	      m_h_taujet1p3p_centralityFraction[Ntaujet1p3p-1]= (*tau1p3pItr)->parameter(TauJetParameters::centralityFraction); 	
	      m_h_taujet1p3p_stripWidth2[Ntaujet1p3p-1]= (*tau1p3pItr)->parameter(TauJetParameters::stripWidth2); 	
	      m_h_taujet1p3p_nStripCells[Ntaujet1p3p-1]= (*tau1p3pItr)->parameter(TauJetParameters::nStripCells);	
	      if (m_h_taujet1p3p_author[Ntaujet1p3p-1] == 1)
		{
		  m_h_taujet1p3p_logLikelihoodRatio[Ntaujet1p3p-1]= (*tau1p3pItr)->parameter(TauJetParameters::logLikelihoodRatio); 	
		  m_h_taujet1p3p_lowPtTauJetDiscriminant[Ntaujet1p3p-1]= (*tau1p3pItr)->parameter(TauJetParameters::lowPtTauJetDiscriminant); 	
		  m_h_taujet1p3p_lowPtTauEleDiscriminant[Ntaujet1p3p-1]= (*tau1p3pItr)->parameter(TauJetParameters::lowPtTauEleDiscriminant); 	
		  m_h_taujet1p3p_tauJetNeuralnetwork[Ntaujet1p3p-1]= (*tau1p3pItr)->parameter(TauJetParameters::tauJetNeuralnetwork);
		  m_h_taujet1p3p_tauENeuralNetwork[Ntaujet1p3p-1]= (*tau1p3pItr)->parameter(TauJetParameters::tauENeuralNetwork);
		}
	      else if (m_h_taujet1p3p_author[Ntaujet1p3p-1] == 2)
		{
		  m_h_taujet1p3p_annularIsolationFraction[Ntaujet1p3p-1]= (*tau1p3pItr)->parameter(TauJetParameters::annularIsolationFraction);	
		  m_h_taujet1p3p_etCaloAtEMScale[Ntaujet1p3p-1]= (*tau1p3pItr)->parameter(TauJetParameters::etCaloAtEMScale);
		  m_h_taujet1p3p_etChargedHadCells[Ntaujet1p3p-1]= (*tau1p3pItr)->parameter(TauJetParameters::etChargedHadCells);
		  m_h_taujet1p3p_etOtherEMCells[Ntaujet1p3p-1]= (*tau1p3pItr)->parameter(TauJetParameters::etOtherEMCells);
		  m_h_taujet1p3p_etOtherHadCells[Ntaujet1p3p-1]= (*tau1p3pItr)->parameter(TauJetParameters::etOtherHadCells);
		  m_h_taujet1p3p_discriminant[Ntaujet1p3p-1]= (*tau1p3pItr)->parameter(TauJetParameters::discriminant);
		  m_h_taujet1p3p_discPDERS[Ntaujet1p3p-1]= (*tau1p3pItr)->parameter(TauJetParameters::discPDERS);	
		  m_h_taujet1p3p_discNN[Ntaujet1p3p-1]= (*tau1p3pItr)->parameter(TauJetParameters::discNN);
		  m_h_taujet1p3p_sumPTtracks[Ntaujet1p3p-1]= (*tau1p3pItr)->parameter(TauJetParameters::sumPTtracks);	
		  m_h_taujet1p3p_nAssocTracksCore[Ntaujet1p3p-1]= (*tau1p3pItr)->parameter(TauJetParameters::nAssocTracksCore);	
		  m_h_taujet1p3p_nAssocTracksIsol[Ntaujet1p3p-1]= (*tau1p3pItr)->parameter(TauJetParameters::nAssocTracksIsol);	
		  m_h_taujet1p3p_rWidth2Trk3P[Ntaujet1p3p-1]= (*tau1p3pItr)->parameter(TauJetParameters::rWidth2Trk3P); 	
		  m_h_taujet1p3p_massTrk3P[Ntaujet1p3p-1]= (*tau1p3pItr)->parameter(TauJetParameters::massTrk3P);	
		  m_h_taujet1p3p_signDOTrk3P[Ntaujet1p3p-1]= (*tau1p3pItr)->parameter(TauJetParameters::signDOTrk3P);
		}
	    }
	}
    }
  m_h_taujet1p3p_num = Ntaujet1p3p < max_taujet1p3p_num ? Ntaujet1p3p : max_taujet1p3p_num;

  mLog << MSG::INFO << "TauJet1p3p added to Ntuple. " << endreq;


  return StatusCode::SUCCESS;
}












StatusCode AODtoLiverpoolNtuple::UnpackTruthParticle()
{
  MsgStream mLog(messageService(), name());
  StatusCode status = StatusCode::SUCCESS;

  const TruthParticleContainer*  truthTES;
  status = m_storeGate->retrieve( truthTES, "SpclMC");
  if( status.isFailure()  ||  !truthTES ) {
    mLog << MSG::WARNING << "No SpclMC truth particle container found in TDS" << endreq; 
    return StatusCode::SUCCESS;
  }

  const IsGenerator isGenerator;
  mLog << MSG::DEBUG << "SpclMC Truth Container Successfully Retrieved" << endreq;
  TruthParticleContainer::const_iterator truthItr  = (*truthTES).begin();
  TruthParticleContainer::const_iterator truthItrE = (*truthTES).end();

  // Truth particle
  int Ntruth = 0;
  for (; truthItr != truthItrE ; ++truthItr) {  
    if ( !isGenerator((*truthItr)->getGenParticle()) )  continue;
    Ntruth++ ;
    
    if(Ntruth < max_truth_num ) {
      const HepMC::GenParticle* GeneratedParticle = (*truthItr)->genParticle();
      m_h_truth_pt[Ntruth-1]                   = (*truthItr)->pt();
      m_h_truth_eta[Ntruth-1]                  = (*truthItr)->eta();
      m_h_truth_phi[Ntruth-1]                  = (*truthItr)->phi();
      m_h_truth_e[Ntruth-1]                    = (*truthItr)->e();
      m_h_truth_pdgId[Ntruth-1]                = (*truthItr)->pdgId();
      m_h_truth_Barcode[Ntruth-1]              = GeneratedParticle->barcode();
      m_h_truth_Status[Ntruth-1]               = GeneratedParticle->status();
      m_h_truth_etisolation_cone10[Ntruth-1]   = (*truthItr)->etIsol(TruthParticleParameters::etcone10);
      m_h_truth_etisolation_cone20[Ntruth-1]   = (*truthItr)->etIsol(TruthParticleParameters::etcone20);
      m_h_truth_etisolation_cone30[Ntruth-1]   = (*truthItr)->etIsol(TruthParticleParameters::etcone30);
      m_h_truth_etisolation_cone40[Ntruth-1]   = (*truthItr)->etIsol(TruthParticleParameters::etcone40);
      m_h_truth_etisolation_cone45[Ntruth-1]   = (*truthItr)->etIsol(TruthParticleParameters::etcone);
      m_h_truth_etisolation_cone50[Ntruth-1]   = (*truthItr)->etIsol(TruthParticleParameters::etcone50);
      m_h_truth_etisolation_cone60[Ntruth-1]   = (*truthItr)->etIsol(TruthParticleParameters::etcone60);
      m_h_truth_etisolation_cone70[Ntruth-1]   = (*truthItr)->etIsol(TruthParticleParameters::etcone70);
      //      m_h_truth_NbrOfCones[Ntruth-1]           = (*truthItr)->getEtIsol(TruthParticleParameters::NbrOfCones);

      // Production vertex
      const HepMC::GenVertex* productionVertex = GeneratedParticle->production_vertex();
      if (productionVertex) {
	m_h_truth_hasProductionVertex[Ntruth-1]    = 1;
	m_h_truth_productionVertex_x[Ntruth-1]     = productionVertex->position().x();
	m_h_truth_productionVertex_y[Ntruth-1]     = productionVertex->position().y();
	m_h_truth_productionVertex_z[Ntruth-1]     = productionVertex->position().z();
	m_h_truth_productionVertex_t[Ntruth-1]     = productionVertex->position().t();
	m_h_truth_productionVertex_perp[Ntruth-1]  = productionVertex->position().perp();
	m_h_truth_productionVertex_eta[Ntruth-1]   = productionVertex->position().eta();
	m_h_truth_productionVertex_phi[Ntruth-1]   = productionVertex->position().phi();
	m_h_truth_productionVertex_r[Ntruth-1]     = productionVertex->position().rho();
      } else {
	m_h_truth_hasProductionVertex[Ntruth-1]    = 0;
      }

      // Decay vertex
      const HepMC::GenVertex* decayVertex = GeneratedParticle->end_vertex();
      if (decayVertex) {
	m_h_truth_hasDecayVertex[Ntruth-1]    = 1;
	m_h_truth_decayVertex_x[Ntruth-1]     = decayVertex->position().x();
	m_h_truth_decayVertex_y[Ntruth-1]     = decayVertex->position().y();
	m_h_truth_decayVertex_z[Ntruth-1]     = decayVertex->position().z();
	m_h_truth_decayVertex_t[Ntruth-1]     = decayVertex->position().t();
	m_h_truth_decayVertex_perp[Ntruth-1]  = decayVertex->position().perp();
	m_h_truth_decayVertex_eta[Ntruth-1]   = decayVertex->position().eta();
	m_h_truth_decayVertex_phi[Ntruth-1]   = decayVertex->position().phi();
	m_h_truth_decayVertex_r[Ntruth-1]     = decayVertex->position().rho();
      } else {
	m_h_truth_hasDecayVertex[Ntruth-1]    = 0;	  
      }

      // Loop over children and record barcodes/ID
      const unsigned int Nchildren   = (*truthItr)->nDecay();
      m_h_truth_child_num[Ntruth-1]  = Nchildren;
      for (unsigned int ichildren = 0; ichildren<Nchildren; ichildren++) {
	if (ichildren < max_truth_child_num) {
	  const HepMC::GenParticle* Child              = (*truthItr)->genChild(ichildren);
	  m_h_truth_child_Barcode[Ntruth-1][ichildren] = Child->barcode();
	  m_h_truth_child_pdgId[Ntruth-1][ichildren]   = Child->pdg_id();
	  ////////// Note: could also use (*truthItr)->pdgDecay(ichildren);
	  ////////// but other parent/child variables aren't implemeted like this
	}
      }

      // Loop over parents and record barcodes/ID
      const unsigned int Nparents     = (*truthItr)->nParents();
      m_h_truth_parent_num[Ntruth-1]  = Nparents;
      for (unsigned int iparents=0; iparents<Nparents; iparents++) {
	if (iparents < max_truth_parent_num) {
	  const HepMC::GenParticle* Parent             = (*truthItr)->genMother(iparents);
	  m_h_truth_parent_Barcode[Ntruth-1][iparents] = Parent->barcode();
	  m_h_truth_parent_pdgId[Ntruth-1][iparents]   = Parent->pdg_id();
	}
      }
    }
  }

  m_h_truth_num = Ntruth < max_truth_num ? Ntruth : max_truth_num;
  mLog << MSG::INFO << "Truth Particles added to Ntuple. " << endreq;

  return StatusCode::SUCCESS;
}
    
StatusCode AODtoLiverpoolNtuple::UnpackEtMiss()
{
  MsgStream mLog(messageService(), name());
  StatusCode status = StatusCode::SUCCESS;

  const MissingET* etmissTES;
  if (m_fastsim == 1) {
    status = m_storeGate->retrieve(etmissTES, "AtlfastMissingEt");
    if (status.isFailure()  ||  !etmissTES) {
      mLog << MSG::WARNING << "No atlfast AOD MissingEt container found in TDS" << endreq;
      return StatusCode::SUCCESS;
    }
  } else {
    status = m_storeGate->retrieve(etmissTES, "MET_Final");
    if (status.isFailure()  ||  !etmissTES) {
      mLog << MSG::WARNING << "No AOD MissingEt container found in TDS" << endreq;
      return StatusCode::SUCCESS;
    }
  }

  mLog << MSG::DEBUG << "MissingEt successfully retrieved" << endreq;

  // Et miss (do we want anything other than MET_Final)
  m_h_etmiss_final_ex      = (*etmissTES).etx();   
  m_h_etmiss_final_ey      = (*etmissTES).ety();  
  m_h_etmiss_final_sumet   = (*etmissTES).sumet();
  m_h_etmiss_final_phi     = (*etmissTES).phi();  
  m_h_etmiss_final_et      = (*etmissTES).et();   

  if (m_fastsim == 1) {  // Is this available for fast?
    m_h_etmiss_final_source = (*etmissTES).getSource();   

    //MissingEtRegions* regions = (*etmissTES).getRegions();
    //int regionIdx = regions->getRegions();
    //m_h_etmiss_final_region       = regionIdx;
    //m_h_etmiss_final_region_ex    = regions->exReg(regionIdx);
    //m_h_etmiss_final_region_ey    = regions->eyReg(regionIdx);
    //m_h_etmiss_final_region_etsum = regions->etSumReg(regionIdx);
  }

  mLog << MSG::INFO << "Etmiss added to Ntuple. " << endreq;
  return StatusCode::SUCCESS;
}
//ED start
StatusCode AODtoLiverpoolNtuple::UnpackEtMissRefFinal()
{
  MsgStream mLog(messageService(), name());
  StatusCode status = StatusCode::SUCCESS;

  const MissingET* etmissTES;
  if (m_fastsim == 1) {
    return StatusCode::SUCCESS;
  } else {
    status = m_storeGate->retrieve(etmissTES, "MET_RefFinal");
    if (status.isFailure()  ||  !etmissTES) {
      mLog << MSG::WARNING << "No AOD MissingEt container found in TDS" << endreq;
      return StatusCode::SUCCESS;
    }
  }
  
  mLog << MSG::DEBUG << "MissingEt successfully retrieved" << endreq;
  
  // Et miss (do we want anything other than MET_Final)
  m_h_etmiss_reffinal_sumet   = (*etmissTES).sumet();   
  m_h_etmiss_reffinal_ex      = (*etmissTES).etx();  
  m_h_etmiss_reffinal_ey      = (*etmissTES).ety();
  m_h_etmiss_reffinal_phi     = (*etmissTES).phi();  
  m_h_etmiss_reffinal_et      = (*etmissTES).et();   
  
  mLog << MSG::INFO << "EtmissRefFinal added to Ntuple. " << endreq;
  return StatusCode::SUCCESS;
}
 
StatusCode AODtoLiverpoolNtuple::UnpackEtMissLocHadTopoObj()
{
  MsgStream mLog(messageService(), name());
  StatusCode status = StatusCode::SUCCESS;

  const MissingET* etmissTES;
  if (m_fastsim == 1) {
    return StatusCode::SUCCESS;
  }
  else {
    status = m_storeGate->retrieve(etmissTES, "MET_LocHadTopoObj");
    if (status.isFailure()  ||  !etmissTES) {
      mLog << MSG::WARNING << "No AOD MissingEt container found in TDS" << endreq;
      return StatusCode::SUCCESS;
    }
  }

  mLog << MSG::DEBUG << "MissingEt successfully retrieved" << endreq;

  // Et miss (do we want anything other than MET_Final)
  m_h_etmiss_lochadtopoobj_sumet   = (*etmissTES).sumet();   
  m_h_etmiss_lochadtopoobj_ex      = (*etmissTES).etx();  
  m_h_etmiss_lochadtopoobj_ey      = (*etmissTES).ety();
  m_h_etmiss_lochadtopoobj_phi     = (*etmissTES).phi();  
  m_h_etmiss_lochadtopoobj_et      = (*etmissTES).et();   

  mLog << MSG::INFO << "EtmissLocHadTopoObj added to Ntuple. " << endreq;
  return StatusCode::SUCCESS;
}

StatusCode AODtoLiverpoolNtuple::UnpackEtMissObjFinal()
{
MsgStream mLog(messageService(), name());
StatusCode status = StatusCode::SUCCESS;

  const MissingET* etmissTES;
  if (m_fastsim == 1) {
      return StatusCode::SUCCESS;
  } else {
    status = m_storeGate->retrieve(etmissTES, "ObjMET_Final");
    if (status.isFailure()  ||  !etmissTES) {
      mLog << MSG::WARNING << "No AOD MissingEt container found in TDS" << endreq;
      return StatusCode::SUCCESS;
    }
  }
    
  mLog << MSG::DEBUG << "MissingEt successfully retrieved" << endreq;
  
  // Et miss (do we want anything other than MET_Final)
  m_h_etmiss_objfinal_sumet   = (*etmissTES).sumet();   
  m_h_etmiss_objfinal_ex      = (*etmissTES).etx();  
  m_h_etmiss_objfinal_ey      = (*etmissTES).ety();
  m_h_etmiss_objfinal_phi     = (*etmissTES).phi();  
  m_h_etmiss_objfinal_et      = (*etmissTES).et(); 
  
  mLog << MSG::INFO << "EtmissObjFinal added to Ntuple. " << endreq;
  return StatusCode::SUCCESS;
}
//ED end

StatusCode AODtoLiverpoolNtuple::UnpackTruthEtMiss()
{
  MsgStream mLog(messageService(), name());
  StatusCode status = StatusCode::SUCCESS;

  const MissingEtTruth* truthetmissTES;
  status = m_storeGate->retrieve( truthetmissTES, "MET_Truth");
  if (status.isFailure()  ||  !truthetmissTES) {
    mLog << MSG::WARNING << "No AOD MissingEt Truth container found in TDS" << endreq;
    return StatusCode::SUCCESS;
  }

  mLog << MSG::DEBUG << "MissingEtTruth successfully retrieved" << endreq;

  // Et miss truth
  m_h_etmiss_true_ex    = (*truthetmissTES).etx();	 
  m_h_etmiss_true_ey    = (*truthetmissTES).ety();	
  m_h_etmiss_true_sumet = (*truthetmissTES).sumet();
  m_h_etmiss_true_phi   = (*truthetmissTES).phi(); 	
  m_h_etmiss_true_et    = (*truthetmissTES).et();   

  mLog << MSG::INFO << "Etmiss truth added to Ntuple. " << endreq;
  return StatusCode::SUCCESS;
}

StatusCode AODtoLiverpoolNtuple::UnpackTrack()
{
  MsgStream mLog(messageService(), name());
  StatusCode status = StatusCode::SUCCESS;


  const Rec::TrackParticleContainer* trackTES;
  if (m_fastsim == 1) {
    status = m_storeGate->retrieve(trackTES, "AtlfastTrackParticles");
    if (status.isFailure()  ||  !trackTES) {
      mLog << MSG::WARNING << "No AOD TrackParticles container found in TDS" << endreq;
      return StatusCode::SUCCESS;
    }
  } else {
    status = m_storeGate->retrieve(trackTES, "TrackParticleCandidate");
    if (status.isFailure()  ||  !trackTES) {
      mLog << MSG::WARNING << "No AOD TrackParticles container found in TDS" << endreq;
      return StatusCode::SUCCESS;
    }
  }

  mLog << MSG::DEBUG << "TrackParticles successfully retrieved" << endreq;

  Rec::TrackParticleContainer::const_iterator trackItr  = (*trackTES).begin();
  Rec::TrackParticleContainer::const_iterator trackItrE = (*trackTES).end();

  // Track
  int Ntracs = 0;
  for (; trackItr != trackItrE; ++trackItr) {
    const Rec::TrackParticle *trackParticle = (*trackItr);
    Ntracs++ ;

    if(Ntracs < max_track_num) {

      //m_h_track_px[Ntracs-1]     = trackParticle->px();	
      //m_h_track_py[Ntracs-1]     = trackParticle->py();	
      //m_h_track_pz[Ntracs-1]     = trackParticle->pz();	
      //m_h_track_p[Ntracs-1]      = trackParticle->p();	
      m_h_track_pt[Ntracs-1]     = trackParticle->pt();	
      m_h_track_charge[Ntracs-1] = trackParticle->charge();
      m_h_track_eta[Ntracs-1]    = trackParticle->eta();	
      m_h_track_phi[Ntracs-1]    = trackParticle->phi();   

      if (m_fastsim == 0) {
 	m_h_track_ndf[Ntracs-1]                  = (*trackItr)->fitQuality()->numberDoF();
 	m_h_track_chi2[Ntracs-1]                 = (*trackItr)->fitQuality()->chiSquared();

	// Track parameters
	const Trk::MeasuredPerigee* perigee = (*trackItr)->measuredPerigee();
	if (perigee) {
	  m_h_track_d0[Ntracs-1]                   = perigee->parameters()[Trk::d0];
	  m_h_track_z0[Ntracs-1]                   = perigee->parameters()[Trk::z0];
	  m_h_track_perigee_par0[Ntracs-1]         = perigee->parameters()[0];
	  m_h_track_perigee_par1[Ntracs-1]         = perigee->parameters()[1];
	  m_h_track_perigee_par2[Ntracs-1]         = perigee->parameters()[2];
	  m_h_track_perigee_par3[Ntracs-1]         = perigee->parameters()[3];
	  m_h_track_perigee_par4[Ntracs-1]         = perigee->parameters()[4];
	  m_h_track_perigee_error11[Ntracs-1]      = perigee->localErrorMatrix().covariance().fast(1,1);
	  m_h_track_perigee_error22[Ntracs-1]      = perigee->localErrorMatrix().covariance().fast(2,2);
	  m_h_track_perigee_error33[Ntracs-1]      = perigee->localErrorMatrix().covariance().fast(3,3);
	  m_h_track_perigee_error44[Ntracs-1]      = perigee->localErrorMatrix().covariance().fast(4,4);
	  m_h_track_perigee_error55[Ntracs-1]      = perigee->localErrorMatrix().covariance().fast(5,5);
	}

	m_h_track_origin[Ntracs-1]               = trackParticle->particleOriginType();

	const Trk::TrackSummary* summary = trackParticle->trackSummary();
	long hitMask  = 0;
	if (summary) {
	  m_h_track_NumBLayerHits[Ntracs-1]        = summary->get(Trk::numberOfBLayerHits);	    
	  m_h_track_NumPixelHits[Ntracs-1]         = summary->get(Trk::numberOfPixelHits);	   
	  m_h_track_NumPixelHoles[Ntracs-1]        = summary->get(Trk::numberOfPixelHoles);	   
	  m_h_track_NumSCTHits[Ntracs-1]           = summary->get(Trk::numberOfSCTHits);		   
	  m_h_track_NumSCTHoles[Ntracs-1]          = summary->get(Trk::numberOfSCTHoles);		   
	  m_h_track_NumTRTHits[Ntracs-1]           = summary->get(Trk::numberOfTRTHits);		   
	  m_h_track_NumTRTHighThreshHits[Ntracs-1] = summary->get(Trk::numberOfTRTHighThresholdHits);
	  //m_h_track_NumTRTHoles[Ntracs-1]          = -999;

	  // Hit mask
	  if (summary->isHit(Trk::pixelBarrel0)) hitMask |= 1;
	  if (summary->isHit(Trk::pixelBarrel1)) hitMask |= 2;
	  if (summary->isHit(Trk::pixelBarrel2)) hitMask |= 4;
	  if (summary->isHit(Trk::pixelEndCap0)) hitMask |= 8;
	  if (summary->isHit(Trk::pixelEndCap1)) hitMask |= 0x10;
	  if (summary->isHit(Trk::pixelEndCap2)) hitMask |= 0x20;
	  if (summary->isHit(Trk::sctBarrel0)) hitMask |= 0x40;
	  if (summary->isHit(Trk::sctBarrel1)) hitMask |= 0x80;
	  if (summary->isHit(Trk::sctBarrel2)) hitMask |= 0x100;
	  if (summary->isHit(Trk::sctBarrel3)) hitMask |= 0x200;
	  if (summary->isHit(Trk::sctEndCap0)) hitMask |= 0x400;
	  if (summary->isHit(Trk::sctEndCap1)) hitMask |= 0x800;
	  if (summary->isHit(Trk::sctEndCap2)) hitMask |= 0x1000;
	  if (summary->isHit(Trk::sctEndCap3)) hitMask |= 0x2000;
	  if (summary->isHit(Trk::sctEndCap4)) hitMask |= 0x4000;
	  if (summary->isHit(Trk::sctEndCap5)) hitMask |= 0x8000;
	  if (summary->isHit(Trk::sctEndCap6)) hitMask |= 0x10000;
	  if (summary->isHit(Trk::sctEndCap7)) hitMask |= 0x20000;
	  if (summary->isHit(Trk::sctEndCap8)) hitMask |= 0x40000;
	  if (summary->isHit(Trk::trtBarrel)) hitMask |= 0x80000;
	  if (summary->isHit(Trk::trtEndCap)) hitMask |= 0x100000;

	  m_h_track_hitMask[Ntracs-1] = hitMask;
	} 

	//	else {
	//  m_h_track_NumBLayerHits[Ntracs-1]        = -999;
	//  m_h_track_NumPixelHits[Ntracs-1]         = -999;
	//  m_h_track_NumPixelHoles[Ntracs-1]        = -999;
	//  m_h_track_NumSCTHits[Ntracs-1]           = -999;
	//  m_h_track_NumSCTHoles[Ntracs-1]          = -999;
	//  m_h_track_NumTRTHits[Ntracs-1]           = -999;
	//  m_h_track_NumTRTHighThreshHits[Ntracs-1] = -999;
	//  m_h_track_NumTRTHoles[Ntracs-1]          = -999;
	//}

      } 
      //else {
      //m_h_track_d0[Ntracs-1]                   = -999.0;
      //m_h_track_z0[Ntracs-1]                   = -999.0;
      //m_h_track_origin[Ntracs-1]               = -999;
      //}
    }
  }

  m_h_track_num = Ntracs < max_track_num ? Ntracs : max_track_num;
  mLog << MSG::INFO << "Tracks added to Ntuple. " << endreq;

  return StatusCode::SUCCESS;
}

StatusCode AODtoLiverpoolNtuple::UnpackJet()
{
  MsgStream mLog(messageService(), name());
  StatusCode status = StatusCode::SUCCESS;

  const ParticleJetContainer* conTES;
  if (m_fastsim == 1) {
    status = m_storeGate->retrieve(conTES,"AtlfastParticleJetContainer");
    if (status.isFailure()  ||  !conTES) {
      mLog << MSG::WARNING << "No Atlfast AOD ConeJet container found in TDS" << endreq;
      return StatusCode::SUCCESS;
    }
  } else {
    status = m_storeGate->retrieve(conTES,"ConeTowerParticleJets");
    if (status.isFailure()  ||  !conTES) {
      mLog << MSG::WARNING << "No AOD ConeJet container found in TDS" << endreq;
      return StatusCode::SUCCESS;
    }
  }

  mLog << MSG::DEBUG << " ConeJets successfully retrieved" << endreq;

  ParticleJetContainer::const_iterator coneItr  = (*conTES).begin();
  ParticleJetContainer::const_iterator coneItrE = (*conTES).end();

  // Cone jets
  int Ncone = 0;
  for (; coneItr != coneItrE; ++coneItr) {  
    Ncone++ ;
    if(Ncone < max_jet_num) {
      // General

      //m_h_cone_px[Ncone-1] = (*coneItr)->px(); 
      //m_h_cone_py[Ncone-1] = (*coneItr)->py(); 
      //m_h_cone_pz[Ncone-1] = (*coneItr)->pz(); 
      m_h_cone_pt[Ncone-1] = (*coneItr)->pt(); 
      m_h_cone_eta[Ncone-1]= (*coneItr)->eta();
      m_h_cone_phi[Ncone-1]= (*coneItr)->phi();


      // Btag
      // Weights + likelihoods
      m_h_cone_lhsig[Ncone-1]     = (*coneItr)->lhSig();  // Fast: 0 for non-tagged, 1 for tagged
      m_h_cone_w[Ncone-1]         = (*coneItr)->weight(); // Fast -30 for non-tagged, 100 for tagged                      


      if (m_fastsim == 0) {
	m_h_cone_wIP2D[Ncone-1]     = (*coneItr)->weightForTag("IP2D");	    
	m_h_cone_wIP3D[Ncone-1]     = (*coneItr)->weightForTag("IP3D");	    
	m_h_cone_wSV1[Ncone-1]      = (*coneItr)->weightForTag("SV1");	    
	m_h_cone_wSV2[Ncone-1]      = (*coneItr)->weightForTag("SV2");	    
	m_h_cone_wLT1D[Ncone-1]     = (*coneItr)->weightForTag("LifeTimeTag1D");  
	m_h_cone_wLT2D[Ncone-1]     = (*coneItr)->weightForTag("LifeTimeTag2D");  
	//m_h_cone_wLT3D[Ncone-1]     = (*coneItr)->weightForTag("LifeTimeTag3D");  
	m_h_cone_SVBU[Ncone-1]      = (*coneItr)->weightForTag("SecVtxTagBU");    
	//m_h_cone_SVTD[Ncone-1]      = (*coneItr)->weightForTag("SecVtxTagTD");    
	m_h_cone_wLHSig[Ncone-1]    = (*coneItr)->weightForTag("lhSig");          
	m_h_cone_wSoftMu[Ncone-1]   = (*coneItr)->weightForTag("SoftMuonTag");          
	m_h_cone_wSoftElec[Ncone-1] = (*coneItr)->weightForTag("SoftMuonTag");          
	m_h_cone_wJP[Ncone-1]       = (*coneItr)->weightForTag("JetProbTag");          

	// Store indicies of tracks used
	std::vector<const Rec::TrackParticle*>* trackVector = (*coneItr)->constituent<Analysis::TrackConstituents>("Tracks")->tracks();
	int nTracks = trackVector->size();	
	m_h_cone_ntracks[Ncone -1] = nTracks;

	int imatch = 0;
	for (int i = 0; i < nTracks; i++) {	  
	  const Rec::TrackParticle *trackInJet = trackVector->at(i);
	  const Rec::TrackParticleContainer* trackTES;
	  status = m_storeGate->retrieve(trackTES, "TrackParticleCandidate");
	  if (status.isFailure()  ||  !trackTES) {
	    mLog << MSG::WARNING << "No AOD TrackParticles container found in TDS" << endreq;
	    return StatusCode::SUCCESS;
	  }
	  mLog << MSG::DEBUG << "TrackParticles successfully retrieved" << endreq;
	  
	  Rec::TrackParticleContainer::const_iterator trackItr  = (*trackTES).begin();
	  Rec::TrackParticleContainer::const_iterator trackItrE = (*trackTES).end();
	  
	  int Ntracks = 0, NMatch = 0;
	  for (; trackItr != trackItrE; ++trackItr) {
	    Ntracks++;
	    if(Ntracks < max_track_num) {
	      const Rec::TrackParticle *trackParticle = (*trackItr);
	      if (trackInJet == trackParticle) {
		m_h_cone_trackidx[Ncone-1] [imatch] = Ntracks-1;
		//cout << "CONE(Z) - > " << trackInJet->eta() << " " << trackInJet->phi() << " vs " << m_h_track_eta[m_h_cone_trackidx[Ncone-1] [imatch]] << " " 
		//     << m_h_track_phi[m_h_cone_trackidx[Ncone-1] [imatch]] << endl;
		imatch++; NMatch++;
	      }
	    }
	  }
	  
	  if (NMatch > 1) mLog << MSG::WARNING << "Track used in cone has more than one match in track container!" << endreq;
	  else if (NMatch == 0) mLog << MSG::WARNING << "Track used in cone has no match in track container!" << endreq;
	  
	}	

	// Truth
	std::string label("N/A");  
	const Analysis::TruthInfo* mcinfo = (*coneItr)->tagInfo<Analysis::TruthInfo>("TruthInfo");
	if (mcinfo) label = mcinfo->jetTruthLabel();

	int flav(0);      
	if(label == "B") flav = 5;    // b-jet
	if(label == "C") flav = 4;    // c-jet
	if(label == "T") flav = 15;   // tau

	m_h_cone_truth[Ncone-1] = flav;

	// SV1
	const Analysis::SVInfoBase* infoSV1 = (*coneItr)->tagInfo<Analysis::SVInfoBase>("SV1");
	if (infoSV1) { // if these aren't true what will be stored ?  Might cause a problem.
	  m_h_cone_sv1_pu[Ncone-1] = infoSV1->tagLikelihood()[0];
	  m_h_cone_sv1_pb[Ncone-1] = infoSV1->tagLikelihood()[1];
	} else {
	  m_h_cone_sv1_pu[Ncone-1] = -999.9;
	  m_h_cone_sv1_pb[Ncone-1] = -999.9;
	}

	// SV2
	const Analysis::SVInfoBase* infoSV2 = (*coneItr)->tagInfo<Analysis::SVInfoBase>("SV2");
	if (infoSV2) {
	  m_h_cone_sv2_pu[Ncone-1] = infoSV2->tagLikelihood()[0];
	  m_h_cone_sv2_pb[Ncone-1] = infoSV2->tagLikelihood()[1];
	} else {
	  m_h_cone_sv2_pu[Ncone-1] = -999.9;
	  m_h_cone_sv2_pb[Ncone-1] = -999.9;
	}

	// SVPlus
	const Analysis::SVInfoPlus* infoSVPlus = (*coneItr)->tagInfo<Analysis::SVInfoPlus>("SVInfoPlus");
	if (infoSVPlus) {
	  m_h_cone_svplus_n2t [Ncone-1]     = infoSVPlus->getN2T();							       
	  m_h_cone_svplus_eFrac[Ncone-1]    = infoSVPlus->getEnergyFraction();						       
	  m_h_cone_svplus_mass[Ncone-1]     = infoSVPlus->getMass();							       
	  m_h_cone_svplus_normDist[Ncone-1] = infoSVPlus->getNormDist();						       
	  m_h_cone_svplus_NGTracks[Ncone-1] = infoSVPlus->getNGTrackInSvx();			       
	  m_h_cone_svplus_chi2[Ncone-1]     = infoSVPlus->getRecSvx().chi2();
	  m_h_cone_svplus_ndf[Ncone-1]      = infoSVPlus->getRecSvx().ndf();
	  m_h_cone_svplus_x[Ncone-1]        = infoSVPlus->getRecSvx().position().x();                                             
	  m_h_cone_svplus_y [Ncone-1]       = infoSVPlus->getRecSvx().position().y();                                             
	  m_h_cone_svplus_z [Ncone-1]       = infoSVPlus->getRecSvx().position().z();                                             	

	  int ntrk = infoSVPlus->numTrackInfo();  
	  if (ntrk > max_jettrack_num) ntrk = max_jettrack_num;
	  m_h_cone_svplus_ntracks[Ncone-1]  = ntrk;

	  int imatch = 0;
	  for(int i = 0; i < ntrk; i++) {
	    Analysis::SVTrackInfo trackInfo = infoSVPlus->getTrackInfo(i);
	    
	    // Store indicies of tracks used
	    const Rec::TrackParticle *trackInJet = trackInfo.track();
	    const Rec::TrackParticleContainer* trackTES;
	    status = m_storeGate->retrieve(trackTES, "TrackParticleCandidate");
	    if (status.isFailure()  ||  !trackTES) {
	      mLog << MSG::WARNING << "No AOD TrackParticles container found in TDS" << endreq;
	      return StatusCode::SUCCESS;
	    }
	    mLog << MSG::DEBUG << "TrackParticles successfully retrieved" << endreq;
	    
	    Rec::TrackParticleContainer::const_iterator trackItr  = (*trackTES).begin();
	    Rec::TrackParticleContainer::const_iterator trackItrE = (*trackTES).end();
	    
	    int Ntracks = 0, NMatch = 0;
	    for (; trackItr != trackItrE; ++trackItr) {
	      Ntracks++;
	      if(Ntracks < max_track_num) {
		const Rec::TrackParticle *trackParticle = (*trackItr);
		if (trackInJet == trackParticle) {
		  m_h_cone_svplus_trackidx[Ncone-1] [imatch] = Ntracks-1;
		  //cout << "CONE(Z) - > " << trackInJet->eta() << " " << trackInJet->phi() << " vs " << m_h_track_eta[m_h_cone_svplus_trackidx[Ncone-1] [imatch]] << " " 
		  //   << m_h_track_phi[m_h_cone_svplus_trackidx[Ncone-1] [imatch]] << endl;
		  imatch++; NMatch++;
		}
	      }
	    }
	  
	    if (NMatch > 1) mLog << MSG::WARNING << "Track used in cone SV tagger has more than one match in track container!" << endreq;
	    else if (NMatch == 0) mLog << MSG::WARNING << "Track used in cone SV tagger has no match in track container!" << endreq;
	  }

	} else {
	  m_h_cone_svplus_n2t [Ncone-1]     = -999;
	  m_h_cone_svplus_eFrac[Ncone-1]    = -999.9;
	  m_h_cone_svplus_mass[Ncone-1]     = -999.9;
	  m_h_cone_svplus_normDist[Ncone-1] = -999.9;
	  m_h_cone_svplus_NGTracks[Ncone-1] = -999;
	  m_h_cone_svplus_chi2[Ncone-1]     = -999.9;
	  m_h_cone_svplus_ndf[Ncone-1]      = -999;
	  m_h_cone_svplus_x[Ncone-1]        = -999.9;
	  m_h_cone_svplus_y [Ncone-1]       = -999.9;
	  m_h_cone_svplus_z [Ncone-1]       = -999.9;
	}

	// SVBU
	const Analysis::SecVtxInfo* infoSVBU = (*coneItr)->tagInfo<Analysis::SecVtxInfo>("SecVtxTagBU");
	if (infoSVBU) {
	  m_h_cone_svbu_pu[Ncone-1]        = infoSVBU->tagLikelihood()[0];
	  m_h_cone_svbu_pb[Ncone-1]        = infoSVBU->tagLikelihood()[1];
	  m_h_cone_svbu_eFrac[Ncone-1]     = infoSVBU->energyFraction();							     
	  m_h_cone_svbu_mass[Ncone-1]      = infoSVBU->mass();								     
	  m_h_cone_svbu_normDist[Ncone-1]  = infoSVBU->distance();								     
	  m_h_cone_svbu_mult[Ncone-1]      = infoSVBU->mult();						     
	  m_h_cone_svbu_chi2[Ncone-1]      = infoSVBU->secVertexPos().chi2();
	  m_h_cone_svbu_ndf[Ncone-1]       = infoSVBU->secVertexPos().ndf();
	  m_h_cone_svbu_x[Ncone-1]         = infoSVBU->secVertexPos().position().x();                                                
	  m_h_cone_svbu_y[Ncone-1]         = infoSVBU->secVertexPos().position().y();                                                
	  m_h_cone_svbu_z[Ncone-1]         = infoSVBU->secVertexPos().position().z();                                                
	} else {
	  m_h_cone_svbu_pu[Ncone-1]        =  -999.9;
	  m_h_cone_svbu_pb[Ncone-1]        =  -999.9;
	  m_h_cone_svbu_eFrac[Ncone-1]     =  -999.9;
	  m_h_cone_svbu_mass[Ncone-1]      =  -999.9;
	  m_h_cone_svbu_normDist[Ncone-1]  =  -999.9;
	  m_h_cone_svbu_mult[Ncone-1]      =  -999;
	  m_h_cone_svbu_chi2[Ncone-1]      =  -999.9;
	  m_h_cone_svbu_ndf[Ncone-1]       =  -999;
	  m_h_cone_svbu_x[Ncone-1]         =  -999.9;
	  m_h_cone_svbu_y[Ncone-1]         =  -999.9;
	  m_h_cone_svbu_z[Ncone-1]         =  -999.9;
	}

	// IP2D
	const Analysis::IPInfoBase* info2D = (*coneItr)->tagInfo<Analysis::IPInfoBase>("IP2D");
	if (info2D) {
	  m_h_cone_ip2d_pu[Ncone-1]        = info2D->tagLikelihood()[0];
	  m_h_cone_ip2d_pb[Ncone-1]        = info2D->tagLikelihood()[1];
	  m_h_cone_ip2d_numTracks[Ncone-1] = info2D->nbTracks();  
	} else {
	  m_h_cone_ip2d_pu[Ncone-1]        = -999.9;
	  m_h_cone_ip2d_pb[Ncone-1]        = -999.9;
	  m_h_cone_ip2d_numTracks[Ncone-1] = -999;
	}

	// IP3D
	const Analysis::IPInfoBase* info3D = (*coneItr)->tagInfo<Analysis::IPInfoBase>("IP3D");
	if (info3D) {
	  m_h_cone_ip3d_pu[Ncone-1]        = info3D->tagLikelihood()[0];
	  m_h_cone_ip3d_pb[Ncone-1]        = info3D->tagLikelihood()[1];
	  m_h_cone_ip3d_numTracks[Ncone-1] = info3D->nbTracks();
	} else {
	  m_h_cone_ip3d_pu[Ncone-1]        = -999.9;
	  m_h_cone_ip3d_pb[Ncone-1]        = -999.9;
	  m_h_cone_ip3d_numTracks[Ncone-1] = -999;
	}

	// IPPlus
	const Analysis::IPInfoPlus* infoIPPlus = (*coneItr)->tagInfo<Analysis::IPInfoPlus>("IPInfoPlus");
	if (infoIPPlus) {
	  int ntrk = infoIPPlus->numTrackInfo();  // Is this the same as nbtrack? If not might need to store
	  if (ntrk > max_jettrack_num) ntrk = max_jettrack_num;

	  int imatch = 0;
	  for(int i = 0; i < ntrk; i++) {
	    Analysis::IPTrackInfo trackInfo = infoIPPlus->getTrackInfo(i);

	    Analysis::TrackGrade grade = trackInfo.trackGrade(); // Good, Shared, ...
	    std::string gradeStr = static_cast<std::string>(grade);
	    int gradeInt = -1;
	    if (gradeStr == "Good")   gradeInt = 0; 
	    else if (gradeStr == "Shared") gradeInt = 1; 

	    m_h_cone_ipplus_d0[Ncone-1] [i]          = trackInfo.d0Value();		
	    m_h_cone_ipplus_d0Sig[Ncone-1] [i]       = trackInfo.d0Significance();
	    m_h_cone_ipplus_z0[Ncone-1] [i]          = trackInfo.z0Value();	
	    m_h_cone_ipplus_z0Sig[Ncone-1] [i]       = trackInfo.z0Significance();
	    m_h_cone_ipplus_trackW2D[Ncone-1] [i]    = trackInfo.trackWeight2D(); 
	    m_h_cone_ipplus_trackW3D[Ncone-1] [i]    = trackInfo.trackWeight3D(); 
	    m_h_cone_ipplus_isFromV0[Ncone-1] [i]    = trackInfo.isFromV0();	
	    m_h_cone_ipplus_trackJP[Ncone-1] [i]     = trackInfo.trackProbJP();   
	    m_h_cone_ipplus_trackGrade[Ncone-1] [i]  = gradeInt;

	    // Store indicies of tracks used
	    const Rec::TrackParticle *trackInJet = trackInfo.track();
	    const Rec::TrackParticleContainer* trackTES;
	    status = m_storeGate->retrieve(trackTES, "TrackParticleCandidate");
	    if (status.isFailure()  ||  !trackTES) {
	      mLog << MSG::WARNING << "No AOD TrackParticles container found in TDS" << endreq;
	      return StatusCode::SUCCESS;
	    }
	    mLog << MSG::DEBUG << "TrackParticles successfully retrieved" << endreq;
	    
	    Rec::TrackParticleContainer::const_iterator trackItr  = (*trackTES).begin();
	    Rec::TrackParticleContainer::const_iterator trackItrE = (*trackTES).end();

	    int Ntracks = 0, NMatch = 0;
	    for (; trackItr != trackItrE; ++trackItr) {
	      Ntracks++;
	      if(Ntracks < max_track_num) {
		const Rec::TrackParticle *trackParticle = (*trackItr);
		if (trackInJet == trackParticle) {
		  m_h_cone_ipplus_trackidx[Ncone-1] [imatch] = Ntracks-1;
		  //cout << "CONE(Z) - > " << trackInJet->eta() << " " << trackInJet->phi() << " vs " << m_h_track_eta[m_h_cone_ipplus_trackidx[Ncone-1] [imatch]] << " " 
		  //     << m_h_track_phi[m_h_cone_ipplus_trackidx[Ncone-1] [imatch]] << endl;
		  imatch++; NMatch++;
		}
	      }
	    }

	    if (NMatch > 1) mLog << MSG::WARNING << "Track used in cone IP tagger has more than one match in track container!" << endreq;
	    else if (NMatch == 0) mLog << MSG::WARNING << "Track used in cone IP tagger has no match in track container!" << endreq;

	  }
	}

	// LT1D
	const Analysis::LifetimeInfo* infoLT1D = (*coneItr)->tagInfo<Analysis::LifetimeInfo>("LifetimeTag1D");
	if (infoLT1D) {
	  int ntrk =  infoLT1D->significance().size();
	  m_h_cone_lt1d_numTracks[Ncone-1] = ntrk;

	  if (ntrk > max_jettrack_num) ntrk = max_jettrack_num;

	  for(int i = 0; i < ntrk; i++) {
	    m_h_cone_lt1d_z0[Ncone-1] [i]    = infoLT1D->signedIP()[i];
	    m_h_cone_lt1d_z0Sig[Ncone-1] [i] = infoLT1D->significance()[i];
	  }
	} else {
	  m_h_cone_lt1d_numTracks[Ncone-1] = -999;
	}

	// LT2D
	const Analysis::LifetimeInfo* infoLT2D = (*coneItr)->tagInfo<Analysis::LifetimeInfo>("LifetimeTag2D");
	if (infoLT2D) {
	  int ntrk =  infoLT2D->significance().size();
	  m_h_cone_lt2d_numTracks[Ncone-1] = ntrk;

	  if (ntrk > max_jettrack_num) ntrk = max_jettrack_num;

	  for(int i = 0; i < ntrk; i++) {
	    m_h_cone_lt2d_d0[Ncone-1] [i]    = infoLT2D->signedIP()[i];
	    m_h_cone_lt2d_d0Sig[Ncone-1] [i] = infoLT2D->significance()[i];
	  }
	} else {
	  m_h_cone_lt2d_numTracks[Ncone-1] = -999;
	}

	// Soft muon
	const Analysis::SoftMuonInfo* infoMu = (*coneItr)->tagInfo<Analysis::SoftMuonInfo>("SoftMuonTag");
	if (infoMu) {
	  int ntrk = infoMu->numTrackInfo();
	  m_h_cone_softmu_numTracks[Ncone-1] = ntrk;
	
	  if (ntrk > max_jettrack_num) ntrk = max_jettrack_num;
	  for(int i = 0; i < ntrk; i++) {
	    Analysis::SMTrackInfo trackInfo = infoMu->getTrackInfo(i);
	    m_h_cone_softmu_d0[Ncone-1] [i] = trackInfo.d0Value();
	    m_h_cone_softmu_ptrel[Ncone-1] [i] = trackInfo.pTrel();
	    // what do about muon ptr?
	  }
	} else {
	  m_h_cone_softmu_numTracks[Ncone-1] = -999;
	}

	// Soft elec
	const Analysis::SoftElectronInfo* infoElec = (*coneItr)->tagInfo<Analysis::SoftElectronInfo>("SoftElectronTag");
	if (infoElec) {
	  int ntrk = infoElec->numTrackInfo();
	  m_h_cone_softelec_numTracks[Ncone-1] = ntrk;
	
	  if (ntrk > max_jettrack_num) ntrk = max_jettrack_num;
	  for(int i = 0; i < ntrk; i++) {
	    Analysis::SETrackInfo trackInfo = infoElec->getTrackInfo(i);
	    m_h_cone_softelec_d0[Ncone-1] [i] = trackInfo.d0Value();
	    m_h_cone_softelec_ptrel[Ncone-1] [i] = trackInfo.pTrel();
	    // what do about electron ptr?
	  }
	} else {
	  m_h_cone_softelec_numTracks[Ncone-1] = -999;
	}

	// Jet prob info is now written to IPInfoPlus
      }
    }
  }
  
  m_h_cone_num = Ncone < max_jet_num ? Ncone : max_jet_num;
  //m_h_cone_num = max_jet_num;
  //if (Ncone < max_jet_num) m_h_cone_num = Ncone;
  mLog << MSG::INFO << "Cone Jets added to Ntuple. " << endreq;

  if (m_fastsim == 1) {   // Is this still true?
    // for Atlfast data we are done now
    m_h_cone_num  = 0; // no Cone4 jets in Altfast data
    m_h_kt4jet_num = 0; // no Kt jets in Altfast data
    return StatusCode::SUCCESS;
  }    


  // Cone 4 jets
  const ParticleJetContainer* cone4TES;  
  status = m_storeGate->retrieve( cone4TES, "Cone4TowerParticleJets"); 
  if( status.isFailure()  ||  !cone4TES ) {
    mLog << MSG::WARNING << "No AOD Cone4Jet container found in TDS" << endreq;
    return StatusCode::SUCCESS;
  }
  mLog << MSG::DEBUG << "Cone4Jets successfully retrieved" << endreq;

  ParticleJetContainer::const_iterator cone4Itr  = (*cone4TES).begin();
  ParticleJetContainer::const_iterator cone4ItrE = (*cone4TES).end();

  int Ncone4 = 0;
  for (; cone4Itr != cone4ItrE; ++cone4Itr) {
    Ncone4++ ;
    if(Ncone4 < max_jet_num) {
      //m_h_cone4_px[Ncone4-1] = (*cone4Itr)->px(); 
      //m_h_cone4_py[Ncone4-1] = (*cone4Itr)->py(); 
      //m_h_cone4_pz[Ncone4-1] = (*cone4Itr)->pz(); 
      m_h_cone4_pt[Ncone4-1] = (*cone4Itr)->pt(); 
      m_h_cone4_eta[Ncone4-1]= (*cone4Itr)->eta();
      m_h_cone4_phi[Ncone4-1]= (*cone4Itr)->phi();

      // Btag
      // Weights
      m_h_cone4_lhsig[Ncone4-1]      = (*cone4Itr)->lhSig();
      m_h_cone4_w[Ncone4-1]         = (*cone4Itr)->weight();                       
      m_h_cone4_wIP2D[Ncone4-1]     = (*cone4Itr)->weightForTag("IP2D");	    
      m_h_cone4_wIP3D[Ncone4-1]     = (*cone4Itr)->weightForTag("IP3D");	    
      m_h_cone4_wSV1[Ncone4-1]      = (*cone4Itr)->weightForTag("SV1");	    
      m_h_cone4_wSV2[Ncone4-1]      = (*cone4Itr)->weightForTag("SV2");	    
      m_h_cone4_wLT1D[Ncone4-1]     = (*cone4Itr)->weightForTag("LifeTimeTag1D");  
      m_h_cone4_wLT2D[Ncone4-1]     = (*cone4Itr)->weightForTag("LifeTimeTag2D");  
      //m_h_cone4_wLT3D[Ncone4-1]     = (*cone4Itr)->weightForTag("LifeTimeTag3D");  
      m_h_cone4_SVBU[Ncone4-1]      = (*cone4Itr)->weightForTag("SecVtxTagBU");    
      //m_h_cone4_SVTD[Ncone4-1]      = (*cone4Itr)->weightForTag("SecVtxTagTD");    
      m_h_cone4_wLHSig[Ncone4-1]    = (*cone4Itr)->weightForTag("lhSig");          
      m_h_cone4_wSoftMu[Ncone4-1]   = (*cone4Itr)->weightForTag("SoftMuonTag");          
      m_h_cone4_wSoftElec[Ncone4-1] = (*cone4Itr)->weightForTag("SoftMuonTag");          
      m_h_cone4_wJP[Ncone4-1]       = (*cone4Itr)->weightForTag("JetProbTag");          

      // Store indicies of tracks used
      std::vector<const Rec::TrackParticle*>* trackVector = (*cone4Itr)->constituent<Analysis::TrackConstituents>("Tracks")->tracks();
      int nTracks = trackVector->size();	
      m_h_cone4_ntracks[Ncone4 -1] = nTracks;
      
      int imatch = 0;
      for (int i = 0; i < nTracks; i++) {	  
	const Rec::TrackParticle *trackInJet = trackVector->at(i);
	const Rec::TrackParticleContainer* trackTES;
	status = m_storeGate->retrieve(trackTES, "TrackParticleCandidate");
	if (status.isFailure()  ||  !trackTES) {
	  mLog << MSG::WARNING << "No AOD TrackParticles container found in TDS" << endreq;
	  return StatusCode::SUCCESS;
	}
	mLog << MSG::DEBUG << "TrackParticles successfully retrieved" << endreq;
	
	Rec::TrackParticleContainer::const_iterator trackItr  = (*trackTES).begin();
	Rec::TrackParticleContainer::const_iterator trackItrE = (*trackTES).end();
	
	int Ntracks = 0, NMatch = 0;
	for (; trackItr != trackItrE; ++trackItr) {
	  Ntracks++;
	  if(Ntracks < max_track_num) {
	    const Rec::TrackParticle *trackParticle = (*trackItr);
	    if (trackInJet == trackParticle) {
	      m_h_cone4_trackidx[Ncone4-1] [imatch] = Ntracks-1;
	      //cout << "CONE4(Z) - > " << trackInJet->eta() << " " << trackInJet->phi() << " vs " << m_h_track_eta[m_h_cone4_trackidx[Ncone4-1] [imatch]] << " " 
	      //   << m_h_track_phi[m_h_cone4_trackidx[Ncone4-1] [imatch]] << endl;
	      imatch++; NMatch++;
	    }
	  }
	}
	  
	if (NMatch > 1) mLog << MSG::WARNING << "Track used in cone4 has more than one match in track container!" << endreq;
	else if (NMatch == 0) mLog << MSG::WARNING << "Track used in cone4 has no match in track container!" << endreq;
	
      }	

      // Truth
      std::string label("N/A");  
      const Analysis::TruthInfo* mcinfo = (*cone4Itr)->tagInfo<Analysis::TruthInfo>("TruthInfo");
      if (mcinfo) label = mcinfo->jetTruthLabel();

      int flav(0);      
      if(label == "B") flav = 5;    // b-jet
      if(label == "C") flav = 4;    // c-jet
      if(label == "T") flav = 15;   // tau

      m_h_cone4_truth[Ncone4-1] = flav;

      // SV1
      const Analysis::SVInfoBase* infoSV1 = (*cone4Itr)->tagInfo<Analysis::SVInfoBase>("SV1");
      if (infoSV1) { // if these aren't true what will be stored ?  Might cause a problem.
	m_h_cone4_sv1_pu[Ncone4-1] = infoSV1->tagLikelihood()[0];
	m_h_cone4_sv1_pb[Ncone4-1] = infoSV1->tagLikelihood()[1];
      } else {
	m_h_cone4_sv1_pu[Ncone4-1] = -999.9;
	m_h_cone4_sv1_pb[Ncone4-1] = -999.9;
      }

      // SV2
      const Analysis::SVInfoBase* infoSV2 = (*cone4Itr)->tagInfo<Analysis::SVInfoBase>("SV2");
      if (infoSV2) {
	m_h_cone4_sv2_pu[Ncone4-1] = infoSV2->tagLikelihood()[0];
	m_h_cone4_sv2_pb[Ncone4-1] = infoSV2->tagLikelihood()[1];
      } else {
	m_h_cone4_sv2_pu[Ncone4-1] = -999.9;
	m_h_cone4_sv2_pb[Ncone4-1] = -999.9;
      }

      // SVPlus
      const Analysis::SVInfoPlus* infoSVPlus = (*cone4Itr)->tagInfo<Analysis::SVInfoPlus>("SVInfoPlus");
      if (infoSVPlus) {
	m_h_cone4_svplus_n2t [Ncone4-1]     = infoSVPlus->getN2T();							       
	m_h_cone4_svplus_eFrac[Ncone4-1]    = infoSVPlus->getEnergyFraction();						       
	m_h_cone4_svplus_mass[Ncone4-1]     = infoSVPlus->getMass();							       
	m_h_cone4_svplus_normDist[Ncone4-1] = infoSVPlus->getNormDist();						       
	m_h_cone4_svplus_NGTracks[Ncone4-1] = infoSVPlus->getNGTrackInSvx();			       
	m_h_cone4_svplus_chi2[Ncone4-1]     = infoSVPlus->getRecSvx().chi2();
	m_h_cone4_svplus_ndf[Ncone4-1]      = infoSVPlus->getRecSvx().ndf();
	m_h_cone4_svplus_x[Ncone4-1]        = infoSVPlus->getRecSvx().position().x();                                             
	m_h_cone4_svplus_y [Ncone4-1]       = infoSVPlus->getRecSvx().position().y();                                             
	m_h_cone4_svplus_z [Ncone4-1]       = infoSVPlus->getRecSvx().position().z();                                             	
	
	int ntrk = infoSVPlus->numTrackInfo();  
	if (ntrk > max_jettrack_num) ntrk = max_jettrack_num;
	m_h_cone4_svplus_ntracks[Ncone4-1]  = ntrk;
	
	int imatch = 0;
	for(int i = 0; i < ntrk; i++) {
	  Analysis::SVTrackInfo trackInfo = infoSVPlus->getTrackInfo(i);
	  
	  // Store indicies of tracks used
	  const Rec::TrackParticle *trackInJet = trackInfo.track();
	  const Rec::TrackParticleContainer* trackTES;
	  status = m_storeGate->retrieve(trackTES, "TrackParticleCandidate");
	  if (status.isFailure()  ||  !trackTES) {
	    mLog << MSG::WARNING << "No AOD TrackParticles container found in TDS" << endreq;
	    return StatusCode::SUCCESS;
	  }
	  mLog << MSG::DEBUG << "TrackParticles successfully retrieved" << endreq;
	  
	  Rec::TrackParticleContainer::const_iterator trackItr  = (*trackTES).begin();
	  Rec::TrackParticleContainer::const_iterator trackItrE = (*trackTES).end();
	  
	  int Ntracks = 0, NMatch = 0;
	  for (; trackItr != trackItrE; ++trackItr) {
	    Ntracks++;
	    if(Ntracks < max_track_num) {
	      const Rec::TrackParticle *trackParticle = (*trackItr);
	      if (trackInJet == trackParticle) {
		m_h_cone4_svplus_trackidx[Ncone4-1] [imatch] = Ntracks-1;
		//cout << "CONE4(Z) - > " << trackInJet->eta() << " " << trackInJet->phi() << " vs " << m_h_track_eta[m_h_cone4_svplus_trackidx[Ncone4-1] [imatch]] << " " 
		//   << m_h_track_phi[m_h_cone4_svplus_trackidx[Ncone4-1] [imatch]] << endl;
		imatch++; NMatch++;
	      }
	    }
	  }
	  
	  if (NMatch > 1) mLog << MSG::WARNING << "Track used in cone4 SV tagger has more than one match in track container!" << endreq;
	  else if (NMatch == 0) mLog << MSG::WARNING << "Track used in cone4 SV tagger has no match in track container!" << endreq;
	}


      } else {
	m_h_cone4_svplus_n2t [Ncone4-1]     = -999;
	m_h_cone4_svplus_eFrac[Ncone4-1]    = -999.9;
	m_h_cone4_svplus_mass[Ncone4-1]     = -999.9;
	m_h_cone4_svplus_normDist[Ncone4-1] = -999.9;
	m_h_cone4_svplus_NGTracks[Ncone4-1] = -999;
	m_h_cone4_svplus_chi2[Ncone4-1]     = -999.9;
	m_h_cone4_svplus_ndf[Ncone4-1]      = -999;
	m_h_cone4_svplus_x[Ncone4-1]        = -999.9;
	m_h_cone4_svplus_y [Ncone4-1]       = -999.9;
	m_h_cone4_svplus_z [Ncone4-1]       = -999.9;
      }

      // SVBU
      const Analysis::SecVtxInfo* infoSVBU = (*cone4Itr)->tagInfo<Analysis::SecVtxInfo>("SecVtxTagBU");
      if (infoSVBU) {
	m_h_cone4_svbu_pu[Ncone4-1]        = infoSVBU->tagLikelihood()[0];
	m_h_cone4_svbu_pb[Ncone4-1]        = infoSVBU->tagLikelihood()[1];
	m_h_cone4_svbu_eFrac[Ncone4-1]     = infoSVBU->energyFraction();							     
	m_h_cone4_svbu_mass[Ncone4-1]      = infoSVBU->mass();								     
	m_h_cone4_svbu_normDist[Ncone4-1]  = infoSVBU->distance();								     
	m_h_cone4_svbu_mult[Ncone4-1]      = infoSVBU->mult();						     
	m_h_cone4_svbu_chi2[Ncone4-1]      = infoSVBU->secVertexPos().chi2();
	m_h_cone4_svbu_ndf[Ncone4-1]       = infoSVBU->secVertexPos().ndf();
	m_h_cone4_svbu_x[Ncone4-1]         = infoSVBU->secVertexPos().position().x();                                                
	m_h_cone4_svbu_y[Ncone4-1]         = infoSVBU->secVertexPos().position().y();                                                
	m_h_cone4_svbu_z[Ncone4-1]         = infoSVBU->secVertexPos().position().z();                                                
      } else {
	m_h_cone4_svbu_pu[Ncone4-1]        =  -999.9;
	m_h_cone4_svbu_pb[Ncone4-1]        =  -999.9;
	m_h_cone4_svbu_eFrac[Ncone4-1]     =  -999.9;
	m_h_cone4_svbu_mass[Ncone4-1]      =  -999.9;
	m_h_cone4_svbu_normDist[Ncone4-1]  =  -999.9;
	m_h_cone4_svbu_mult[Ncone4-1]      =  -999;
	m_h_cone4_svbu_chi2[Ncone4-1]      =  -999.9;
	m_h_cone4_svbu_ndf[Ncone4-1]       =  -999;
	m_h_cone4_svbu_x[Ncone4-1]         =  -999.9;
	m_h_cone4_svbu_y[Ncone4-1]         =  -999.9;
	m_h_cone4_svbu_z[Ncone4-1]         =  -999.9;
      }

      // IP2D
      const Analysis::IPInfoBase* info2D = (*cone4Itr)->tagInfo<Analysis::IPInfoBase>("IP2D");
      if (info2D) {
      	m_h_cone4_ip2d_pu[Ncone4-1]        = info2D->tagLikelihood()[0];
	m_h_cone4_ip2d_pb[Ncone4-1]        = info2D->tagLikelihood()[1];
	m_h_cone4_ip2d_numTracks[Ncone4-1] = info2D->nbTracks();
      } else {
	m_h_cone4_ip2d_pu[Ncone4-1]        = -999.9;
	m_h_cone4_ip2d_pb[Ncone4-1]        = -999.9;
	m_h_cone4_ip2d_numTracks[Ncone4-1] = -999;
      }

      // IP3D
      const Analysis::IPInfoBase* info3D = (*cone4Itr)->tagInfo<Analysis::IPInfoBase>("IP3D");
      if (info3D) {
      	m_h_cone4_ip3d_pu[Ncone4-1]        = info3D->tagLikelihood()[0];
	m_h_cone4_ip3d_pb[Ncone4-1]        = info3D->tagLikelihood()[1];
	m_h_cone4_ip3d_numTracks[Ncone4-1] = info3D->nbTracks();
      } else {
	m_h_cone4_ip3d_pu[Ncone4-1]        = -999.9;
	m_h_cone4_ip3d_pb[Ncone4-1]        = -999.9;
	m_h_cone4_ip3d_numTracks[Ncone4-1] = -999;
      }

      // IPPlus
      
      const Analysis::IPInfoPlus* infoIPPlus = (*cone4Itr)->tagInfo<Analysis::IPInfoPlus>("IPInfoPlus");
      if (infoIPPlus) {
	int ntrk = infoIPPlus->numTrackInfo();  // Is this the same as nbtrack? If not might need to store
	if (ntrk > max_jettrack_num) ntrk = max_jettrack_num;

	int imatch = 0;
	for(int i = 0; i < ntrk; i++) {
	  Analysis::IPTrackInfo trackInfo = infoIPPlus->getTrackInfo(i);

	  Analysis::TrackGrade grade = trackInfo.trackGrade(); // Good, Shared, ...
	  std::string gradeStr = static_cast<std::string>(grade);
	  int gradeInt = -1;
	  if (gradeStr == "Good")   gradeInt = 0; 
	  else if (gradeStr == "Shared") gradeInt = 1; 

	  m_h_cone4_ipplus_d0[Ncone4-1] [i]          = trackInfo.d0Value();		
	  m_h_cone4_ipplus_d0Sig[Ncone4-1] [i]       = trackInfo.d0Significance();
	  m_h_cone4_ipplus_z0[Ncone4-1] [i]          = trackInfo.z0Value();	
	  m_h_cone4_ipplus_z0Sig[Ncone4-1] [i]       = trackInfo.z0Significance();
	  m_h_cone4_ipplus_trackW2D[Ncone4-1] [i]    = trackInfo.trackWeight2D(); 
	  m_h_cone4_ipplus_trackW3D[Ncone4-1] [i]    = trackInfo.trackWeight3D(); 
	  m_h_cone4_ipplus_isFromV0[Ncone4-1] [i]    = trackInfo.isFromV0();	
	  m_h_cone4_ipplus_trackJP[Ncone4-1] [i]     = trackInfo.trackProbJP();   
	  m_h_cone4_ipplus_trackGrade[Ncone4-1] [i]  = gradeInt;


	  // Store indicies of tracks used
	  const Rec::TrackParticle *trackInJet = trackInfo.track();
	  const Rec::TrackParticleContainer* trackTES;
	  status = m_storeGate->retrieve(trackTES, "TrackParticleCandidate");
	  if (status.isFailure()  ||  !trackTES) {
	    mLog << MSG::WARNING << "No AOD TrackParticles container found in TDS" << endreq;
	    return StatusCode::SUCCESS;
	  }
	  mLog << MSG::DEBUG << "TrackParticles successfully retrieved" << endreq;
	  
	  Rec::TrackParticleContainer::const_iterator trackItr  = (*trackTES).begin();
	  Rec::TrackParticleContainer::const_iterator trackItrE = (*trackTES).end();
	  
	  int Ntracks = 0, NMatch = 0;
	  for (; trackItr != trackItrE; ++trackItr) {
	    Ntracks++;
	    if(Ntracks < max_track_num) {
	      const Rec::TrackParticle *trackParticle = (*trackItr);
	      if (trackInJet == trackParticle) {
		m_h_cone4_ipplus_trackidx[Ncone4-1] [imatch] = Ntracks-1;
		//cout << "CONE4(Z) - > " << trackInJet->eta() << " " << trackInJet->phi() << " vs " << m_h_track_eta[m_h_cone4_ipplus_trackidx[Ncone4-1] [imatch]] << " " 
		//     << m_h_track_phi[m_h_cone4_ipplus_trackidx[Ncone4-1] [imatch]] << endl;
		imatch++; NMatch++;
	      }
	    }
	  }
	  
	  if (NMatch > 1) mLog << MSG::WARNING << "Track used in cone4 IP tagger has more than one match in track container!" << endreq;
	  else if (NMatch == 0) mLog << MSG::WARNING << "Track used in cone4 IP tagger has no match in track container!" << endreq;
	}
      }

      // LT1D
      const Analysis::LifetimeInfo* infoLT1D = (*cone4Itr)->tagInfo<Analysis::LifetimeInfo>("LifetimeTag1D");
      if (infoLT1D) {
	int ntrk =  infoLT1D->significance().size();
	m_h_cone4_lt1d_numTracks[Ncone4-1] = ntrk;

	if (ntrk > max_jettrack_num) ntrk = max_jettrack_num;

	for(int i = 0; i < ntrk; i++) {
	  m_h_cone4_lt1d_z0[Ncone4-1] [i]    = infoLT1D->signedIP()[i];
	  m_h_cone4_lt1d_z0Sig[Ncone4-1] [i] = infoLT1D->significance()[i];
	}
      } else {
	m_h_cone4_lt1d_numTracks[Ncone4-1] = -999;
      }

      // LT2D
      const Analysis::LifetimeInfo* infoLT2D = (*cone4Itr)->tagInfo<Analysis::LifetimeInfo>("LifetimeTag2D");
      if (infoLT2D) {
	int ntrk =  infoLT2D->significance().size();
	m_h_cone4_lt2d_numTracks[Ncone4-1] = ntrk;

	if (ntrk > max_jettrack_num) ntrk = max_jettrack_num;

	for(int i = 0; i < ntrk; i++) {
	  m_h_cone4_lt2d_d0[Ncone4-1] [i]    = infoLT2D->signedIP()[i];
	  m_h_cone4_lt2d_d0Sig[Ncone4-1] [i] = infoLT2D->significance()[i];
	}
      } else {
	m_h_cone4_lt2d_numTracks[Ncone4-1] = -999;
      }

      // Soft muon
      const Analysis::SoftMuonInfo* infoMu = (*cone4Itr)->tagInfo<Analysis::SoftMuonInfo>("SoftMuonTag");
      if (infoMu) {
	int ntrk = infoMu->numTrackInfo();
	m_h_cone4_softmu_numTracks[Ncone4-1] = ntrk;
	
	if (ntrk > max_jettrack_num) ntrk = max_jettrack_num;
	for(int i = 0; i < ntrk; i++) {
	  Analysis::SMTrackInfo trackInfo = infoMu->getTrackInfo(i);
	  m_h_cone4_softmu_d0[Ncone4-1] [i] = trackInfo.d0Value();
	  m_h_cone4_softmu_ptrel[Ncone4-1] [i] = trackInfo.pTrel();
	  // what do about muon ptr?
	}
      } else {
	m_h_cone4_softmu_numTracks[Ncone4-1] = -999;
      }

      // Soft elec
      const Analysis::SoftElectronInfo* infoElec = (*cone4Itr)->tagInfo<Analysis::SoftElectronInfo>("SoftElectronTag");
      if (infoElec) {
	int ntrk = infoElec->numTrackInfo();
	m_h_cone4_softelec_numTracks[Ncone4-1] = ntrk;
	
	if (ntrk > max_jettrack_num) ntrk = max_jettrack_num;
	for(int i = 0; i < ntrk; i++) {
	  Analysis::SETrackInfo trackInfo = infoElec->getTrackInfo(i);
	  m_h_cone4_softelec_d0[Ncone4-1] [i] = trackInfo.d0Value();
	  m_h_cone4_softelec_ptrel[Ncone4-1] [i] = trackInfo.pTrel();
	  // what do about electron ptr?
	}
      } else {
	m_h_cone4_softelec_numTracks[Ncone4-1] = -999;
      }

      // Jet prob info is now written to IPInfoPlus

    }
  }
  m_h_cone4_num = Ncone4 < max_jet_num ? Ncone4 : max_jet_num;
  //m_h_cone4_num = max_jet_num;
  //if(Ncone4 < max_jet_num) m_h_cone4_num = Ncone4;
  mLog << MSG::INFO << "Cone4 Jets added to Ntuple. " << endreq;

  // Kt jets
  const ParticleJetContainer* ktTES;
  if (m_release == 4) {
    status = m_storeGate->retrieve( ktTES, "Kt4TowerParticleJets");
    if (status.isFailure()  ||  !ktTES) {
      mLog << MSG::WARNING << "No AOD Kt4jet container found in TDS" << endreq;
      return StatusCode::SUCCESS;
    }
  } else if (m_release == 31) {
    status = m_storeGate->retrieve( ktTES, "KtTowerParticleJets");
    if (status.isFailure()  ||  !ktTES) {
      mLog << MSG::WARNING << "No AOD Kt4jet container found in TDS" << endreq;
      return StatusCode::SUCCESS;
    }
  }

  mLog << MSG::DEBUG << " Kt4jets successfully retrieved" << endreq;

  ParticleJetContainer::const_iterator kt4jetItr  = (*ktTES).begin();
  ParticleJetContainer::const_iterator kt4jetItrE = (*ktTES).end();

  int Nkt4jet = 0;
  for (; kt4jetItr != kt4jetItrE; ++kt4jetItr) {
    Nkt4jet++ ;
    if(Nkt4jet < max_jet_num) {
      //m_h_kt4jet_px[Nkt4jet-1] = (*kt4jetItr)->px(); 
      //m_h_kt4jet_py[Nkt4jet-1] = (*kt4jetItr)->py(); 
      //m_h_kt4jet_pz[Nkt4jet-1] = (*kt4jetItr)->pz(); 
      m_h_kt4jet_pt[Nkt4jet-1] = (*kt4jetItr)->pt(); 
      m_h_kt4jet_eta[Nkt4jet-1]= (*kt4jetItr)->eta();
      m_h_kt4jet_phi[Nkt4jet-1]= (*kt4jetItr)->phi();

      // Btag
      // Weights
      m_h_kt4jet_lhsig[Nkt4jet-1]      = (*kt4jetItr)->lhSig();
      m_h_kt4jet_w[Nkt4jet-1]         = (*kt4jetItr)->weight();                       
      m_h_kt4jet_wIP2D[Nkt4jet-1]     = (*kt4jetItr)->weightForTag("IP2D");	    
      m_h_kt4jet_wIP3D[Nkt4jet-1]     = (*kt4jetItr)->weightForTag("IP3D");	    
      m_h_kt4jet_wSV1[Nkt4jet-1]      = (*kt4jetItr)->weightForTag("SV1");	    
      m_h_kt4jet_wSV2[Nkt4jet-1]      = (*kt4jetItr)->weightForTag("SV2");	    
      m_h_kt4jet_wLT1D[Nkt4jet-1]     = (*kt4jetItr)->weightForTag("LifeTimeTag1D");  
      m_h_kt4jet_wLT2D[Nkt4jet-1]     = (*kt4jetItr)->weightForTag("LifeTimeTag2D");  
      //m_h_kt4jet_wLT3D[Nkt4jet-1]     = (*kt4jetItr)->weightForTag("LifeTimeTag3D");  
      m_h_kt4jet_SVBU[Nkt4jet-1]      = (*kt4jetItr)->weightForTag("SecVtxTagBU");    
      //m_h_kt4jet_SVTD[Nkt4jet-1]      = (*kt4jetItr)->weightForTag("SecVtxTagTD");    
      m_h_kt4jet_wLHSig[Nkt4jet-1]    = (*kt4jetItr)->weightForTag("lhSig");          
      m_h_kt4jet_wSoftMu[Nkt4jet-1]   = (*kt4jetItr)->weightForTag("SoftMuonTag");          
      m_h_kt4jet_wSoftElec[Nkt4jet-1] = (*kt4jetItr)->weightForTag("SoftMuonTag");          
      m_h_kt4jet_wJP[Nkt4jet-1]       = (*kt4jetItr)->weightForTag("JetProbTag");          

      // Store indicies of tracks used
      std::vector<const Rec::TrackParticle*>* trackVector = (*kt4jetItr)->constituent<Analysis::TrackConstituents>("Tracks")->tracks();
      int nTracks = trackVector->size();	
      m_h_kt4jet_ntracks[Nkt4jet -1] = nTracks;
      
      int imatch = 0;
      for (int i = 0; i < nTracks; i++) {	  
	const Rec::TrackParticle *trackInJet = trackVector->at(i);
	const Rec::TrackParticleContainer* trackTES;
	status = m_storeGate->retrieve(trackTES, "TrackParticleCandidate");
	if (status.isFailure()  ||  !trackTES) {
	  mLog << MSG::WARNING << "No AOD TrackParticles container found in TDS" << endreq;
	  return StatusCode::SUCCESS;
	}
	mLog << MSG::DEBUG << "TrackParticles successfully retrieved" << endreq;
	
	Rec::TrackParticleContainer::const_iterator trackItr  = (*trackTES).begin();
	Rec::TrackParticleContainer::const_iterator trackItrE = (*trackTES).end();
	
	int Ntracks = 0, NMatch = 0;
	for (; trackItr != trackItrE; ++trackItr) {
	  Ntracks++;
	  if(Ntracks < max_track_num) {
	    const Rec::TrackParticle *trackParticle = (*trackItr);
	    if (trackInJet == trackParticle) {
	      m_h_kt4jet_trackidx[Nkt4jet-1] [imatch] = Ntracks-1;
	      //cout << "KT4JET(Z) - > " << trackInJet->eta() << " " << trackInJet->phi() << " vs " << m_h_track_eta[m_h_kt4jet_trackidx[Nkt4jet-1] [imatch]] << " " 
	      //   << m_h_track_phi[m_h_kt4jet_trackidx[Nkt4jet-1] [imatch]] << endl;
	      imatch++; NMatch++;
	    }
	  }
	}
	  
	if (NMatch > 1) mLog << MSG::WARNING << "Track used in kt4jet has more than one match in track container!" << endreq;
	else if (NMatch == 0) mLog << MSG::WARNING << "Track used in kt4jet has no match in track container!" << endreq;
	
      }	

      // Truth
      std::string label("N/A");  
      const Analysis::TruthInfo* mcinfo = (*kt4jetItr)->tagInfo<Analysis::TruthInfo>("TruthInfo");
      if (mcinfo) label = mcinfo->jetTruthLabel();

      int flav(0);      
      if(label == "B") flav = 5;    // b-jet
      if(label == "C") flav = 4;    // c-jet
      if(label == "T") flav = 15;   // tau

      m_h_kt4jet_truth[Nkt4jet-1] = flav;

      // SV1
      const Analysis::SVInfoBase* infoSV1 = (*kt4jetItr)->tagInfo<Analysis::SVInfoBase>("SV1");
      if (infoSV1) { // if these aren't true what will be stored ?  Might cause a problem.
	m_h_kt4jet_sv1_pu[Nkt4jet-1] = infoSV1->tagLikelihood()[0];
	m_h_kt4jet_sv1_pb[Nkt4jet-1] = infoSV1->tagLikelihood()[1];
      } else {
	m_h_kt4jet_sv1_pu[Nkt4jet-1] = -999.9;
	m_h_kt4jet_sv1_pb[Nkt4jet-1] = -999.9;
      }

      // SV2
      const Analysis::SVInfoBase* infoSV2 = (*kt4jetItr)->tagInfo<Analysis::SVInfoBase>("SV2");
      if (infoSV2) {
	m_h_kt4jet_sv2_pu[Nkt4jet-1] = infoSV2->tagLikelihood()[0];
	m_h_kt4jet_sv2_pb[Nkt4jet-1] = infoSV2->tagLikelihood()[1];
      } else {
	m_h_kt4jet_sv2_pu[Nkt4jet-1] = -999.9;
	m_h_kt4jet_sv2_pb[Nkt4jet-1] = -999.9;
      }

      // SVPlus
      const Analysis::SVInfoPlus* infoSVPlus = (*kt4jetItr)->tagInfo<Analysis::SVInfoPlus>("SVInfoPlus");
      if (infoSVPlus) {
	m_h_kt4jet_svplus_n2t [Nkt4jet-1]     = infoSVPlus->getN2T();							       
	m_h_kt4jet_svplus_eFrac[Nkt4jet-1]    = infoSVPlus->getEnergyFraction();						       
	m_h_kt4jet_svplus_mass[Nkt4jet-1]     = infoSVPlus->getMass();							       
	m_h_kt4jet_svplus_normDist[Nkt4jet-1] = infoSVPlus->getNormDist();						       
	m_h_kt4jet_svplus_NGTracks[Nkt4jet-1] = infoSVPlus->getNGTrackInSvx();			       
	m_h_kt4jet_svplus_chi2[Nkt4jet-1]     = infoSVPlus->getRecSvx().chi2();
	m_h_kt4jet_svplus_ndf[Nkt4jet-1]      = infoSVPlus->getRecSvx().ndf();
	m_h_kt4jet_svplus_x[Nkt4jet-1]        = infoSVPlus->getRecSvx().position().x();                                             
	m_h_kt4jet_svplus_y [Nkt4jet-1]       = infoSVPlus->getRecSvx().position().y();                                             
	m_h_kt4jet_svplus_z [Nkt4jet-1]       = infoSVPlus->getRecSvx().position().z();                                             	

	int ntrk = infoSVPlus->numTrackInfo();  
	if (ntrk > max_jettrack_num) ntrk = max_jettrack_num;
	m_h_kt4jet_svplus_ntracks[Nkt4jet-1]  = ntrk;
	
	int imatch = 0;
	for(int i = 0; i < ntrk; i++) {
	  Analysis::SVTrackInfo trackInfo = infoSVPlus->getTrackInfo(i);
	  
	  // Store indicies of tracks used
	  const Rec::TrackParticle *trackInJet = trackInfo.track();
	  const Rec::TrackParticleContainer* trackTES;
	  status = m_storeGate->retrieve(trackTES, "TrackParticleCandidate");
	  if (status.isFailure()  ||  !trackTES) {
	    mLog << MSG::WARNING << "No AOD TrackParticles container found in TDS" << endreq;
	    return StatusCode::SUCCESS;
	  }
	  mLog << MSG::DEBUG << "TrackParticles successfully retrieved" << endreq;
	  
	  Rec::TrackParticleContainer::const_iterator trackItr  = (*trackTES).begin();
	  Rec::TrackParticleContainer::const_iterator trackItrE = (*trackTES).end();
	  
	  int Ntracks = 0, NMatch = 0;
	  for (; trackItr != trackItrE; ++trackItr) {
	    Ntracks++;
	    if(Ntracks < max_track_num) {
	      const Rec::TrackParticle *trackParticle = (*trackItr);
	      if (trackInJet == trackParticle) {
		m_h_kt4jet_svplus_trackidx[Nkt4jet-1] [imatch] = Ntracks-1;
		//cout << "KT4(Z) - > " << trackInJet->eta() << " " << trackInJet->phi() << " vs " << m_h_track_eta[m_h_kt4jet_svplus_trackidx[Nkt4jet-1] [imatch]] << " " 
		//   << m_h_track_phi[m_h_kt4jet_svplus_trackidx[Nkt4jet-1] [imatch]] << endl;
		imatch++; NMatch++;
	      }
	    }
	  }
	  
	  if (NMatch > 1) mLog << MSG::WARNING << "Track used in kt4jet SV tagger has more than one match in track container!" << endreq;
	  else if (NMatch == 0) mLog << MSG::WARNING << "Track used in kt4jet SV tagger has no match in track container!" << endreq;
	}

      } else {
	m_h_kt4jet_svplus_n2t [Nkt4jet-1]     = -999;
	m_h_kt4jet_svplus_eFrac[Nkt4jet-1]    = -999.9;
	m_h_kt4jet_svplus_mass[Nkt4jet-1]     = -999.9;
	m_h_kt4jet_svplus_normDist[Nkt4jet-1] = -999.9;
	m_h_kt4jet_svplus_NGTracks[Nkt4jet-1] = -999;
	m_h_kt4jet_svplus_chi2[Nkt4jet-1]     = -999.9;
	m_h_kt4jet_svplus_ndf[Nkt4jet-1]      = -999;
	m_h_kt4jet_svplus_x[Nkt4jet-1]        = -999.9;
	m_h_kt4jet_svplus_y [Nkt4jet-1]       = -999.9;
	m_h_kt4jet_svplus_z [Nkt4jet-1]       = -999.9;
      }

      // SVBU
      const Analysis::SecVtxInfo* infoSVBU = (*kt4jetItr)->tagInfo<Analysis::SecVtxInfo>("SecVtxTagBU");
      if (infoSVBU) {
	m_h_kt4jet_svbu_pu[Nkt4jet-1]        = infoSVBU->tagLikelihood()[0];
	m_h_kt4jet_svbu_pb[Nkt4jet-1]        = infoSVBU->tagLikelihood()[1];
	m_h_kt4jet_svbu_eFrac[Nkt4jet-1]     = infoSVBU->energyFraction();							     
	m_h_kt4jet_svbu_mass[Nkt4jet-1]      = infoSVBU->mass();								     
	m_h_kt4jet_svbu_normDist[Nkt4jet-1]  = infoSVBU->distance();								     
	m_h_kt4jet_svbu_mult[Nkt4jet-1]      = infoSVBU->mult();						     
	m_h_kt4jet_svbu_chi2[Nkt4jet-1]      = infoSVBU->secVertexPos().chi2();
	m_h_kt4jet_svbu_ndf[Nkt4jet-1]       = infoSVBU->secVertexPos().ndf();
	m_h_kt4jet_svbu_x[Nkt4jet-1]         = infoSVBU->secVertexPos().position().x();                                                
	m_h_kt4jet_svbu_y[Nkt4jet-1]         = infoSVBU->secVertexPos().position().y();                                                
	m_h_kt4jet_svbu_z[Nkt4jet-1]         = infoSVBU->secVertexPos().position().z();                                                
      } else {
	m_h_kt4jet_svbu_pu[Nkt4jet-1]        =  -999.9;
	m_h_kt4jet_svbu_pb[Nkt4jet-1]        =  -999.9;
	m_h_kt4jet_svbu_eFrac[Nkt4jet-1]     =  -999.9;
	m_h_kt4jet_svbu_mass[Nkt4jet-1]      =  -999.9;
	m_h_kt4jet_svbu_normDist[Nkt4jet-1]  =  -999.9;
	m_h_kt4jet_svbu_mult[Nkt4jet-1]      =  -999;
	m_h_kt4jet_svbu_chi2[Nkt4jet-1]      =  -999.9;
	m_h_kt4jet_svbu_ndf[Nkt4jet-1]       =  -999;
	m_h_kt4jet_svbu_x[Nkt4jet-1]         =  -999.9;
	m_h_kt4jet_svbu_y[Nkt4jet-1]         =  -999.9;
	m_h_kt4jet_svbu_z[Nkt4jet-1]         =  -999.9;
      }

      // IP2D
      const Analysis::IPInfoBase* info2D = (*kt4jetItr)->tagInfo<Analysis::IPInfoBase>("IP2D");
      if (info2D) {
      	m_h_kt4jet_ip2d_pu[Nkt4jet-1]        = info2D->tagLikelihood()[0];
	m_h_kt4jet_ip2d_pb[Nkt4jet-1]        = info2D->tagLikelihood()[1];
	m_h_kt4jet_ip2d_numTracks[Nkt4jet-1] = info2D->nbTracks();
      } else {
	m_h_kt4jet_ip2d_pu[Nkt4jet-1]        = -999.9;
	m_h_kt4jet_ip2d_pb[Nkt4jet-1]        = -999.9;
	m_h_kt4jet_ip2d_numTracks[Nkt4jet-1] = -999;
      }

      // IP3D
      const Analysis::IPInfoBase* info3D = (*kt4jetItr)->tagInfo<Analysis::IPInfoBase>("IP3D");
      if (info3D) {
      	m_h_kt4jet_ip3d_pu[Nkt4jet-1]        = info3D->tagLikelihood()[0];
	m_h_kt4jet_ip3d_pb[Nkt4jet-1]        = info3D->tagLikelihood()[1];
	m_h_kt4jet_ip3d_numTracks[Nkt4jet-1] = info3D->nbTracks();
      } else {
	m_h_kt4jet_ip3d_pu[Nkt4jet-1]        = -999.9;
	m_h_kt4jet_ip3d_pb[Nkt4jet-1]        = -999.9;
	m_h_kt4jet_ip3d_numTracks[Nkt4jet-1] = -999;
      }

      // IPPlus
      const Analysis::IPInfoPlus* infoIPPlus = (*kt4jetItr)->tagInfo<Analysis::IPInfoPlus>("IPInfoPlus");
      if (infoIPPlus) {
	int ntrk = infoIPPlus->numTrackInfo();  // Is this the same as nbtrack? If not might need to store
	if (ntrk > max_jettrack_num) ntrk = max_jettrack_num;

	int imatch = 0;
	for(int i = 0; i < ntrk; i++) {
	  Analysis::IPTrackInfo trackInfo = infoIPPlus->getTrackInfo(i);

	  Analysis::TrackGrade grade = trackInfo.trackGrade(); // Good, Shared, ...
	  std::string gradeStr = static_cast<std::string>(grade);
	  int gradeInt = -1;
	  if (gradeStr == "Good")   gradeInt = 0; 
	  else if (gradeStr == "Shared") gradeInt = 1; 

	  m_h_kt4jet_ipplus_d0[Nkt4jet-1] [i]          = trackInfo.d0Value();		
	  m_h_kt4jet_ipplus_d0Sig[Nkt4jet-1] [i]       = trackInfo.d0Significance();
	  m_h_kt4jet_ipplus_z0[Nkt4jet-1] [i]          = trackInfo.z0Value();	
	  m_h_kt4jet_ipplus_z0Sig[Nkt4jet-1] [i]       = trackInfo.z0Significance();
	  m_h_kt4jet_ipplus_trackW2D[Nkt4jet-1] [i]    = trackInfo.trackWeight2D(); 
	  m_h_kt4jet_ipplus_trackW3D[Nkt4jet-1] [i]    = trackInfo.trackWeight3D(); 
	  m_h_kt4jet_ipplus_isFromV0[Nkt4jet-1] [i]    = trackInfo.isFromV0();	
	  m_h_kt4jet_ipplus_trackJP[Nkt4jet-1] [i]     = trackInfo.trackProbJP();   
	  m_h_kt4jet_ipplus_trackGrade[Nkt4jet-1] [i]  = gradeInt;

	  // Store indicies of tracks used
	  const Rec::TrackParticle *trackInJet = trackInfo.track();
	  const Rec::TrackParticleContainer* trackTES;
	  status = m_storeGate->retrieve(trackTES, "TrackParticleCandidate");
	  if (status.isFailure()  ||  !trackTES) {
	    mLog << MSG::WARNING << "No AOD TrackParticles container found in TDS" << endreq;
	    return StatusCode::SUCCESS;
	  }
	  mLog << MSG::DEBUG << "TrackParticles successfully retrieved" << endreq;
	  
	  Rec::TrackParticleContainer::const_iterator trackItr  = (*trackTES).begin();
	  Rec::TrackParticleContainer::const_iterator trackItrE = (*trackTES).end();
	  
	  int Ntracks = 0, NMatch = 0;
	  for (; trackItr != trackItrE; ++trackItr) {
	    Ntracks++;
	    if(Ntracks < max_track_num) {
	      const Rec::TrackParticle *trackParticle = (*trackItr);
	      if (trackInJet == trackParticle) {
		m_h_kt4jet_ipplus_trackidx[Nkt4jet-1] [imatch] = Ntracks-1;
		//cout << "KT4(Z) - > " << trackInJet->eta() << " " << trackInJet->phi() << " vs " << m_h_track_eta[m_h_kt4jet_ipplus_trackidx[Nkt4jet-1] [imatch]] << " " 
		//     << m_h_track_phi[m_h_kt4jet_ipplus_trackidx[Nkt4jet-1] [imatch]] << endl;
		imatch++; NMatch++;
	      }
	    }
	  }
	  
	  if (NMatch > 1) mLog << MSG::WARNING << "Track used in kt4jet IP tagger has more than one match in track container!" << endreq;
	  else if (NMatch == 0) mLog << MSG::WARNING << "Track used in kt4jet IP tagger has no match in track container!" << endreq;

	}
      }

      // LT1D
      const Analysis::LifetimeInfo* infoLT1D = (*kt4jetItr)->tagInfo<Analysis::LifetimeInfo>("LifetimeTag1D");
      if (infoLT1D) {
	int ntrk =  infoLT1D->significance().size();
	m_h_kt4jet_lt1d_numTracks[Nkt4jet-1] = ntrk;

	if (ntrk > max_jettrack_num) ntrk = max_jettrack_num;

	for(int i = 0; i < ntrk; i++) {
	  m_h_kt4jet_lt1d_z0[Nkt4jet-1] [i]    = infoLT1D->signedIP()[i];
	  m_h_kt4jet_lt1d_z0Sig[Nkt4jet-1] [i] = infoLT1D->significance()[i];
	}
      } else {
	m_h_kt4jet_lt1d_numTracks[Nkt4jet-1] = -999;
      }

      // LT2D
      const Analysis::LifetimeInfo* infoLT2D = (*kt4jetItr)->tagInfo<Analysis::LifetimeInfo>("LifetimeTag2D");
      if (infoLT2D) {
	int ntrk =  infoLT2D->significance().size();
	m_h_kt4jet_lt2d_numTracks[Nkt4jet-1] = ntrk;

	if (ntrk > max_jettrack_num) ntrk = max_jettrack_num;

	for(int i = 0; i < ntrk; i++) {
	  m_h_kt4jet_lt2d_d0[Nkt4jet-1] [i]    = infoLT2D->signedIP()[i];
	  m_h_kt4jet_lt2d_d0Sig[Nkt4jet-1] [i] = infoLT2D->significance()[i];
	}
      } else {
	m_h_kt4jet_lt2d_numTracks[Nkt4jet-1] = -999;
      }

      // Soft muon
      const Analysis::SoftMuonInfo* infoMu = (*kt4jetItr)->tagInfo<Analysis::SoftMuonInfo>("SoftMuonTag");
      if (infoMu) {
	int ntrk = infoMu->numTrackInfo();
	m_h_kt4jet_softmu_numTracks[Nkt4jet-1] = ntrk;
	
	if (ntrk > max_jettrack_num) ntrk = max_jettrack_num;
	for(int i = 0; i < ntrk; i++) {
	  Analysis::SMTrackInfo trackInfo = infoMu->getTrackInfo(i);
	  m_h_kt4jet_softmu_d0[Nkt4jet-1] [i] = trackInfo.d0Value();
	  m_h_kt4jet_softmu_ptrel[Nkt4jet-1] [i] = trackInfo.pTrel();
	  // what do about muon ptr?
	}
      } else {
	m_h_kt4jet_softmu_numTracks[Nkt4jet-1] = -999;
      }

      // Soft elec
      const Analysis::SoftElectronInfo* infoElec = (*kt4jetItr)->tagInfo<Analysis::SoftElectronInfo>("SoftElectronTag");
      if (infoElec) {
	int ntrk = infoElec->numTrackInfo();
	m_h_kt4jet_softelec_numTracks[Nkt4jet-1] = ntrk;
	
	if (ntrk > max_jettrack_num) ntrk = max_jettrack_num;
	for(int i = 0; i < ntrk; i++) {
	  Analysis::SETrackInfo trackInfo = infoElec->getTrackInfo(i);
	  m_h_kt4jet_softelec_d0[Nkt4jet-1] [i] = trackInfo.d0Value();
	  m_h_kt4jet_softelec_ptrel[Nkt4jet-1] [i] = trackInfo.pTrel();
	  // what do about electron ptr?
	}
      } else {
	m_h_kt4jet_softelec_numTracks[Nkt4jet-1] = -999;
      }

      // Jet prob info is now written to IPInfoPlus
    }
  }

  m_h_kt4jet_num = Nkt4jet < max_jet_num ? Nkt4jet : max_jet_num;
  //m_h_kt4jet_num = max_jet_num;
  //if(Nkt4jet < max_jet_num) m_h_kt4jet_num = Nkt4jet;
  mLog << MSG::INFO << "Kt Jets added to Ntuple. " << endreq;
  return StatusCode::SUCCESS;
}

StatusCode AODtoLiverpoolNtuple::UnpackTruthJet()
{
  MsgStream mLog(messageService(), name());
  StatusCode status = StatusCode::SUCCESS;

  if (m_fastsim == 1) {
    // for Atlfast data we are done now
    m_h_cone_true_num  = 0; 
    m_h_cone4_true_num = 0; 
    m_h_kt4jet_true_num = 0; 
    return StatusCode::SUCCESS;
  }    

  // Cone truth jets
  const ParticleJetContainer* truconTES;
  status = m_storeGate->retrieve( truconTES, "ConeTruthParticleJets");
  if (status.isFailure()  ||  !truconTES) {
    mLog << MSG::WARNING << "No AOD Truth ConeJet container found in TDS" << endreq;
    return StatusCode::SUCCESS;
  }

  mLog << MSG::DEBUG << " Truth ConeJets successfully retrieved" << endreq;

  ParticleJetContainer::const_iterator truconeItr  = (*truconTES).begin();
  ParticleJetContainer::const_iterator truconeItrE = (*truconTES).end();

  int Ntrucone = 0;
  for (; truconeItr != truconeItrE; ++truconeItr) {  
    Ntrucone++ ;
    if(Ntrucone < max_jet_num) {
      //m_h_cone_true_px[Ntrucone-1] = (*truconeItr)->px(); 
      //m_h_cone_true_py[Ntrucone-1] = (*truconeItr)->py(); 
      //m_h_cone_true_pz[Ntrucone-1] = (*truconeItr)->pz(); 
      m_h_cone_true_pt[Ntrucone-1] = (*truconeItr)->pt(); 
      m_h_cone_true_eta[Ntrucone-1]= (*truconeItr)->eta();
      m_h_cone_true_phi[Ntrucone-1]= (*truconeItr)->phi();
    }
  }

  m_h_cone_true_num = Ntrucone < max_jet_num ? Ntrucone : max_jet_num;
  mLog << MSG::INFO << "Truth Cone Jets added to Ntuple. " << endreq;

  const ParticleJetContainer* trucone4TES;  
  status = m_storeGate->retrieve( trucone4TES, "Cone4TruthParticleJets");
  if (status.isFailure()  ||  !trucone4TES) {
    mLog << MSG::WARNING << "No AOD Truth Cone4Jet container found in TDS" << endreq;
    return StatusCode::SUCCESS;
  }
  mLog << MSG::DEBUG << "Truth Cone4Jets successfully retrieved" << endreq;

  ParticleJetContainer::const_iterator trucone4Itr  = (*trucone4TES).begin();
  ParticleJetContainer::const_iterator trucone4ItrE = (*trucone4TES).end();

  // Cone 4 truth jets
  int Ntrucone4 = 0;
  for (; trucone4Itr != trucone4ItrE; ++trucone4Itr) {  
    Ntrucone4++ ;
    if(Ntrucone4 < max_jet_num) {
      //m_h_cone4_true_px[Ntrucone4-1] = (*trucone4Itr)->px(); 
      //m_h_cone4_true_py[Ntrucone4-1] = (*trucone4Itr)->py(); 
      //m_h_cone4_true_pz[Ntrucone4-1] = (*trucone4Itr)->pz(); 
      m_h_cone4_true_pt[Ntrucone4-1] = (*trucone4Itr)->pt(); 
      m_h_cone4_true_eta[Ntrucone4-1]= (*trucone4Itr)->eta();
      m_h_cone4_true_phi[Ntrucone4-1]= (*trucone4Itr)->phi();
    }
  }

  m_h_cone4_true_num = Ntrucone4 < max_jet_num ? Ntrucone4 : max_jet_num;
  mLog << MSG::INFO << "Truth Cone4 Jets added to Ntuple. " << endreq;

  // Kt truth jets
  const ParticleJetContainer* truktTES;
  if (m_release == 4) {
    status = m_storeGate->retrieve( truktTES, "Kt4TruthParticleJets");
    if (status.isFailure()  ||  !truktTES) {
      mLog << MSG::WARNING << "No AOD Truth Kt4jet container found in TDS" << endreq;
      return StatusCode::SUCCESS;
    }
  } else if (m_release == 31) {
    status = m_storeGate->retrieve( truktTES, "KtTruthParticleJets");
    if (status.isFailure()  ||  !truktTES) {
      mLog << MSG::WARNING << "No AOD Truth Kt4jet container found in TDS" << endreq;
      return StatusCode::SUCCESS;
    }
  }

  mLog << MSG::DEBUG << " Truth Kt4jets successfully retrieved" << endreq;

  ParticleJetContainer::const_iterator trukt4jetItr  = (*truktTES).begin();
  ParticleJetContainer::const_iterator trukt4jetItrE = (*truktTES).end();

  int Ntrukt4jet = 0;
  for (; trukt4jetItr != trukt4jetItrE; ++trukt4jetItr) {  
    Ntrukt4jet++ ;
    if(Ntrukt4jet < max_jet_num) {
      //m_h_kt4jet_true_px[Ntrukt4jet-1] = (*trukt4jetItr)->px(); 
      //m_h_kt4jet_true_py[Ntrukt4jet-1] = (*trukt4jetItr)->py(); 
      //m_h_kt4jet_true_pz[Ntrukt4jet-1] = (*trukt4jetItr)->pz(); 
      m_h_kt4jet_true_pt[Ntrukt4jet-1] = (*trukt4jetItr)->pt(); 
      m_h_kt4jet_true_eta[Ntrukt4jet-1]= (*trukt4jetItr)->eta();
      m_h_kt4jet_true_phi[Ntrukt4jet-1]= (*trukt4jetItr)->phi();
    }
  }

  m_h_kt4jet_true_num = Ntrukt4jet < max_jet_num ? Ntrukt4jet : max_jet_num;
  mLog << MSG::INFO << "Truth Kt4jet Jets added to Ntuple. " << endreq;

  mLog << MSG::INFO << "TruthKt Jets added to Ntuple. " << endreq;
  return StatusCode::SUCCESS;
}

StatusCode AODtoLiverpoolNtuple::UnpackPrimaryVertex()
{
  MsgStream mLog(messageService(), name());
  StatusCode status = StatusCode::SUCCESS;

  // for Atlfast data nothing to do here
  if (m_fastsim == 1) {
    m_h_primaryvtx_num = 0; // no vertex data in Altfast data ???
    return StatusCode::SUCCESS;
  }    

  const VxContainer* vtxTES;
  status = m_storeGate->retrieve(vtxTES, "VxPrimaryCandidate");
  if (status.isFailure()  ||  !vtxTES) {
    mLog << MSG::WARNING << "No AOD PrimVtx container found in TDS" << endreq;
    return StatusCode::SUCCESS;
  }

  mLog << MSG::DEBUG << "Prim Vtx successfully retrieved" << endreq;

  VxContainer::const_iterator vtxItr  = (*vtxTES).begin();
  VxContainer::const_iterator vtxItrE = (*vtxTES).end();

  // Primary vertex
  int Npvtx = 0;
  for (; vtxItr != vtxItrE; ++vtxItr) {
    Npvtx++ ;

    if (Npvtx < max_vertex_num) {
      const Trk::RecVertex& primaryVertex = (*vtxItr)->recVertex();
           
      m_h_primaryvtx_x[Npvtx-1]    = primaryVertex.position().x();
      m_h_primaryvtx_y[Npvtx-1]    = primaryVertex.position().y();
      m_h_primaryvtx_z[Npvtx-1]    = primaryVertex.position().z();
      m_h_primaryvtx_dX[Npvtx-1]   = -999;  // should get these from error matrix
      m_h_primaryvtx_dY[Npvtx-1]   = -999;
      m_h_primaryvtx_dZ[Npvtx-1]   = -999;
      m_h_primaryvtx_chi2[Npvtx-1] = primaryVertex.chi2();	
      m_h_primaryvtx_ndf[Npvtx-1]  = primaryVertex.ndf();  
    }
  }
   
  m_h_primaryvtx_num = Npvtx < max_vertex_num ? Npvtx : max_vertex_num;
  mLog << MSG::INFO << "Primary Vertices added to Ntuple. " << endreq;

  return StatusCode::SUCCESS;
}
